'use strict';
/**
 * @ngdoc controller
 * @name ChangeEmailController
 * @description
 *  email 변경 화면
 */
app.controller('ChangeEmailController', function ($scope, $state, $window, toastr, User, Auth) {
  $scope.save = function () {

    var password = $scope.formData.password;
    var newEmail = $scope.formData.newEmail;
    var jsonData = JSON.stringify( { password : password, email : newEmail });
    
    // 사용자 email 변경
    User.changeEmail(jsonData).then( function() { 
      var user = JSON.parse($window.localStorage.getItem('hcm_editor'));
      angular.extend(user, {'email': newEmail});
      Auth.setCurrentUser(user);
      toastr.success('Successfully changed email!');
      $state.transitionTo('account-settings.main', null, { reload: true, inherit: true, notify: true  });
    }).
    catch(function (error) {
      toastr.error(error);
    });
  };
});

/**
 * @ngdoc controller
 * @name ChangePasswordController
 * @description
 *  password 변경 화면
 */
app.controller('ChangePasswordController', function ($scope, $state, toastr, User) {
  $scope.save = function () {

    var oldPassData = $scope.formData.currentPassword;
    var newPassData = $scope.formData.newPassword;
    var jsonData = JSON.stringify( { old_password : oldPassData, new_password : newPassData });

    // 사용자 password 변경
    User.changePassword(jsonData).then(function (data) {
      toastr.success('Successfully changed password!');
      $state.transitionTo('account-settings.main', null, { reload: true, inherit: true, notify: true  });     
    }).
    catch(function (error) {
      toastr.error(error);
    });
  };
});



'use strict';

/**
 * @ngdoc controller
 * @name ActivitiesController
 * @description
 *  simple activity 설정 화면 (아래 controller 들의 state 를 포함하는 state 의 controller)
 */
app.controller('ActivitiesController', function ($scope, room, Devices) {
  // MXHOMEPRO-444 Issue 의 문제를 해결하기 위해 room 데이터를 갱신함
  $scope.room = room;

  $scope.simpleActivity = { 'currentDevice': null, 'devices': [] };
  $scope.simpleActivity.index = 0;

  // function 들 array 초기화
  Devices.initFunctions();

});

/**
 * @ngdoc controller
 * @name ChooseActivityDeviceController
 * @description
 *  based on device 선택 화면
 */
app.controller('ChooseActivityDeviceController', function ($scope, $state, Systems ) {

  $scope.doNext = function() {

    // 해당 deivce ID 와 연계된 button 의 데이터 얻어와서 저장
    Systems.updateCurrentMacroForDevice($scope.room.id, $scope.simpleActivity.currentDevice.id).then(
      function(data) {
        console.log(data);
        $state.go('menu.programming.activities.devices-to-use');
      },
      function (error) {
        alert(error);
      }
    );
  };
});

/**
 * @ngdoc controller
 * @name ActivityDevicesController
 * @description
 *  simple activity 선택 화면
 */
app.controller('ActivityDevicesController', function ($scope, $state, Systems, Devices, Auth) {
  var dataActivity = Systems.getCurrentMacro();
  $scope.simpleActivity.devices = [];
  console.log(dataActivity);

  var navigate = {};

  if (dataActivity) {
    _.each(dataActivity.activity, function(item) {
      if (item.type === 2 && item.navigate) {
        navigate[item.device_id] = true;
      }
    });
  }

  if(dataActivity && dataActivity.edit_type === 0 && dataActivity.activity.length ) {//simple activity
    _.forEach(dataActivity.activity, function(data) {
      var device = Systems.getDevice($scope.room.id, data.device_id);

      if (device) {
        var obj = {
          'device_id': data.device_id,
          'name': device.name,
          'type': data.type,
          'func': data.func,
          'repeat_min': device.repeat_min,
          'repeat_macro': device.repeat_macro,
          'target_type': device.target_type,
          'target_addr': device.target_addr,
          'target_port': device.target_port
        };
        if (navigate[data.device_id]) {
          obj.navigate = 1;
        }
        $scope.simpleActivity.devices.push(obj);
      }
    });
  }
  else {
    _.forEach($scope.room.devices, function(device) {
      if ($scope.simpleActivity.currentDevice.id === device.id && _.isEmpty($scope.simpleActivity.devices)) {
        var obj = {
          'device_id': device.id,
          'name': device.name,
          'type': 1,
          'func': null,
          'repeat_min': device.repeat_min,
          'repeat_macro': device.repeat_macro,
          'target_type': device.target_type,
          'target_addr': device.target_addr,
          'target_port': device.target_port
        };
        if (navigate[device.id]) {
          obj.navigate = 1;
        }
        $scope.simpleActivity.devices.push(obj);
      }
    });
  }

  $scope.goBack = function() {
    $scope.simpleActivity.devices = [];
    Devices.initFunctions();
    $state.go('menu.programming.activities.choose-device' );
  };

  $scope.doNext = function() {
    $state.go('menu.programming.activities.commands');
  };

  Auth.currentUser().then(function(user) {
    $scope.optMenuShow = user.optMenuShow;
  });
  
  $scope.goAdvanced = function() {
    $state.go('menu.advanced.add-macros.macro-type');
  };

});

/**
 * @ngdoc controller
 * @name ActivityDeviceCommandsController
 * @description
 *  command 선택 화면
 */
app.controller('ActivityDeviceCommandsController', function ($scope, $state, Systems) {

  $scope.goBack = function() {
    if ($scope.simpleActivity.index === 0) {
      $state.go('menu.programming.activities.devices-to-use');
    } else {
      $scope.simpleActivity.index--;
    }
  };

  $scope.incrIdx = function() { $scope.simpleActivity.index++; };


  $scope.finish = function() {
    //Save to server
    var sendData = angular.copy($scope.simpleActivity.devices);

    for (var i=0,  tot=sendData.length; i < tot; i++) {
      sendData[i].func = sendData[i].func.id;
      delete sendData[i].name;
      delete sendData[i].repeat_min;
      delete sendData[i].repeat_macro;
      delete sendData[i].target_type;
      delete sendData[i].target_addr;
      delete sendData[i].target_port;
    }

    var jsonData = JSON.stringify( { auto_wait : 1, edit_type : 0, popup : 1, activity : sendData, macro : []} );
    console.log(jsonData);

    // 해당 device ID 와 연계된 button 의 데이터 저장
    Systems.storeDeviceButtonMacro($scope.room.id, $scope.simpleActivity.currentDevice.id, jsonData).then(function(response) {
      if(response.data.result === true) {
        $state.go('menu.programming.options');
      }
      else {
        alert('Your edit/create action failed.  Please try again.');
      }
    }, function(error){
      alert("Please try again");
    });

  };

});

'use strict';
/**
 * @ngdoc controller
 * @name AdvancedDeviceTypeController
 * @description
 *  device type 선택
 */
app.controller('AdvancedDeviceTypeController', function ($rootScope, $scope, $state, toastr, categories, Systems) {
  var currentSystem = Systems.getCurrentSystem(),
    twowayDevicesCount = 0;

  $scope.twowayDevicesLimit = $rootScope.env.twowayDevicesLimit;

  $scope.allowTwowayDevices = true;
  _.each(currentSystem.rooms, function(room) {
    twowayDevicesCount += room.twoway_devices.length || 0;
  });

  // Advanced Device 는 최대 $scope.twowayDevicesLimit 개까지만 추가할 수 있음
  if (twowayDevicesCount >= $scope.twowayDevicesLimit) {
    $scope.allowTwowayDevices = false;
    toastr.error('There are already ' + $scope.twowayDevicesLimit + ' Two Way Devices in the system.  You must delete one of them before adding another. Go to the Home Overview to delete devices.');
  }

  if (!$scope.device.type) {
    $scope.device.type = categories[0];
  }

  $scope.types = categories;

  // 브랜드 네임과 디바이스 네임을 조합한다.
  $scope.getCombolabel = function(item){
    if(item.brand_name){
      if(item.name === 'TRF-ZW1'){
        return item.brand_name + ' ' + 'TRF-ZW';
      } else if(item.brand_name === item.name || 0 <= item.name.indexOf(item.brand_name)) {
        return item.name;
      } else {
        return item.brand_name + ' ' + item.name;
      }
    } else {
      return item.name;
    }
  }

  // Two Way Device Type 을 선택할 때 More... 가 나올지 여부 설정
  if ($rootScope.env.advancedDeviceMore) {
    if (!_.findWhere($scope.types, { name: 'More...' })) {
      $scope.types.push({
        name: 'More...'
      });
    }

    // More... 를 선택한 경우 선택가능한 device type 을 Systems.getTwowayCategories 를 통해 다시 불러옴
    $scope.$watch('device.type', function(newVal, oldVal) {
      if (newVal.name === 'More...') {
        $state.go('menu.programming.add-advanced-device.more-device-category');
      }
    });
  }

  $scope.doNext = function () {
    // System에 걸쳐 TRF-ZW1은 한개
    // 이미 등록된 TRF-ZW1 이 있으면 진행하지 않음
    var trfzw1Exist = false;

    _.each(currentSystem.rooms, function(room) {
      _.each(room.twoway_devices, function(twoway_device) {
        if (twoway_device.model === 'TRF-ZW1') {
          trfzw1Exist = true;
        }
      });
    });

    // 이미 등록된 같은 model id 를 가진 core device 가 있는지 확인
    var coreDeviceExist = false;

    _.each(currentSystem.rooms, function (room) {
      _.each(room.twoway_devices, function (twoway_device) {
        if (twoway_device.model_id === $scope.device.type.id) {
          coreDeviceExist = true;
        }
      });
    });

    if (trfzw1Exist && $scope.device.type.name === 'TRF-ZW1') {  // system 에 TRF-ZW1 이 존재하는데 TRF-ZW1 을 추가하려고 선택한 경우
      toastr.error('There is already a TRF-ZW in the system.');
    } else {
      // $scope.device.type.device_type
      // 0: Normal
      // 1: Dynamic-Core
      // 2: Dynamic-Interface

      // core device 를 추가하는데 system 에 이미 등록된 core device 가 있는 경우
      if ($scope.device.type.device_type === 1 && coreDeviceExist) {
        $state.go('menu.programming.add-advanced-device.choose-core-device');
      } else {
        // variables 가 있는 경우 또는 core device 인 경우
        if (($scope.device.type.variables && $scope.device.type.variables.length) || $scope.device.type.device_type === 1) {
          $scope.device.model = angular.copy($scope.device.type);
          $state.go('menu.programming.add-advanced-device.more-device-name');
        } else {
          $state.go('menu.programming.add-advanced-device.device-name');
        }
      }
    }
  };
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceCategoryController
 * @description
 *  more device category 선택
 */
app.controller('MoreAdvancedDeviceCategoryController', function ($scope, $state, toastr, moreCategories, Systems) {
  delete($scope.device.type);

  var currentSystem = Systems.getCurrentSystem();

  if (!$scope.device.category) {
    $scope.device.category = {id: moreCategories[0].id, name: moreCategories[0].name};  // 기본값 설정
  }
  $scope.categories = moreCategories;

  $scope.$watch('device.category', function(newVal, oldVal) {
    if (newVal && oldVal && newVal.id !== oldVal.id) {
      $scope.device.brand = null;
      $scope.device.model = null;
    }
  });

  $scope.doNext = function () {
    // System에 걸쳐 TRF-ZW1은 한개
    // 이미 등록된 TRF-ZW1 이 있으면 진행하지 않음
    var trfzw1Exist = false;

    _.each(currentSystem.rooms, function(room) {
      _.each(room.twoway_devices, function(twoway_device) {
        if (twoway_device.model === 'TRF-ZW1') {
          trfzw1Exist = true;
        }
      });
    });

    if (trfzw1Exist && $scope.device.category.name === 'TRF-ZW1') {  // system 에 TRF-ZW1 이 존재하는데 TRF-ZW1 을 추가하려고 선택한 경우
      toastr.error('There is already a TRF-ZW in the system.');
    } else {
      $state.go('menu.programming.add-advanced-device.more-device-name');
    }
  };
});

/**
 * @ngdoc controller
 * @name ChooseCoreAdvancedDeviceController
 * @description
 *  Core device 선택
 */
app.controller('ChooseCoreAdvancedDeviceController', function ($rootScope, $scope, $state, Systems) {
  // core device 와 interface device 는 device.type.variables 가 있다고 가정함
  $scope.device.model = angular.copy($scope.device.type);

  $scope.device.model.coreDevices = [{  // core device 생성 radio button 을 위한 dummy object
    id: 'createNewCoreDevice',
    name: 'Create a new core device'
  }];

  $scope.coreDevice = $scope.device.model.coreDevices[0];

  var currentSystem = Systems.getCurrentSystem();

  _.each(currentSystem.rooms, function (room) {
    _.each(room.twoway_devices, function (twoway_device) {
      if (twoway_device.device_type === 1 && twoway_device.model_id === $scope.device.model.id) {
        $scope.device.model.coreDevices.push(twoway_device);
      }
    });
  });

  $scope.doNext = function () {
    // interface device 추가
    if ($scope.coreDevice.id !== 'createNewCoreDevice') {
      var roomId = null;

      _.each(currentSystem.rooms, function (room) {
        _.each(room.twoway_devices, function (twoway_device) {
          if (twoway_device.id === $scope.coreDevice.id) {
            roomId = room.id;
          }
        });
      });

      $scope.device.model.device_type = 2;
      $scope.device.model.link_room_id = roomId;
      $scope.device.model.link_dev_id = $scope.coreDevice.id;
      $scope.device.model.coreDevice = angular.copy($scope.coreDevice);
    }

    $state.go('menu.programming.add-advanced-device.more-device-name');
  };
});

/**
 * @ngdoc controller
 * @name AdvancedDeviceNameController
 * @description
 *  device name 입력
 */
app.controller('AdvancedDeviceNameController', function ($scope, $state, $modal, toastr, Database, Rooms, Devices, Systems) {
  $scope.device.name = $scope.device.type.name === 'TRF-ZW1' ? 'TRF-ZW' : angular.copy($scope.device.type.name);

  // MAC
  if ($scope.device.type.target_type & 0x0001) {
    // discovery
    if ($scope.device.type.target_type & 0x00010000) {
      $scope.device.addType = 'discovery';
    }
    // mac
    else {
      $scope.device.addType = 'mac';
    }
  } else {
    // IP
    if ($scope.device.type.target_type & 0x0002) {
      $scope.device.addType = 'ip';
    }
    // service
    else if ($scope.device.type.target_type & 0x0004) {
      $scope.device.addType = 'service';
    }
  }

  // 같은 이름의 device name 이 이미 등록되어 있음을 알리는 modal
  var modal = $modal({
    scope: $scope,
    title: 'Duplicate device name',
    container: '.editor-container',
    template: 'views/modal-tpls/twoway-duplicate-name.html',
    show: false
  });

  $scope.doNext = function() {
    var deviceNames = _.pluck(Rooms.getTwowayDevices($scope.room.id), 'name');
    if (_.contains(deviceNames, $scope.device.name)) {
      modal.$promise.then(modal.show);
    } else {
      $scope.doNext2();
    }
  };

  $scope.doNext2 = function() {
    if ($scope.device.addType === 'service') {
      $state.go('menu.programming.add-advanced-device.device-review', {'addType': 'service'});
    } else {
      $state.go('menu.programming.add-advanced-device.device-identification', {'addType': $scope.device.addType, 'model': $scope.device.type.name});
    }
  };
});

/**
 * @ngdoc controller
 * @name AdvancedDeviceIdentificationController
 * @description
 *  device 추가
 */
app.controller('AdvancedDeviceIdentificationController', function ($rootScope, $scope, $state, $modal, toastr, devices, Database, Rooms, Devices, Systems) {
  $scope.trfZwLink = $rootScope.env.trfZwLink;

  if ($scope.device.addType === 'discovery') {
    $scope.twowayDevices = devices.searchedDevices || [];
  }

  $scope.device.key = {};
  $scope.device.ip = {};

  // discovery
  if ($scope.device.addType === 'discovery') {
    $scope.$watch('device.mac', function() {
      $scope.device.targetAddr = $scope.device.mac;
    });
  }
  // mac
  else if ($scope.device.addType === 'mac') {
    $scope.$watch('device.key', function() {
      $scope.device.targetAddr = $scope.device.key.mac1 + $scope.device.key.mac2 + $scope.device.key.mac3 + $scope.device.key.mac4 + $scope.device.key.mac5 + $scope.device.key.mac6;
      $scope.device.targetAddr = $scope.device.targetAddr.toUpperCase();
    }, true);
  }
  // IP
  else if ($scope.device.addType === 'ip') {
    $scope.$watch('device.ip', function() {
      $scope.device.targetAddr = $scope.device.ip.address + ':' + $scope.device.ip.port;
    }, true);
  }

  $scope.doNext = function() {
    if ($scope.device.addType === 'mac') {
      $state.go('menu.programming.add-advanced-device.device-review', {'addType': 'mac'});
    } else {
      if (!$scope.device.model) {
        $scope.device.model = {};
      }

      var d = {
          name: $scope.device.name,
          model: $scope.device.type.name,
          modelId: $scope.device.type.id,
        categoryId: $scope.device.type.category_id,
          targetType: $scope.device.type.target_type,
        targetAddr: $scope.device.targetAddr || '""',
        deviceType: $scope.device.model.device_type || 0,
        variables: '""',
        vol_cmd: 0
      };

      if ($scope.device.model.link_room_id) {
        d.linkRoomId = $scope.device.model.link_room_id;
      }
      if ($scope.device.model.link_dev_id) {
        d.linkDevId = $scope.device.model.link_dev_id;
      }

      if (typeof $scope.device.type.cmds != 'undefined' && $scope.device.type.cmds.length) {
        if ($scope.device.type.cmds.indexOf('275') > -1 || $scope.device.type.cmds.indexOf('276') > -1 || $scope.device.type.cmds.indexOf('277') > -1) {
          d.vol_cmd = 1;
        }
      }
      var jsonData = JSON.stringify(d);

      // device 생성
      return Devices.createTwowayDevice($scope.room.id, jsonData)
        .then(function(data) {
          console.log(data);

          Systems.updateSystem(Systems.getCurrentSystem().id).then(function() {
            if ($scope.trfZwLink && $scope.device.type.name === 'TRF-ZW1') {
              $state.go('menu.programming.add-advanced-device.trf-zw-setup');
            } else {
              $state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true  });
            }
          }).catch(function(error) {
            toastr.error(error);
          });
        })
        .catch(toastr.error);
    }
  };

  $scope.getTwowayDevices = function(type) {
    Systems.getTwowayDevices(type).then(
      function(data) {
        $scope.twowayDevices = data.searchedDevices || [];
      },
      function(error) {
        toastr.error(error, 'Error');
      }
    );
  };
});

/**
 * @ngdoc controller
 * @name AdvancedDeviceReviewController
 * @description
 *  device review
 */
app.controller('AdvancedDeviceReviewController', function ($scope, $state, $stateParams, $modal, toastr, Devices, Systems) {
  var d, jsonData;

  $scope.doNext = function() {
    if ($stateParams.addType === 'service') {
      d = {
        name: $scope.device.name,
        model: $scope.device.type.name,
        modelId: $scope.device.type.id,
        categoryId: $scope.device.type.category_id,
        targetType: $scope.device.type.target_type,
        targetAddr: '""',
        variables: '""',
        deviceType: $scope.device.type.device_type,
        vol_cmd: 0
      };

      if ($scope.device.model && $scope.device.model.link_room_id) {
        d.linkRoomId = $scope.device.model.link_room_id;
      }
      if ($scope.device.model && $scope.device.model.link_dev_id) {
        d.linkDevId = $scope.device.model.link_dev_id;
      }

      if (typeof $scope.device.type.cmds != 'undefined' && $scope.device.type.cmds.length) {
        if ($scope.device.type.cmds.indexOf('275') > -1 || $scope.device.type.cmds.indexOf('276') > -1 || $scope.device.type.cmds.indexOf('277') > -1) {
          d.vol_cmd = 1;
        }
      }
      jsonData = JSON.stringify(d);

      // device 생성
      return Devices.createTwowayDevice($scope.room.id, jsonData)
        .then(function(data) {
          console.log(data);

          Systems.updateSystem(Systems.getCurrentSystem().id).then(function() {
            $state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true  });
          }).catch(function(error) {
            toastr.error(error);
          });
        })
        .catch(toastr.error);
    } else if ($stateParams.addType === 'mac') {
      d = {
        name: $scope.device.name,
        model: $scope.device.type.name,
        modelId: $scope.device.type.id,
        categoryId: $scope.device.type.category_id,
        targetType: $scope.device.type.target_type,
        targetAddr: $scope.device.targetAddr || '""',
        variables: '""',
        deviceType: $scope.device.type.device_type,
        vol_cmd: 0
      };

      if ($scope.device.model && $scope.device.model.link_room_id) {
        d.linkRoomId = $scope.device.model.link_room_id;
      }
      if ($scope.device.model && $scope.device.model.link_dev_id) {
        d.linkDevId = $scope.device.model.link_dev_id;
      }

      if (typeof $scope.device.type.cmds != 'undefined' && $scope.device.type.cmds.length) {
        if ($scope.device.type.cmds.indexOf('275') > -1 || $scope.device.type.cmds.indexOf('276') > -1 || $scope.device.type.cmds.indexOf('277') > -1) {
          d.vol_cmd = 1;
        }
      }
      jsonData = JSON.stringify(d);

      // device 생성
      return Devices.createTwowayDevice($scope.room.id, jsonData)
        .then(function(data) {
          console.log(data);

          Systems.updateSystem(Systems.getCurrentSystem().id).then(function() {
            $state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true  });
          }).catch(function(error) {
            toastr.error(error);
          });
        })
        .catch(toastr.error);
    }
  };
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceNameController
 * @description
 *  more device name 입력
 */
app.controller('MoreAdvancedDeviceNameController', function ($scope, $state, $modal, toastr, Database, Rooms, Devices, Systems) {
  $scope.device.name = '';    // name 초기화
  $scope.device.allVariables = [];

  // 같은 이름의 device name 이 이미 등록되어 있음을 알리는 modal
  var modal = $modal({
    scope: $scope,
    title: 'Duplicate device name',
    container: '.editor-container',
    template: 'views/modal-tpls/twoway-duplicate-name.html',
    show: false
  });

  $scope.doNext = function() {
    var deviceNames = _.pluck(Rooms.getTwowayDevices($scope.room.id), 'name');
    if (_.contains(deviceNames, $scope.device.name)) {
      // spec 문서의 내용에 따라, Core device 를 추가하다가 이 상황이 발생하였을 때에는 modal 대신 toastr 을 사용함
      if ($scope.device.model.device_type === 1) {
        toastr.error('This Core Device name is already in use. Please enter a unique name for this Core Device.');
      } else {
        modal.$promise.then(modal.show);
      }
    } else {
      $scope.doNext2();
    }
  };

  $scope.doNext2 = function() {
    // Interface device 인 경우에는 variables 대신 interface_variables 를 사용함
    if ($scope.device.model.device_type === 2) {
      $scope.device.model.variables = $scope.device.model.interface_variables;
    }

    if ($scope.device.category && $scope.device.category.id) {
      $state.go('menu.programming.add-advanced-device.more-device-brand', {'categoryId': $scope.device.category.id});
    }
    // Interface device 이면서 variables 가 없는 경우 device review 화면으로 이동
    else if ($scope.device.model.device_type === 2 && (!$scope.device.model.variables || !$scope.device.model.variables.length)) {
      $state.go('menu.programming.add-advanced-device.more-device-review');
    }
    else {
      // variables 입력 - mac, ip 도 입력해야 할 수 있음
      if ($scope.device.model.variables && $scope.device.model.variables.length) {
        $scope.device.addType = 'variables';
        $state.go('menu.programming.add-advanced-device.more-device-identification');
      }
      // mac 입력
      else if ($scope.device.model.target_type === 1) {
        $scope.device.addType = 'mac';
        $state.go('menu.programming.add-advanced-device.more-device-identification');
      }
      // IP 입력
      else if ($scope.device.model.target_type === 2) {
        $scope.device.addType = 'ip';
        $state.go('menu.programming.add-advanced-device.more-device-identification');
      }
      // device review 화면으로 이동
      else {
        $state.go('menu.programming.add-advanced-device.more-device-review');
      }
    }
  };
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceBrandController
 * @description
 *  more device brand 선택
 */
app.controller('MoreAdvancedDeviceBrandController', function ($scope, $state, $modal, toastr, brands) {
  $scope.brands = brands;

  $scope.$watch('device.brand', function(newVal, oldVal) {
    if (newVal && oldVal && newVal.id !== oldVal.id) {
      $scope.device.model = null;
    }
  });

  $scope.doNext = function() {
    $state.go('menu.programming.add-advanced-device.more-device-model', {'brandId': $scope.device.brand.id});
  };
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceModelController
 * @description
 *  more device model 선택
 */
app.controller('MoreAdvancedDeviceModelController', function ($scope, $state, $modal, toastr, models) {
  $scope.models = models;

  $scope.doNext = function() {
    // variables 입력 - mac, ip 도 입력해야 할 수 있음
    if ($scope.device.model.variables && $scope.device.model.variables.length) {
      $scope.device.addType = 'variables';
      $state.go('menu.programming.add-advanced-device.more-device-identification');
    }
    // mac 입력
    else if ($scope.device.model.target_type === 1) {
      $scope.device.addType = 'mac';
      $state.go('menu.programming.add-advanced-device.more-device-identification');
    }
    // IP 입력
    else if ($scope.device.model.target_type === 2) {
      $scope.device.addType = 'ip';
      $state.go('menu.programming.add-advanced-device.more-device-identification');
    }
    // device review 화면으로 이동
    else {
      $state.go('menu.programming.add-advanced-device.more-device-review');
    }
  };
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceIdentificationController
 * @description
 *  more device 정보 설정
 */
app.controller('MoreAdvancedDeviceIdentificationController', function ($scope, $state, $modal, toastr, Database, Rooms, Devices, Systems) {
  $scope.device.key = {};
  $scope.device.ip = {};

  // mac or IP
  if ($scope.device.addType === 'mac' || $scope.device.addType === 'ip') {
    $scope.doNext = function() {
      $state.go('menu.programming.add-advanced-device.more-device-review');
    };
  }
  // variables
  else if ($scope.device.addType === 'variables') {
    $scope.device.varData = {};
    $scope.device.varUsername = false;
    $scope.device.varPassword = false;
    $scope.device.varPort = false;
    $scope.device.variables = [];

    _.each($scope.device.model.variables, function(item) {
      if (item.name === 'username') {
        $scope.device.varUsername = true;
      }
      if (item.name === 'password') {
        $scope.device.varPassword = true;
      }
      if (item.name === 'device_port') {
        $scope.device.varPort = true;

        if (item['default']) {
          $scope.device.ip.port = +item['default'];
        }
      }

      $scope.device.varData[item.name] = item;
      if (item['default']) {
        $scope.device.varData[item.name].value = item['default'];
      }

      if (item.attr.match('n')) {
        item.required = true;
      } else {
        item.required = false;
      }

      if (item.attr.match('p')) {
        item.type = 'password';
      } else if (item.attr.match('m')) {
        item.type = 'textarea';
      } else {
        item.type = 'text';
      }

      if (item.attr.match('r')) {
        item.readonly = true;
      } else {
        item.readonly = false;
      }

      if (item.name === 'device_port') {
        $scope.device.varPortReadonly = item.readonly;
      }

      $scope.device.variables.push(item);
    });

    var upRestLength = $scope.device.varPort ? 3 : 2;   // device_port 가 있는 경우에는 username, password, device_port 까지 3개를 빼고 계산해야 함

    // username/password 가 있는지, 나머지 variables 가 있는지 여부에 따라 페이지 구성이 달라짐
    // 1. username/password 가 있으면서 나머지 variables 도 있는 경우
    if ($scope.device.varUsername && $scope.device.varPassword && $scope.device.variables.length > upRestLength) {
      $scope.device.varType = 'UPV';    // username, password, others
    }
    // 2. username/password 만 있는 경우
    else if ($scope.device.varUsername && $scope.device.varPassword && $scope.device.variables.length === upRestLength) {
      $scope.device.varType = 'UP';     // username, password
    }
    // 3. username/password 가 없으면서 나머지 variables 가 있는 경우
    else if (($scope.device.varPort && $scope.device.variables.length > 1) || (!$scope.device.varPort && $scope.device.variables.length > 0)) {
      $scope.device.varType = 'V';      // 나머지 variables
    }
    // 4. IP/Port 입력만 필요한 경우
    else if ($scope.device.varPort && $scope.device.variables.length === 1) {
      $scope.device.varType = 'IP';
    }

    // mac/ip 입력하는 화면, username/password 입력하는 화면, 나머지 variables 입력하는 화면
    if (($scope.device.model.target_type === 1 || $scope.device.model.target_type === 2) && $scope.device.model.device_type !== 2) {    // Interface device 는 skip
      $scope.device.varStep = 'mac_ip';
    } else if ($scope.device.varType === 'UPV' || $scope.device.varType === 'UP') {
      $scope.device.varStep = 'UP';
    } else {
      $scope.device.varStep = 'V';
    }

    $scope.doNext = function() {
      if ($scope.device.varStep === 'mac_ip') {
        if ($scope.device.varUsername && $scope.device.varPassword) {
          $scope.device.varStep = 'UP';
        } else if ($scope.device.varType === 'IP') {
          // review 페이지로 이동
            // json data 생성
            $scope.device.allVariables = [];

            _.each($scope.device.variables, function(item) {
              $scope.device.allVariables.push(item);
            });

            $state.go('menu.programming.add-advanced-device.more-device-review');
        } else {
          $scope.device.varStep = 'V';
        }
      }
      else if ($scope.device.varStep === 'UP' && $scope.device.varType === 'UPV') {
        $scope.device.varStep = 'V';
      }
      // review 페이지로 이동
      else {
        // json data 생성
        $scope.device.allVariables = [];

        _.each($scope.device.variables, function(item) {
          $scope.device.allVariables.push(item);
        });

        // OneWay UI with Fav Device 인 경우에는 favorite 설정 page로 이동함.
        if ($scope.device.model.device_type === 4) {
          $state.go('menu.programming.add-advanced-device.more-device-favchannels-option');
        } else {
          $state.go('menu.programming.add-advanced-device.more-device-review');
        }
      }
    };
  }
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceFavChannelsOptionController
 * @description
 *  favorite channels 설정
 */
app.controller('MoreAdvancedDeviceFavChannelsOptionController', function ($scope, $state, $modal, toastr, Systems, Devices) {

  console.log('MoreAdvancedDeviceFavChannelsOptionController');
	$scope.device.favEnable = $scope.device.favEnable || 1;
	$scope.device.favDelay = $scope.device.favDelay || 400;
	$scope.device.favEnter = $scope.device.favEnter || 1;

	$scope.doNext = function () {
    console.log('doNext');
    $state.go('menu.programming.add-advanced-device.more-device-review');
  };
});

/**
 * @ngdoc controller
 * @name MoreAdvancedDeviceReviewController
 * @description
 *  more device review
 */
app.controller('MoreAdvancedDeviceReviewController', function ($scope, $state, $modal, toastr, Devices, Systems) {
  if (!$scope.device.model) {
    $scope.device.model = {};
  }

  var d = {
      name: $scope.device.name,
      model: $scope.device.model.name,
      modelId: $scope.device.model.id,
      categoryId: $scope.device.model.category_id,
      targetType: $scope.device.model.target_type,
      targetAddr: '""',
      variables: '""',
      deviceType: $scope.device.model.device_type || 0,
      vol_cmd: 0
    },
    jsonData;

  if ($scope.device.model.link_room_id) {
    d.linkRoomId = $scope.device.model.link_room_id;
  }
  if ($scope.device.model.link_dev_id) {
    d.linkDevId = $scope.device.model.link_dev_id;
  }

  if (typeof $scope.device.model.cmds != 'undefined' && $scope.device.model.cmds.length) {
    if ($scope.device.model.cmds.indexOf('275') > -1 || $scope.device.model.cmds.indexOf('276') > -1 || $scope.device.model.cmds.indexOf('277') > -1) {
      d.vol_cmd = 1;
    }
  }

  // mac
  if ($scope.device.addType === 'mac') {
    d.targetAddr = $scope.device.key.mac1 + $scope.device.key.mac2 + $scope.device.key.mac3 + $scope.device.key.mac4 + $scope.device.key.mac5 + $scope.device.key.mac6;
    d.targetAddr = d.targetAddr.toUpperCase();
  }
  // IP
  else if ($scope.device.addType === 'ip') {
    d.targetAddr = $scope.device.ip.address + ':' + $scope.device.ip.port;
  }
  // variables
  else if ($scope.device.addType === 'variables') {
    if ($scope.device.model.target_type === 1) {
      d.targetAddr = $scope.device.key.mac1 + $scope.device.key.mac2 + $scope.device.key.mac3 + $scope.device.key.mac4 + $scope.device.key.mac5 + $scope.device.key.mac6;
    } else if ($scope.device.model.target_type === 2) {
      d.targetAddr = $scope.device.ip.address + ':' + $scope.device.ip.port;
    }

    d.variables = [];
    _.each($scope.device.allVariables, function(item) {
      var obj = angular.copy(item);
      delete(obj['$$hashKey']);
      delete(obj.required);
      delete(obj.readonly);
      delete(obj.type);

      d.variables.push(obj);
    });
  }

  // Interface device 인 경우
  if ($scope.device.model.device_type === 2) {
    d.targetAddr = '""';
  }

  // OneWay UI or OneWay UI with Fav Device 인 경우에 정보 추가.
  if ($scope.device.model.device_type === 3 || $scope.device.model.device_type === 4) {
    d.targetAddr = $scope.device.ip.address;
    d.targetPort = $scope.device.ip.port;
    d.favEnable = 0;
    d.favDelay = 0;
    d.favEnter = 0;

    // OneWay UI with Fav Device 인 경우에 favorite 정보 추가.
    if ($scope.device.model.device_type === 4) {
      d.favEnable = $scope.device.favEnable;
      d.favDelay = $scope.device.favDelay;
      d.favEnter = $scope.device.favEnter;
    }
  }

  jsonData = JSON.stringify(d);
  console.log('jsonData=', jsonData);

  // device 생성
  $scope.doNext = function() {
    if ($scope.device.model.device_type === 3 || $scope.device.model.device_type === 4) { 

      return Devices.createTwowayDevice_OnewayUI($scope.room.id, jsonData)
        .then(function(data) {
          console.log(data);

          Systems.updateSystem(Systems.getCurrentSystem().id).then(function() {
            $state.transitionTo('menu.programming.options', null, {reload: true, inherit: true, notify: true});
          }).catch(function(error) {
            toastr.error(error);
          });
        })
        .catch(toastr.error);

    } else {

      return Devices.createTwowayDevice($scope.room.id, jsonData)
        .then(function(data) {
          console.log(data);

          Systems.updateSystem(Systems.getCurrentSystem().id).then(function() {
            // Core device 를 add 한 경우
            if ($scope.device.model.device_type === 1) {
              $state.go('menu.programming.add-advanced-device.core-device-added');
            } else {
              $state.transitionTo('menu.programming.options', null, {reload: true, inherit: true, notify: true});
            }
          }).catch(function(error) {
            toastr.error(error);
          });
        })
        .catch(toastr.error);
        
    }
  };
});

/**
 * @ngdoc controller
 * @name AdvancedDeviceTrfZwSetupController
 * @description
 *  TRF-ZW setup 안내 페이지
 */
app.controller('AdvancedDeviceTrfZwSetupController', function ($rootScope, $scope, $state) {
  $scope.urcControlRoomLink = $rootScope.env.urcControlRoomLink;
  $scope.urcControlRoomUrl = $rootScope.env.urcControlRoomUrl;

  $scope.finish = function() {
    $state.go('menu.programming.options');
  };
});

/**
 * @ngdoc controller
 * @name CoreAdvancedDeviceAddedController
 * @description
 *  Core device 추가 완료
 */
app.controller('CoreAdvancedDeviceAddedController', function ($rootScope, $scope, $state) {
  $scope.coreDeviceAddedNextStep = 'interfaceDevice';

  $scope.doNext = function () {
    if ($scope.coreDeviceAddedNextStep === 'interfaceDevice') {
      $state.go('menu.programming.add-advanced-device.choose-core-device');
    } else {
      $state.go('menu.programming.options');
    }
  };
});


'use strict';
/**
 * @ngdoc controller
 * @name SelectBaseController
 * @description
 *  base station 목록 표시 및 추가
 */
app.controller('SelectBaseController', function ($rootScope, $scope, $state, toastr, searchResults, Systems, Rooms ) {
  $scope.searchKey = searchResults.searchKey;
  $scope.baseStations = searchResults.searchedDevices;

  $scope.bsType = 'BASESTATIONS';

  $scope.changedValue = function() {
    if (!$scope.addBStationForm.displayText.$dirty) {
      var bs = $scope.addBaseStation.selection;
      $scope.addBaseStation.displayText =  bs.name + ' ' + bs.mac.slice(10,12);
    }
  };

  $scope.finish = function () {
    var currentSystem = Systems.getCurrentSystem(),
            givenName = $scope.addBaseStation.displayText,
          baseStation = $scope.addBaseStation.selection,
             jsonData = JSON.stringify({ mac : baseStation.mac, type : baseStation.name, name: givenName, lightLevel: 2, searchKey: $scope.searchKey });

    // slave base station 을 room 에 추가
    var addBStation = function(roomId, json) {
      return Rooms.baseStations('add', roomId, json).then(function(data) { 
        toastr.success('Successfully added ' + givenName + '.');
        return Systems.getCurrentSystem().id;
      });
    };

    var updateSystem = function(id) {
      return Systems.updateSystem(id).then(function (data) {
        console.log(data);
        return $state.transitionTo('menu.programming.options', {}, { reload: true, inherit: true, notify: true  });
        //$state.go('menu.programming.option');
      });
    };

    addBStation($scope.room.id, jsonData)
      .then(updateSystem)
      .catch(function(error) {
        toastr.error(error, 'Error');
      });

  };
});
'use strict';
/**
 * @ngdoc controller
 * @name DeviceTypeController
 * @description
 *  device type 선택
 */
app.controller('DeviceTypeController', function ($scope, $state, categories) {
	if (!$scope.device.type) {
		$scope.device.type = { id: categories[0].id, name: categories[0].name, device: categories[0].device };  // 기본값 설정
	}
	$scope.types = categories;

	$scope.doNext = function () {
		$state.go('menu.programming.add-device.device-name');
	};
});

/**
 * @ngdoc controller
 * @name DeviceNameController
 * @description
 *  device name 입력
 */
app.controller('DeviceNameController', function ($scope, $state, $modal, toastr, Database, Rooms) {
	$scope.device.name = angular.copy($scope.device.type.device);

	var getDeviceBrands = function () {
		Database.retrieveDeviceBrands($scope.device.type.name).then(function (data) {
			$state.go('menu.programming.add-device.device-brand');
		}).catch(toastr.error);
	};

	$scope.getBrandsFromDB = getDeviceBrands;

	// 같은 이름의 device name 이 이미 등록되어 있음을 알리는 modal
	var modal = $modal({
		scope: $scope,
		title: 'Duplicate device name',
		container: '.editor-container',
		template: 'views/modal-tpls/duplicate-name.html',
		show: false
	});

	$scope.doNext = function () {
		var deviceNames = _.pluck(Rooms.getDevices($scope.room.id), 'name');
		if (_.contains(deviceNames, $scope.device.name)) {
			modal.$promise.then(modal.show);
		} else {
			getDeviceBrands();
		}
	};
});

/**
 * @ngdoc controller
 * @name DeviceBrandController
 * @description
 *  device brand 선택
 */
app.controller('DeviceBrandController', function ($scope, $state, Database, Devices) {
	$scope.device.brand = null;
	$scope.brands = Database.getBrands();

	$scope.doNext = function () {
		Devices.setCurrentDevice(angular.copy($scope.device));

		$state.go('menu.programming.add-device.device-model');
	};

	$scope.goCustomDriver = function () {
		$scope.device.customDriver = true;

		Devices.setCurrentDevice(angular.copy($scope.device));

		$state.go('menu.programming.add-device.custom-driver');
	};
});

/**
 * @ngdoc controller
 * @name DeviceCustomController
 * @description
 *  custom driver 생성
 */
app.controller('DeviceCustomDriverController', function ($scope, $state, Database, Devices) {
	$scope.$watch('device.targetType', function(newVal, oldVal) {
		if (newVal == 1) {
			$scope.device.targetPort = 255;
			$scope.device.targetAddr = $scope.room.bstations[0].mac;
			if('VIRTUAL' == $scope.room.bstations[0].type) {
				$scope.device.targetAddr = $scope.masterBStation.mac;
			}
		}
	});

	$scope.doNext = function () {
		$state.go('menu.programming.add-device.device-settings');
	};
});

/**
 * @ngdoc controller
 * @name DeviceModelController
 * @description
 *  device popuplar model 선택
 */
app.controller('DeviceModelController', function ($scope, $state, $modal, $http, toastr, popularDevices, Database) {

	$scope.device.model = null;
	$scope.device.info = null;
	$scope.query.value = '';

	$scope.models = popularDevices;
	$scope.models.ir = [];
	$scope.models.ip = [];

	_.forEach(popularDevices.p_ir, function (device, i) {
		device.index = i;
		$scope.models.ir.push(device);
	});

	_.forEach(popularDevices.p_ip, function (device, i) {
		device.index = i;
		$scope.models.ip.push(device);
	});

	if ($scope.models.ir.length) {
		$scope.device.targetType = 1;
	}

	$scope.doNext = function () {
		return Database.getDeviceInfo($scope.device.type.name, $scope.device.brand.name, $scope.device.model.id, $scope.device.targetType, $scope.device.model.type)
			.then(function (data) {
				var attrs = data.attr;
				console.log(attrs);
				if (attrs.codeset === 'ip') {
					$scope.device.targetPort = $scope.device.targetPort || parseInt(attrs.port, 10);
				} else {
					$scope.device.targetPort = 255;
					$scope.device.targetAddr = $scope.room.bstations[0].mac;
					if('VIRTUAL' == $scope.room.bstations[0].type) {
						$scope.device.targetAddr = $scope.masterBStation.mac;
					}
				}

				$scope.device.powerOff = false;
				if (data.meta.length) {
					_.each(data.meta, function (meta) {
						if (meta.strid === 'POWER_OFF' || meta.strid === 'POWER_TOGGLE') {
							$scope.device.powerOff = true;
						}
					});
				}

				$state.go('menu.programming.add-device.device-settings');
			})
			.catch(toastr.error);
	};

	// 테스트 팝업 화면 띄우기
	$scope.showTestModal = function (allCodes) {
		if (allCodes) {
			$scope.allCodes = true;
		} else {
			$scope.allCodes = false;
		}

		// power on, mute, 나머지 commands 얻기 위해 getDeviceInfo
		Database.getDeviceInfo($scope.device.type.name, $scope.device.brand.name, $scope.device.model.id, $scope.device.targetType, $scope.device.model.type)
			.then(function (data) {
				$scope.device.info = data;
				$scope.device.powerBtn = false;
				$scope.device.powerBtnData = null;
				$scope.device.muteBtn = false;
				$scope.device.muteBtnData = null;

				_.each(data.btns, function (btn) {
					if (btn.def == 'POWER ON' || btn.def == 'POWER TOGGLE') {
						$scope.device.powerBtn = true;
						$scope.device.powerBtnData = btn.cmd;
					}
					if (btn.def == 'MUTE') {
						$scope.device.muteBtn = true;
						$scope.device.muteBtnData = btn.cmd;
					}
				});

				if (data.attr.codeset === 'ip') {
					$scope.device.targetPort = parseInt(data.attr.port, 10);
				} else {
					$scope.device.targetPort = -1;
				}

				var modal = $modal({
					scope: $scope,
					placement: 'top',
					template: 'views/modal-tpls/test-commands.html',
					container: 'body',
					show: false
				});
				modal.$promise.then(modal.show);
			})
			.catch(toastr.error);
	};

	$scope.device.testBstationMac = $scope.room.bstations[0].mac;
	if('VIRTUAL' == $scope.room.bstations[0].type) {
		$scope.device.testBstationMac = $scope.masterBStation.mac;
	}

	// 테스트 실행
	$scope.btnTest = function (func) {
		var testData = {},
			data;

		if ($scope.device.targetType === 2) {
			if (!$scope.device.targetAddr) {
				toastr.error('An IP address is required.');
				return;
			}
		}

		// power on
		if (func == 'power') {
			data = $scope.device.powerBtnData;
		}
		// mute
		else if (func == 'mute') {
			data = $scope.device.muteBtnData;
		}
		// other commands
		else {
			data = func;
		}

		if ($scope.device.targetType === 1) {
			testData = {
				"systemId": $scope.system.id, "type": $scope.device.targetType,
				"addr": $scope.device.testBstationMac, "port": $scope.device.targetPort, "repeat": $scope.device.info.attr.rpt_cnt,
				"ir_type": func.type || 0, "data": data
			};
		}
		else {
			testData = {
				"systemId": $scope.system.id, "type": $scope.device.targetType,
				"addr": $scope.device.targetAddr, "port": $scope.device.targetPort, "repeat": $scope.device.info.attr.rpt_cnt,
				"ramp_start": $scope.device.info.attr.ramp_st, "ramp_speed": $scope.device.info.attr.ramp_sp, "auto_cr": $scope.device.info.attr.auto_cr,
				"data": data
			};
		}

		var jsonTestData = JSON.stringify(testData);

		$http({
			method: 'POST',
			url: '/api/cloud/test',
			data: jsonTestData
		}).
			success(function (data) {
				if (data.result === true) {
					toastr.success('Test command sent');
				}
				else {
					toastr.error(data.error);
				}
			}).
			error(function () {
				toastr.error("Please try again");
			});
	};

	$scope.goSearchModel = function () {
		$state.go('menu.programming.add-device.device-model-search');
	};

	$scope.goAllModels = function () {
		$state.go('menu.programming.add-device.device-model-all');
	};

	// UI 에서 IR controlled 와 Network controlled 를 변경하면 device.model 초기화
	$scope.$watch('device.targetType', function () {
		$scope.device.model = null;
	});
});

/**
 * @ngdoc controller
 * @name DeviceModelAllController
 * @description
 *  device all model 선택
 */
app.controller('DeviceModelAllController', function ($scope, $state, $modal, $http, toastr, allDevices, Database) {

	$scope.device.model = null;
	$scope.device.info = null;
	$scope.query.value = '';

	$scope.models = allDevices;
	$scope.models.ir = [];
	$scope.models.ip = [];

	_.forEach(allDevices.p_ir, function (device, i) {
		device.index = i;
		$scope.models.ir.push(device);
	});

	_.forEach(allDevices.p_ip, function (device, i) {
		device.index = i;
		$scope.models.ip.push(device);
	});

	if ($scope.models.ir.length) {
		$scope.device.targetType = 1;
	}

	$scope.doNext = function () {
		return Database.getDeviceInfo($scope.device.type.name, $scope.device.brand.name, $scope.device.model.id, $scope.device.targetType, $scope.device.model.type)
			.then(function (data) {
				var attrs = data.attr;
				console.log(attrs);
				if (attrs.codeset === 'ip') {
					$scope.device.targetPort = $scope.device.targetPort || parseInt(attrs.port, 10);
				} else {
					$scope.device.targetPort = 255;
					$scope.device.targetAddr = $scope.room.bstations[0].mac;
					if('VIRTUAL' == $scope.room.bstations[0].type) {
						$scope.device.targetAddr = $scope.masterBStation.mac;
					}
				}

				$scope.device.powerOff = false;
				if (data.meta.length) {
					_.each(data.meta, function (meta) {
						if (meta.strid === 'POWER_OFF' || meta.strid === 'POWER_TOGGLE') {
							$scope.device.powerOff = true;
						}
					});
				}

				$state.go('menu.programming.add-device.device-settings');
			})
			.catch(toastr.error);
	};

	// 테스트 팝업 화면 띄우기
	$scope.showTestModal = function (allCodes) {
		if (allCodes) {
			$scope.allCodes = true;
		} else {
			$scope.allCodes = false;
		}

		// power on, mute, 나머지 commands 얻기 위해 getDeviceInfo
		Database.getDeviceInfo($scope.device.type.name, $scope.device.brand.name, $scope.device.model.id, $scope.device.targetType, $scope.device.model.type)
			.then(function (data) {
				$scope.device.info = data;
				$scope.device.powerBtn = false;
				$scope.device.powerBtnData = null;
				$scope.device.muteBtn = false;
				$scope.device.muteBtnData = null;

				_.each(data.btns, function (btn) {
					if (btn.def == 'POWER ON' || btn.def == 'POWER TOGGLE') {
						$scope.device.powerBtn = true;
						$scope.device.powerBtnData = btn.cmd;
					}
					if (btn.def == 'MUTE') {
						$scope.device.muteBtn = true;
						$scope.device.muteBtnData = btn.cmd;
					}
				});

				if (data.attr.codeset === 'ip') {
					$scope.device.targetPort = parseInt(data.attr.port, 10);
				} else {
					$scope.device.targetPort = -1;
				}

				var modal = $modal({
					scope: $scope,
					placement: 'top',
					template: 'views/modal-tpls/test-commands.html',
					container: 'body',
					show: false
				});
				modal.$promise.then(modal.show);
			})
			.catch(toastr.error);
	};

	$scope.device.testBstationMac = $scope.room.bstations[0].mac;
	if('VIRTUAL' == $scope.room.bstations[0].type) {
		$scope.device.testBstationMac = $scope.masterBStation.mac;
	}

	// 테스트 실행
	$scope.btnTest = function (func) {
		var testData = {},
			data;

		if ($scope.device.targetType === 2) {
			if (!$scope.device.targetAddr) {
				toastr.error('An IP address is required.');
				return;
			}
		}

		// power on
		if (func == 'power') {
			data = $scope.device.powerBtnData;
		}
		// mute
		else if (func == 'mute') {
			data = $scope.device.muteBtnData;
		}
		// other commands
		else {
			data = func;
		}

		if ($scope.device.targetType === 1) {
			testData = {
				"systemId": $scope.system.id, "type": $scope.device.targetType,
				"addr": $scope.device.testBstationMac, "port": $scope.device.targetPort, "repeat": $scope.device.info.attr.rpt_cnt,
				"ir_type": func.type || 0, "data": data
			};
		}
		else {
			testData = {
				"systemId": $scope.system.id, "type": $scope.device.targetType,
				"addr": $scope.device.targetAddr, "port": $scope.device.targetPort, "repeat": $scope.device.info.attr.rpt_cnt,
				"ramp_start": $scope.device.info.attr.ramp_st, "ramp_speed": $scope.device.info.attr.ramp_sp, "auto_cr": $scope.device.info.attr.auto_cr,
				"data": data
			};
		}

		var jsonTestData = JSON.stringify(testData);

		$http({
			method: 'POST',
			url: '/api/cloud/test',
			data: jsonTestData
		}).
			success(function (data) {
				if (data.result === true) {
					toastr.success('Test command sent');
				}
				else {
					toastr.error(data.error);
				}
			}).
			error(function () {
				toastr.error("Please try again");
			});
	};

	// UI 에서 IR controlled 와 Network controlled 를 변경하면 device.model 초기화
	$scope.$watch('device.targetType', function () {
		$scope.device.model = null;
	});
});

/**
 * @ngdoc controller
 * @name DeviceModelSearchController
 * @description
 *  device model 검색
 */
app.controller('DeviceModelSearchController', function ($scope, $state, $modal, $http, toastr, Database) {

	$scope.device.model = null;
	$scope.device.info = null;
	$scope.query.value = '';


	$scope.doNext = function () {
		return Database.getDeviceInfo($scope.device.type.name, $scope.device.brand.name, $scope.device.model.id, $scope.device.targetType, $scope.device.model.type)
			.then(function (data) {
				var attrs = data.attr;
				console.log(attrs);
				if (attrs.codeset === 'ip') {
					$scope.device.targetPort = $scope.device.targetPort || parseInt(attrs.port, 10);
				} else {
					$scope.device.targetPort = 255;
					$scope.device.targetAddr = $scope.room.bstations[0].mac;
					if('VIRTUAL' == $scope.room.bstations[0].type) {
						$scope.device.targetAddr = $scope.masterBStation.mac;
					}
				}

				$scope.device.powerOff = false;
				if (data.meta.length) {
					_.each(data.meta, function (meta) {
						if (meta.strid === 'POWER_OFF' || meta.strid === 'POWER_TOGGLE') {
							$scope.device.powerOff = true;
						}
					});
				}

				$state.go('menu.programming.add-device.device-settings');
			})
			.catch(toastr.error);
	};

	// 테스트 팝업 화면 띄우기
	$scope.showTestModal = function (allCodes) {
		if (allCodes) {
			$scope.allCodes = true;
		} else {
			$scope.allCodes = false;
		}

		// power on, mute, 나머지 commands 얻기 위해 getDeviceInfo
		Database.getDeviceInfo($scope.device.type.name, $scope.device.brand.name, $scope.device.model.id, $scope.device.targetType, $scope.device.model.type)
			.then(function (data) {
				$scope.device.info = data;
				$scope.device.powerBtn = false;
				$scope.device.powerBtnData = null;
				$scope.device.muteBtn = false;
				$scope.device.muteBtnData = null;

				_.each(data.btns, function (btn) {
					if (btn.def == 'POWER ON' || btn.def == 'POWER TOGGLE') {
						$scope.device.powerBtn = true;
						$scope.device.powerBtnData = btn.cmd;
					}
					if (btn.def == 'MUTE') {
						$scope.device.muteBtn = true;
						$scope.device.muteBtnData = btn.cmd;
					}
				});

				if (data.attr.codeset === 'ip') {
					$scope.device.targetPort = parseInt(data.attr.port, 10);
				} else {
					$scope.device.targetPort = -1;
				}

				var modal = $modal({
					scope: $scope,
					placement: 'top',
					template: 'views/modal-tpls/test-commands.html',
					container: 'body',
					show: false
				});
				modal.$promise.then(modal.show);
			})
			.catch(toastr.error);
	};

	$scope.device.testBstationMac = $scope.room.bstations[0].mac;
	if('VIRTUAL' == $scope.room.bstations[0].type) {
		$scope.device.testBstationMac = $scope.masterBStation.mac;
	}

	// 테스트 실행
	$scope.btnTest = function (func) {
		var testData = {},
			data;

		if ($scope.device.targetType === 2) {
			if (!$scope.device.targetAddr) {
				toastr.error('An IP address is required.');
				return;
			}
		}

		// power on
		if (func == 'power') {
			data = $scope.device.powerBtnData;
		}
		// mute
		else if (func == 'mute') {
			data = $scope.device.muteBtnData;
		}
		// other commands
		else {
			data = func;
		}

		if ($scope.device.targetType === 1) {
			testData = {
				"systemId": $scope.system.id, "type": $scope.device.targetType,
				"addr": $scope.device.testBstationMac, "port": $scope.device.targetPort, "repeat": $scope.device.info.attr.rpt_cnt,
				"ir_type": func.type || 0, "data": data
			};
		}
		else {
			testData = {
				"systemId": $scope.system.id, "type": $scope.device.targetType,
				"addr": $scope.device.targetAddr, "port": $scope.device.targetPort, "repeat": $scope.device.info.attr.rpt_cnt,
				"ramp_start": $scope.device.info.attr.ramp_st, "ramp_speed": $scope.device.info.attr.ramp_sp, "auto_cr": $scope.device.info.attr.auto_cr,
				"data": data
			};
		}

		var jsonTestData = JSON.stringify(testData);

		$http({
			method: 'POST',
			url: '/api/cloud/test',
			data: jsonTestData
		}).
			success(function (data) {
				if (data.result === true) {
					toastr.success('Test command sent');
				}
				else {
					toastr.error(data.error);
				}
			}).
			error(function () {
				toastr.error("Please try again");
			});
	};

	// UI 에서 IR controlled 와 Network controlled 를 변경하면 device.model 초기화
	$scope.$watch('device.targetType', function () {
		$scope.device.model = null;
	});
});

/**
 * @ngdoc controller
 * @name DeviceSettingsController
 * @description
 *  device 정보 설정 후 생성
 */
app.controller('DeviceSettingsController', function ($rootScope, $scope, $state, toastr, types, Systems, Database, Devices) {
	if ($rootScope.virtualEditor) {
		$scope.wireless_ir_port = 1;
		$scope.irPortOptions = [{ name: 'All', value: 255 }, { name: '1', value: 1 }, { name: '2', value: 2 }, {
			name: '3',
			value: 3
		}, { name: '4', value: 4 }];
	} else {
		var bstationType = $scope.room.bstations[0].type;
		if('VIRTUAL' == bstationType) {
			bstationType = $scope.masterBStation.type;
		}

		var	irInfo = _.find(types, { name: bstationType });

		$scope.wireless_ir_port = irInfo.wireless_ir_port;

		getIrPortOptions(irInfo);
	}

	function getIrPortOptions(irInfo) {
		if (irInfo.wired_ir_port_all && irInfo.wired_ir_port == 4) {
			$scope.irPortOptions = [{ name: 'All', value: 255 }, { name: '1', value: 1 }, { name: '2', value: 2 }, {
				name: '3',
				value: 3
			}, { name: '4', value: 4 }];
		} else if (irInfo.wired_ir_port_all && irInfo.wired_ir_port == 5) {
			$scope.irPortOptions = [{ name: 'All', value: 255 }, { name: '1', value: 1 }, { name: '2', value: 2 }, {
				name: '3',
				value: 3
			}, { name: '4', value: 4 }, { name: '5', value: 5 }];
		} else if (!irInfo.wired_ir_port_all && irInfo.wired_ir_port == 4) {
			$scope.irPortOptions = [{ name: '1', value: 1 }, { name: '2', value: 2 }, { name: '3', value: 3 }, {
				name: '4',
				value: 4
			}];
		} else if (!irInfo.wired_ir_port_all && irInfo.wired_ir_port == 5) {
			$scope.irPortOptions = [{ name: '1', value: 1 }, { name: '2', value: 2 }, { name: '3', value: 3 }, {
				name: '4',
				value: 4
			}, { name: '5', value: 5 }];
		}
	}

	$scope.$watch('device.targetAddr', function (newVal, oldVal) {
		var bstation = _.find($scope.bstations, { mac: newVal });
		console.log('newVal=', newVal, 'bstation=', bstation);
		if (bstation) {
			var bstationType = bstation.type,
				irInfo = _.find(types, { name: bstationType });

			$scope.wireless_ir_port = irInfo.wireless_ir_port;

			getIrPortOptions(irInfo);
		}
	});

	var system = Systems.getCurrentSystem(),
		allBstations = [];
	_.each(system.rooms, function (room) {
		allBstations = allBstations.concat(angular.copy(room.bstations));
	});
	$scope.bstations = allBstations;
	//virtual base station을 제외시켜야 함.
	var idx = _.findIndex($scope.bstations, {'type': 'VIRTUAL'});
	while(0 <= idx) {
		$scope.bstations.splice(idx, 1);
		idx = _.findIndex($scope.bstations, {'type': 'VIRTUAL'});
	}

	$scope.ir = { target: 0, port: 255 };
	$scope.ir.target = ($scope.device.targetPort ? 1 : 2);
	$scope.ir.port = ($scope.device.targetPort ? $scope.device.targetPort : 255);
	console.log('device = ', $scope.device);
	$scope.isEntDevice = Database.checkIfEntertainment($scope.device.type.name);

	$scope.doNext = function () {
		if ($scope.device.targetType === 1) {
			console.log('ir.target = ', $scope.ir.target);
			if (1 == $scope.ir.target)
				$scope.device.targetPort = $scope.ir.port;
			else
				$scope.device.targetPort = 0;
		}
		console.log($scope.device.targetPort);

		if ($scope.isEntDevice) {   // entertainment device 인 경우

			$state.go('menu.programming.add-device.favchannels-option');

		} else {  // entertainment device 가 아닌 경우
			var d = angular.copy($scope.device);

			if (d.customDriver) {
				var data = {
					"id": "-1",
					"name": d.name,
					"meta_in_toggle": -1,
					"meta_pwr_toggle": -1,
					"meta_pwr_on": -1,
					"meta_pwr_off": -1,
					"repeat": {
						"min": 1,
						"macro": 3
					},
					"dbCon": [],
					"dbKey": [],
					"pages": [{
						"id": -1,
						"btns": []
					}],
					"category": d.type.name,
					"brand": 'MX HomePro Custom Driver',
					"model": '',
					"target": {
						"type": d.targetType,
						"port": d.targetPort,
						"addr": d.targetAddr
					}
				};
	
				var jsonData = JSON.stringify({ data: data });
	
				// learn device 생성
				Devices.createCustomOne($scope.room.id, jsonData)
				.then(function (data) {
					var id = Systems.getCurrentSystem().id;
					return Systems.updateSystem(id);
				})
				.then(function (data) {
					$state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true });
					toastr.success('Successfully added ' + d.name + '. Go to Edit a device\'s layout and functions on the Advanced Menu to edit your Custom Driver.');
				})
				.catch(toastr.error);
			} else {
				d.favEnable = 0;
				d.favDelay = 400;
				d.favEnter = 0;
				d.type = d.type.name;
				d.brand = d.brand.name;
				d.dbType = parseInt(d.model.type, 10);
				d.modelId = parseInt(d.model.id, 10);
				d.model = d.model.name;
				var jsonData = JSON.stringify(d);
	
				// device 생성
				return Devices.createOne($scope.room.id, jsonData)
					.then(function (data) {
						console.log(data);
						var id = Systems.getCurrentSystem().id;
						return Systems.updateSystem(id);
					})
					.then(function (data) {
						$state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true });
						toastr.success('Successfully added ' + d.name + '.');
					})
					.catch(toastr.error);
			}
		}
	};
});

/**
 * @ngdoc controller
 * @name FavChannelsOptionController
 * @description
 *  favorite channels 설정
 */
app.controller('FavChannelsOptionController', function ($scope, $state, $modal, toastr, Systems, Devices) {

	$scope.device.favEnable = $scope.device.favEnable || 1;
	$scope.device.favDelay = $scope.device.favDelay || 400;
	$scope.device.favEnter = $scope.device.favEnter || 1;

	$scope.finish = function () {

		var d = angular.copy($scope.device);

		if (d.customDriver) {
			var data = {
				"id": "-1",
				"name": d.name,
				"meta_in_toggle": -1,
				"meta_pwr_toggle": -1,
				"meta_pwr_on": -1,
				"meta_pwr_off": -1,
				"repeat": {
					"min": 1,
					"macro": 3
				},
				"dbCon": [],
				"dbKey": [],
				"pages": [{
					"id": -1,
					"btns": []
				}],
				"category": d.type.name,
				"brand": 'MX HomePro Custom Driver',
				"model": '11111',
				"target": {
					"type": d.targetType,
					"port": d.targetPort,
					"addr": d.targetAddr
				}
			};

			var jsonData = JSON.stringify({ data: data });

			// learn device 생성
			Devices.createCustomOne($scope.room.id, jsonData)
			.then(function (data) {
				var id = Systems.getCurrentSystem().id;
				return Systems.updateSystem(id);
			})
			.then(function (data) {
				$state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true });
				toastr.success('Successfully added ' + d.name + '. Go to Edit a device\'s layout and functions on the Advanced Menu to edit your Custom Driver.');
			})
			.catch(toastr.error);
		} else {
			d.type = d.type.name;
			d.brand = d.brand.name;
			d.dbType = parseInt(d.model.type, 10);
			d.modelId = parseInt(d.model.id, 10);
			d.model = d.model.name;
	
			var jsonData = JSON.stringify(d);
			console.log(jsonData);
	
			// device 생성
			return Devices.createOne($scope.room.id, jsonData)
				.then(function (data) {
					console.log(data);
					var id = Systems.getCurrentSystem().id;
					return Systems.updateSystem(id);
				})
				.then(function (data) {
					$state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true });
					toastr.success('Successfully added ' + d.name + '.');
				})
				.catch(toastr.error);
		}
	};
});

'use strict';
/**
 * @ngdoc controller
 * @name AddMacroRoomController
 * @description
 *  room 선택 화면
 */
app.controller('AddMacroRoomController', function ($rootScope, $scope, $state, toastr, Systems, Devices) {
	$scope.system = Systems.getCurrentSystem();


	$scope.doNext = function() {
		// room 정보 얻어와서 저장
		return Systems.updateRoomDevices($scope.addMacro.room.id)
		.then(function(data) {
			var roomDevices = Systems.getRoomDevices();
			$scope.addMacro.room.devices = roomDevices.devices;

      		// dev_type이 twoway(1)인 device를 모두 제거하는 과정에서 $scope.addMacro.room.devices 에서 SNP-2 가 제거되는데 SNP-2 가 있는 목록이 "Add jump to" 에서 필요해서 copy 를 만듬
			$scope.addMacro.room.devicesAndSNP2 = angular.copy(roomDevices.devices);

			// twoway_devices 중 TRF-ZW 추가한 device 목록 생성
			$scope.addMacro.room.allDevices = angular.copy($scope.addMacro.room.devices);

			///target_type이 0인 device를 모두 제거.
			_.each($scope.addMacro.room.allDevices, function (dev) {
				if(0 == dev.target_type) {
					var idx = $scope.addMacro.room.allDevices.indexOf(dev);
					$scope.addMacro.room.allDevices.splice(idx, 1);
				}
			});

			_.each($scope.system.rooms, function(room) {
				var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
				if (trfzw) {
					if (!trfzw.data) {
						trfzw.data = [];
					}
					$scope.addMacro.room.allDevices.push(trfzw);

					return false;
				}
			});

			// allDevices 에 interface device 추가
			_.each(roomDevices.twoway_devices, function (twowayDevice) {
				if (twowayDevice.device_type === 2) {
				$scope.addMacro.room.allDevices.push(twowayDevice);
				// core device 의 funcs 를 복사
				Devices.getTwowayDeviceCommands(twowayDevice.link_room_id, twowayDevice.link_dev_id)
					.then(function (data) {
					twowayDevice.funcs = data;
					});
				}
			});

			// room devices 에 entertainment twoway devices 추가 - core device 는 추가하지 않음
			var entTwowayDevices = [];
			_.each(roomDevices.twoway_devices, function (item) {
				if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
				entTwowayDevices.push(item);
				}
			});
			$scope.addMacro.room.devices = $scope.addMacro.room.devices.concat(entTwowayDevices);

			$state.go('menu.advanced.add-macros.macro-type');
		})
		.catch(toastr.error);
	};
});

/**
 * @ngdoc controller
 * @name MacroTypeController
 * @description
 *  macro type 선택 화면
 */
app.controller('MacroTypeController', function ($rootScope, $scope, $state, toastr, Systems, Devices) {
  // Advanced Menu 의 Automate activities on the Entertainment Menu 메뉴를 클릭해서 들어온 경우
  // Programming Menu 의 Setup a simple activity 메뉴에서 넘어온 경우.
  //   => currentRoom 을 선택된 room 으로 설정함
  if (!$scope.addMacro.room.id) {
    var currentRoomId,
      timer;

    // currentRoom 이 설정되었는지 0.1초에 한번씩 확인
    timer = setInterval(function() {
      currentRoomId = Systems.getCurrentRoom();

      if (currentRoomId) {
        clearInterval(timer);

        $scope.addMacro.room = _.findWhere($scope.system.rooms, {
          id: currentRoomId
        });
        console.log('room', $scope.addMacro.room);

        // 아래의 코드는 AddMacroRoomController 의 doNext 에서 하는 동작임
        Systems.updateRoomDevices($scope.addMacro.room.id)
          .then(function(data) {
            var roomDevices = Systems.getRoomDevices();
            $scope.addMacro.room.devices = roomDevices.devices;

            // dev_type이 twoway(1)인 device를 모두 제거하는 과정에서 $scope.addMacro.room.devices 에서 SNP-2 가 제거되는데 SNP-2 가 있는 목록이 "Add jump to" 에서 필요해서 copy 를 만듬
            $scope.addMacro.room.devicesAndSNP2 = angular.copy(roomDevices.devices);

            // twoway_devices 중 TRF-ZW 추가한 device 목록 생성
			$scope.addMacro.room.allDevices = angular.copy($scope.addMacro.room.devices);
			
			///target_type이 0인 device를 모두 제거.
			_.each($scope.addMacro.room.allDevices, function (dev) {
				if(0 == dev.target_type) {
					var idx = $scope.addMacro.room.allDevices.indexOf(dev);
					$scope.addMacro.room.allDevices.splice(idx, 1);
				}
			});

            _.each($scope.system.rooms, function(room) {
              var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
              if (trfzw) {
                if (!trfzw.data) {
                  trfzw.data = [];
                }
                $scope.addMacro.room.allDevices.push(trfzw);

                return false;
              }
            });

            // allDevices 에 interface device 추가
            _.each(roomDevices.twoway_devices, function (twowayDevice) {
              if (twowayDevice.device_type === 2) {
                $scope.addMacro.room.allDevices.push(twowayDevice);
                // core device 의 funcs 를 복사
                Devices.getTwowayDeviceCommands(twowayDevice.link_room_id, twowayDevice.link_dev_id)
                  .then(function (data) {
                    twowayDevice.funcs = data;
                  });
              }
            });

            // room devices 에 entertainment twoway devices 추가 - core device 는 추가하지 않음
            var entTwowayDevices = [];
            _.each(roomDevices.twoway_devices, function (item) {
              if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
                entTwowayDevices.push(item);
              }
            });
            $scope.addMacro.room.notEntDevices = angular.copy($scope.addMacro.room.devices);
            $scope.addMacro.room.devices = $scope.addMacro.room.devices.concat(entTwowayDevices);

            $state.go('menu.advanced.add-macros.macro-type');
          })
          .catch(toastr.error);
      }
    }, 100);
  }

	var steps = $scope.addMacro.steps;
	while(steps.length > 0) {
		steps.pop();
	}
	$scope.addMacro.currentDevice = {};
	$scope.addMacro.button = {};
	$scope.addMacro.doneScreen = {'index': 0,'device': {}, 'devices': []};
  $scope.addMacro.disablePopup = 1;

	$scope.navigateTo = function(state) {

		var base = 'menu.advanced.add-macros';
		switch (state) {
			case 'Turn Off All':
				$scope.macroType.id = 0;
				// 해당 room 의 power macro 데이터 얻어와서 저장
				Systems.updateCurrentMacroForPower($scope.addMacro.room.id).then( function(data) {
					console.log(data);
					$state.go(base + '.finish');
				}).catch(toastr.error);

				break;
			case 'Device Activity':
				$scope.macroType.id =  1;
				$state.go(base + '.choose-device');
				break;
			case 'Custom Button':
				$scope.macroType.id = 2;
				$state.go(base + '.button-name');
				break;
			case 'Scheduled Event':
				$scope.macroType.id = 3;
				$state.go(base + '.button-name');
				break;
			default:
				//do nothing
		}
	};

});

/**
 * @ngdoc controller
 * @name ButtonNameController
 * @description
 *  entertainment page button 생성할 button name 입력 화면
 */
app.controller('ButtonNameController', function ($rootScope, $scope, $state, $stateParams, toastr, Systems, Devices, Macro) {
  // (1) Advanced Menu 의 Edit a device's layout and functions 에서 blank button 을 클릭해서 들어온 경우
  //   => currentRoom 을 선택된 room 으로 설정함
  //   => $stateParams.jsonData 가 있는 경우는 수정하러 들어온 경우
  // (2) Integrate with an External Service 에서 들어온 경우
  if (!$scope.addMacro.room.id || $stateParams.jsonData) {
    if ($stateParams.extService) {
      $scope.addMacro.extService = JSON.parse($stateParams.extService);
    } else {
      $scope.addMacro.fromDeviceLayout = {
        roomId: $stateParams.roomId,
        deviceId: $stateParams.deviceId,
        page: $stateParams.page,
        idx: $stateParams.idx
      };
    }

    var currentRoomId,
      timer;

    // currentRoom 이 설정되었는지 0.1초에 한번씩 확인
    timer = setInterval(function () {
      currentRoomId = Systems.getCurrentRoom();

      if ($scope.addMacro.extService && $scope.addMacro.extService.scene && $scope.addMacro.extService.scene.roomId) {
        currentRoomId = $scope.addMacro.extService.scene.roomId;
      }

      if (currentRoomId) {
        clearInterval(timer);

        $scope.addMacro.room = _.findWhere($scope.system.rooms, {
          id: currentRoomId
        });
        console.log('room', $scope.addMacro.room);

        // 아래의 코드는 AddMacroRoomController 의 doNext 에서 하는 동작임
        Systems.updateRoomDevices($scope.addMacro.room.id)
          .then(function (data) {
            var roomDevices = Systems.getRoomDevices();
            $scope.addMacro.room.devices = roomDevices.devices;

            // dev_type이 twoway(1)인 device를 모두 제거하는 과정에서 $scope.addMacro.room.devices 에서 SNP-2 가 제거되는데 SNP-2 가 있는 목록이 "Add jump to" 에서 필요해서 copy 를 만듬
            $scope.addMacro.room.devicesAndSNP2 = angular.copy(roomDevices.devices);

            // twoway_devices 중 TRF-ZW 추가한 device 목록 생성
            $scope.addMacro.room.allDevices = angular.copy($scope.addMacro.room.devices);

			///target_type이 0인 device를 모두 제거.
			_.each($scope.addMacro.room.allDevices, function (dev) {
				if(0 == dev.target_type) {
					var idx = $scope.addMacro.room.allDevices.indexOf(dev);
					$scope.addMacro.room.allDevices.splice(idx, 1);
				}
			});

            _.each($scope.system.rooms, function (room) {
              var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
              if (trfzw) {
                if (!trfzw.data) {
                  trfzw.data = [];
                }
                $scope.addMacro.room.allDevices.push(trfzw);

                return false;
              }
            });

            // allDevices 에 interface device 추가
            _.each(roomDevices.twoway_devices, function (twowayDevice) {
              if (twowayDevice.device_type === 2) {
                $scope.addMacro.room.allDevices.push(twowayDevice);
                // core device 의 funcs 를 복사
                Devices.getTwowayDeviceCommands(twowayDevice.link_room_id, twowayDevice.link_dev_id)
                  .then(function (data) {
                    twowayDevice.funcs = data;
                  });
              }
            });

            // room devices 에 entertainment twoway devices 추가 - core device 는 추가하지 않음
            var entTwowayDevices = [];
            _.each(roomDevices.twoway_devices, function (item) {
              if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
                entTwowayDevices.push(item);
              }
            });
            $scope.addMacro.room.devices = $scope.addMacro.room.devices.concat(entTwowayDevices);

            // 아래의 코드는 MacroTypeController 에서 하는 동작임
            var steps = $scope.addMacro.steps;
            while (steps.length > 0) {
              steps.pop();
            }
            $scope.addMacro.currentDevice = {};
            $scope.addMacro.button = {};
            $scope.addMacro.doneScreen = {'index': 0, 'device': {}, 'devices': []};
            $scope.addMacro.disablePopup = 1;

            $scope.macroType.id = 2;

            $state.go('menu.advanced.add-macros.button-name');

            // Advanced Menu 의 Edit a device's layout and functions 에서 수정하러 들어온 경우
            if ($stateParams.jsonData) {
              var temp = JSON.parse($stateParams.jsonData),
                activities = _.isObject(temp.activity) ? temp.activity : JSON.parse(temp.activity),
                editDevice = $.extend(true, {}, _.find($scope.addMacro.room.devices, {
                  id: $stateParams.deviceId
                }));

              $scope.addMacro.steps = [];
              $scope.addMacro.steps = Macro.convertMacroDataToDispData($scope.system, $scope.addMacro.room, temp.macro);

              _.each(activities, function (item) {
                var activityDevice = _.find($scope.addMacro.room.devices, {
                    id: +item.device_id
                  }),
                  func = _.find(activityDevice.funcs, {
                    id: item.func
                  }),
                  obj = {
                    device_id: activityDevice.id,
                    func: func,
                    name: activityDevice.name,
                    repeat_macro: activityDevice.repeat_macro,
                    repeat_min: activityDevice.repeat_min,
                    target_addr: activityDevice.target_addr,
                    target_port: activityDevice.target_port,
                    target_type: activityDevice.target_type,
                    type: item.type
                  };

                $scope.addMacro.doneScreen.devices.push(obj);
              });

              $scope.addMacro.button.name = temp.display_text;
              $scope.addMacro.device = editDevice;
              $scope.addMacro.doneScreen.device = editDevice;
              $scope.addMacro.disablePopup = +temp.popup;
              if (temp.func) {
                $scope.addMacro.func = {
                  data: temp.func.data,
                  id: temp.func.id,
                  name: temp.func.name,
                  type: temp.func.type
                };
              }
            }
            // Amazon Alexa 수정하러 들어온 경우
            else if ($scope.addMacro.extService && $scope.addMacro.extService.scene && $scope.addMacro.extService.scene.id && $scope.addMacro.extService.scene.macro) {
              $scope.addMacro.steps = [];
              $scope.addMacro.steps = Macro.convertMacroDataToDispData($scope.system, $scope.addMacro.room, $scope.addMacro.extService.scene.macro);
              $scope.addMacro.button.name = '';
              console.log('steps', $scope.addMacro.steps);
            }

            // Integrate with an Externam Service 에서 들어온 경우
            if ($stateParams.extService) {
              $state.go('menu.advanced.add-macros.finish');
              return;
            }
          })
          .catch(toastr.error);
      }
    }, 100);
  } else {
    if (!$scope.addMacro.fromDeviceLayout) {
      $scope.addMacro.fromDeviceLayout = null;
    }
  }

  if ($scope.addMacro.fromDeviceLayout) {
    if ($stateParams.jsonData) {
      $scope.addMacro.fromDeviceLayout.jsonData = decodeURIComponent($stateParams.jsonData);
    }

    $scope.goDeviceLayout = function () {
      if ($stateParams.jsonData) {
        $state.go('menu.advanced.device.edit-layout', angular.extend({}, $scope.addMacro.fromDeviceLayout, {jsonData: decodeURIComponent($stateParams.jsonData)}));
      } else {
        $state.go('menu.advanced.device.edit-layout', angular.extend({}, $scope.addMacro.fromDeviceLayout, $scope.addMacro.fromDeviceLayout.jsonData || {
          jsonData: JSON.stringify({
            display_text: $scope.addMacro.button.name,
            activity: [],
            auto_wait: 1,
            edit_type: 1,
            popup: 0
          })
        }));
      }
    };
  }

	$scope.doNext = function() {
		$state.go('menu.advanced.add-macros.finish');
	};
});

/**
 * @ngdoc controller
 * @name ChooseMacroDeviceController
 * @description
 *  entertainment page device 생성할 device 선택 화면
 */
app.controller('ChooseMacroDeviceController', function ($scope, $state) {
	$scope.doNext = function() {
		$state.go('menu.advanced.add-macros.finish');
	};
});

/**
 * @ngdoc controller
 * @name FinishMacroController
 * @description
 *  macro device, commands 입력
 */
app.controller('FinishMacroController', function ($rootScope, $scope, $timeout, $state, $modal, toastr, Rooms, Devices, Systems, Macro) {
	var scheEventData = Rooms.getScheEventData();

	function isJsonString(str) {
		try {
			JSON.parse(str);
		} catch (e) {
			return false;
		}
		return true;
	}

	$scope.isJsonString = isJsonString;

	// scheduled event 수정하러 들어온 경우
	if (scheEventData) {
		$scope.addMacro.room = scheEventData.room;

		// room 정보 얻어와서 저장
		Systems.updateRoomDevices($scope.addMacro.room.id)
			.then(function(data) {
				var roomDevices = Systems.getRoomDevices();
				$scope.addMacro.room.devices = roomDevices.devices;

				$scope.addMacro.steps = Macro.convertMacroDataToDispData($scope.system, $scope.addMacro.room, scheEventData.event.macro);
				$scope.addMacro.button.name = scheEventData.event.display_text;
				$scope.addMacro.disablePopup = 1;
				$scope.addMacro.doneScreen.index = 0;
				$scope.addMacro.doneScreen.devices = [];

				$scope.macroType.id = 3;

				// dev_type이 twoway(1)인 device를 모두 제거하는 과정에서 $scope.addMacro.room.devices 에서 SNP-2 가 제거되는데 SNP-2 가 있는 목록이 "Add jump to" 에서 필요해서 copy 를 만듬
				$scope.addMacro.room.devicesAndSNP2 = angular.copy(roomDevices.devices);


				///dev_type이 twoway(1)인 device와 target_type이 0인 device를 모두 제거.
				_.each($scope.addMacro.room.devices, function (dev) {
					if(1 == dev.dev_type || 0 == dev.target_type) {
						var idx = $scope.addMacro.room.devices.indexOf(dev);
						$scope.addMacro.room.devices.splice(idx, 1);
					}
				});

				// twoway_devices 중 TRF-ZW 추가한 device 목록 생성
				$scope.addMacro.room.allDevices = angular.copy($scope.addMacro.room.devices);

				// allDevices 에 interface device 추가
				_.each(roomDevices.twoway_devices, function (twowayDevice) {
					if (twowayDevice.device_type === 2) {
						$scope.addMacro.room.allDevices.push(twowayDevice);
						// core device 의 funcs 를 복사
						Devices.getTwowayDeviceCommands(twowayDevice.link_room_id, twowayDevice.link_dev_id)
						.then(function (data) {
							twowayDevice.funcs = data;
						});
					}
        		});

				// room devices 에 entertainment twoway devices 추가 - core device 는 추가하지 않음
				var entTwowayDevices = [];
				_.each(roomDevices.twoway_devices, function (item) {
				if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
					entTwowayDevices.push(item);
				}
				});
		        $scope.addMacro.room.devices = $scope.addMacro.room.devices.concat(entTwowayDevices);

				var system = Systems.getCurrentSystem();
				_.each(system.rooms, function(room) {
					var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
					if (trfzw) {
						if (!trfzw.data) {
							trfzw.data = [];
						}
						$scope.addMacro.room.allDevices.push(trfzw);

						return false;
					}
				});
			})
			.catch(toastr.error);
	}


	$scope.remove = function(scope) {
		scope.remove();
	};

	$scope.options = {
		accept: function(sourceNodeScope, destNodesScope, destIndex) {
			//console.log('sourceNodeScope=', sourceNodeScope);
			//console.log('destNodesScope=', destNodesScope);
			if (sourceNodeScope.$modelValue.type == null || destNodesScope.nodropEnabled || destNodesScope.outOfDepth(sourceNodeScope)) {
				return false;
			}

	        var subDepth = 0;
	        function countSubDepth(scope) {
				var i, childNodesScope,
					count = 0,
					nodes = scope.childNodes();

				//console.log('childNodes=', nodes);

				for (i = 0; i < nodes.length; i++) {
					childNodesScope = nodes[i].$childNodesScope;

					if (childNodesScope && childNodesScope.childNodesCount() > 0) {
						count = 1;
						countSubDepth(childNodesScope);
					}
				}
				subDepth += count;				
			}

			var maxSubDepth = function (scope) {
				subDepth = 0;
				if (scope.hasChild()) {
					countSubDepth(scope.$childNodesScope);
					subDepth++;
				}
				return subDepth;
			};

			/*
			console.log('sourceNodeScope.childNodes()=', sourceNodeScope.childNodes());
			console.log('sourceNodeScope.childNodesCount()=', sourceNodeScope.childNodesCount());
			console.log('maxSubDepth(sourceNodeScope)=', maxSubDepth(sourceNodeScope));
			console.log('destNodesScope.depth()=', destNodesScope.depth());
			console.log('destNodesScope.maxDepth=', destNodesScope.maxDepth);
			console.log('destNodesScope.$treeScope.maxDepth=', destNodesScope.$treeScope.maxDepth);
			*/			

			//angular-ui-tree module의 maxSubDepth function에서 sub depth count를 child node 존재 유무를 기준으로 함.
			//그래서 2depth인 tree인 경우 depth count를 0으로 return하여 outOfDepth function을 호출해도 제대로된 결과를 받지 못하고,
			// (max-depth -1) depth인 tree에 2depth의 tree를 drag/drop하면 삽입이 됨.
			//그래서, 별도의 maxSubDepth function을 구현함.
			var sumDepth =  (maxSubDepth(sourceNodeScope)+1) + destNodesScope.depth();
			var maxDepth = destNodesScope.maxDepth || destNodesScope.$treeScope.maxDepth;

			if(sumDepth  > maxDepth){
				return false;
			}

			return true;
		},
	};

	var getRootNodesScope = function() {
		return angular.element(document.getElementById("tree-root")).scope();
	};

	$scope.collapseAll = function() {
		var scope = getRootNodesScope();
		scope.collapseAll();
	};

	$scope.expandAll = function() {
		var scope = getRootNodesScope();
		scope.expandAll();
	};


	$scope.addStep = function (device, func, twowayDevice) {
		if (twowayDevice) {
			return {'name': 'S : ' + func.name + ' (' + func.room + ')', 'type': 16, 'sub_type': 'scene', 'id': func.id, 'cmd': 1 };		// 1 : Activate, 2 : Deactivate
		} else {
			return {'name': device.name + ': ' + func.name, 'type': 1, 'device_id': device.id, 'func': func.id };
		}
	};

	$scope.deleteAll = function() {
		var arr = $scope.addMacro.steps;
		while(arr.length > 0) {
			arr.pop();                     
		}
	};

	$scope.goBack = function() {
    if ($scope.addMacro.extService) {
      if('amazon_echo' == $scope.addMacro.extService.service) {
        $state.go('menu.advanced.external-service.spoken-text', {extService: JSON.stringify($scope.addMacro.extService)});
      } else {
        $state.go('menu.advanced.external-service.assist-response', {extService: JSON.stringify($scope.addMacro.extService)});
      }
    } else {
      if ($scope.macroType.id === 1) {
        $state.go('menu.advanced.add-macros.choose-device');
      } else if ($scope.macroType.id === 2) {
        $state.go('menu.advanced.add-macros.button-name');
      } else {
        $state.go('menu.advanced.add-macros.macro-type');
      }
    }
	};

	$scope.doNext = function() {
    console.log('$scope.addMacro.steps=', $scope.addMacro.steps);
    // Edit a device's layout and functions 메뉴에서 넘어온 경우
    if ($scope.addMacro.fromDeviceLayout) {
      var steps = angular.copy($scope.addMacro.steps);
      steps = Macro.cleanUp(steps);

      var jsonData = {
        "text": $scope.addMacro.button.name,
        "auto_wait": 1,
        "edit_type": 1,
        "popup": 0,
        "activity": [],
        "macro": steps
      };

      $scope.addMacro.fromDeviceLayout.jsonData = JSON.stringify(angular.extend({}, jsonData));
      $state.go('menu.advanced.device.edit-layout', $scope.addMacro.fromDeviceLayout);

      return;
    }
    // Integrate with an External Service 메뉴에서 넘어온 경우
    else if ($scope.addMacro.extService) {
      var steps = angular.copy($scope.addMacro.steps);
      steps = Macro.cleanUp(steps);

      $scope.addMacro.extService.scene.macro = steps;

      $state.go('menu.advanced.external-service.complete', {extService: JSON.stringify($scope.addMacro.extService)});

      return;
    }

		if ($scope.macroType.id === 3) {
			$state.go('menu.advanced.add-macros.macro-schedule');
		} else {
			$state.go('menu.advanced.add-macros.macro-done');
		}
	};

	$scope.refreshScenes = function(device) {
		Devices.updateTwowayDevice($scope.addMacro.room.id, device.id)
			.then(function(data) {
				console.log(data);

				var item = _.where($scope.addMacro.room.allDevices, {'id': device.id})[0];
				item.data = data;
			}, function() {
				var modal = $modal({
					scope: $scope,
					title: "Couldn't refresh the list",
					container: '.editor-container',
					template: 'views/modal-tpls/refresh-the-list.html',
					show: false
				});

				modal.$promise.then(modal.show);
			})
			.catch(toastr.error);
	};

});

/**
 * @ngdoc controller
 * @name MacroDoneController
 * @description
 *  macro 설정 완료 화면
 */
app.controller('MacroDoneController', function ($rootScope, $scope, $state, toastr, Systems, Macro) {
  $scope.powerOffBtnOption = $rootScope.env.powerOffBtnOption;

	$scope.addMacro.doneScreen.index = 0;

	$scope.goBack = function() {
		$state.go('menu.advanced.add-macros.finish');
	};

	$scope.doNext = function() {
    if ($scope.addMacro.disablePopup == '0') {
			var steps =  angular.copy($scope.addMacro.steps);
			steps = Macro.cleanUp(steps);
			console.log('steps=', steps);

			//var autoWait = $scope.addMacro.autoWait ? 1:0;
			var jsonData = JSON.stringify( { auto_wait: 1 , edit_type: 1, popup: 0, activity: [], macro: steps } );
			console.log('jsonData=', jsonData);

			var handleSuccess = function(response) {
				if(response.data.result === true) {
					$state.go('menu.advanced.add-macros.next-action');
				} else {
					toastr.error('Your edit/create action failed.  Please try again.');
				}
			};

			var handleError = function(error){
				toastr.error("Please try again");
			};

      // twoway device 인지 검사
      var roomDevices = Systems.getRoomDevices(),
        isTwowayDevice = false;
      if ((_.where(roomDevices.twoway_devices, {
          id: $scope.addMacro.currentDevice.id
        })).length) {
        isTwowayDevice = true;
      }

			if ($scope.macroType.id === 0) {
				// 해당 room 의 power macro 데이터 저장
				Systems.storePowerMacro($scope.addMacro.room.id, jsonData).then(handleSuccess, handleError);
			} else if ($scope.macroType.id === 1) {
				// 해당 device ID 와 연계된 button 의 데이터 저장
        Systems.storeDeviceButtonMacro($scope.addMacro.room.id, $scope.addMacro.currentDevice.id, jsonData, isTwowayDevice).then(handleSuccess, handleError);
			} else if ($scope.macroType.id === 2) {
				jsonData = { "text": $scope.addMacro.button.name, "auto_wait": 1 , "edit_type": 1, "popup": 0, "activity": [], "macro": steps } ;

				// button 생성
				Systems.createCustomButtonMacro($scope.addMacro.room.id, $scope.addMacro.room.main_device_id, jsonData).then(handleSuccess, handleError);
			}
		} else {
			$state.go('menu.advanced.add-macros.macro-commands');
		}
	};

});

/**
 * @ngdoc controller
 * @name MacroScheduleController
 * @description
 *  macro schedule 설정 화면
 */
app.controller('MacroScheduleController', function ($scope, $state, toastr, Rooms, Systems, Macro) {
	var scheEventData = Rooms.getScheEventData();

	// scheduled event 수정하러 들어온 경우
	if (scheEventData) {
		$scope.scheduledEvent = {
			option: scheEventData.event.option,
			day_week: scheEventData.event.day_week,
			time: scheEventData.event.time
		};
	}

	var i,
		temp,
		temp2;

	if (!$scope.scheduledEvent.option) {
		$scope.scheduledEvent.option = 0;
	}
	if (!$scope.scheduledEvent.day_week) {
		$scope.scheduledEvent.day_week = 0;
	}
	if (!$scope.scheduledEvent.time) {
		$scope.scheduledEvent.hour = '01';
		$scope.scheduledEvent.minute = '00';
		$scope.scheduledEvent.ampm = 'AM';
	} else {
		temp = $scope.scheduledEvent.time.split(':');

		if (+temp[0] > 12) {
			$scope.scheduledEvent.hour = '' + (+temp[0] - 12);
			if (+$scope.scheduledEvent.hour < 10) {
				$scope.scheduledEvent.hour = '0' + $scope.scheduledEvent.hour;
			}

			$scope.scheduledEvent.ampm = 'PM';
		} else if (+temp[0] == 12) {
			$scope.scheduledEvent.hour = '' + temp[0];
			$scope.scheduledEvent.ampm = 'PM';
		} else if (+temp[0] == 0) {
			$scope.scheduledEvent.hour = '12';
			$scope.scheduledEvent.ampm = 'AM';
		} else {
			$scope.scheduledEvent.hour = '' + temp[0];
			$scope.scheduledEvent.ampm = 'AM';
		}
		$scope.scheduledEvent.minute = '' + temp[1];
	}

	$scope.scheduledEvent.hours = [];
	$scope.scheduledEvent.minutes = [];

	for (i = 1; i <= 12; i++) {
		if (i < 10) {
			$scope.scheduledEvent.hours.push('0' + i)
		} else {
			$scope.scheduledEvent.hours.push('' + i);
		}
	}

	for (i = 0; i < 60; i++) {
		if (i < 10) {
			$scope.scheduledEvent.minutes.push('0' + i)
		} else {
			$scope.scheduledEvent.minutes.push('' + i);
		}
	}

	$scope.scheduledEvent.day = {
		'sun': 0,
		'mon': 0,
		'tue': 0,
		'wed': 0,
		'thu': 0,
		'fri': 0,
		'sat': 0
	};

	if ($scope.scheduledEvent.day_week !== 0) {
		temp2 = +$scope.scheduledEvent.day_week;

		if (temp2 === 127) {
			$scope.scheduledEvent.everyday = true;
		} else {
			if (temp2 >= 64) {
				$scope.scheduledEvent.day.sat = '64';
				temp2 -= 64;
			}
			if (temp2 >= 32) {
				$scope.scheduledEvent.day.fri = '32';
				temp2 -= 32;
			}
			if (temp2 >= 16) {
				$scope.scheduledEvent.day.thu = '16';
				temp2 -= 16;
			}
			if (temp2 >= 8) {
				$scope.scheduledEvent.day.wed = '8';
				temp2 -= 8;
			}
			if (temp2 >= 4) {
				$scope.scheduledEvent.day.tue = '4';
				temp2 -= 4;
			}
			if (temp2 >= 2) {
				$scope.scheduledEvent.day.mon = '2';
				temp2 -= 2;
			}
			if (temp2 >= 1) {
				$scope.scheduledEvent.day.sun = '1';
				temp2 -= 1;
			}
		}
	}

	function generateTime() {
		if ($scope.scheduledEvent.ampm == 'PM' && $scope.scheduledEvent.hour == '12') {
			return '12:' + $scope.scheduledEvent.minute + ':00';
		} else if ($scope.scheduledEvent.ampm == 'AM' && $scope.scheduledEvent.hour == '12') {
			return '00:' + $scope.scheduledEvent.minute + ':00';
		} else if ($scope.scheduledEvent.ampm == 'PM') {
			return (+$scope.scheduledEvent.hour + 12) + ':' + $scope.scheduledEvent.minute + ':00';
		} else {
			return $scope.scheduledEvent.hour + ':' + $scope.scheduledEvent.minute + ':00';
		}
	}

	$scope.goBack = function() {
		$state.go('menu.advanced.add-macros.finish');
	};

	$scope.doNext = function() {
		$scope.scheduledEvent.day_week = 0;

		var days = _.values($scope.scheduledEvent.day);

		if ($scope.scheduledEvent.everyday) {
			$scope.scheduledEvent.day_week = 127;
		} else {
			_.each(days, function(day) {
				$scope.scheduledEvent.day_week += +day;
			});
		}

		$scope.scheduledEvent.time = generateTime();

		var steps =  angular.copy($scope.addMacro.steps);
		steps = Macro.cleanUp(steps);
		console.log('steps=', steps);

		var	systemId = Systems.getCurrentSystem().id,
			data = {
				display_text: $scope.addMacro.button.name,
				image_name: '' ,
				macro: steps,
				default_state: 1,
				option: $scope.scheduledEvent.option,
				day_week: $scope.scheduledEvent.day_week,
				time: $scope.scheduledEvent.time
			},
			jsonData;

		if (scheEventData) {
			data.id = scheEventData.event.id;
		}

		jsonData = JSON.stringify( data )
		console.log('jsonData=', jsonData);

		// scheduled event 수정
		if (scheEventData) {
			Rooms.setScheduledEventMacro(systemId, $scope.addMacro.room.id, jsonData).then(function(response) {
				if(response.data.result === true) {
					Systems.updateSystem(systemId).then(function () {
						Rooms.setScheEventData(null);
						$state.go('menu.advanced.add-macros.next-action');
					});
				} else {
					toastr.error('Your edit/create action failed.  Please try again.');
				}
			}, function(error){
				toastr.error("Please try again");
			});
		}
		// scheduled event 생성
		else {
			Rooms.createScheduledEventMacro(systemId, $scope.addMacro.room.id, jsonData).then(function(response) {
				if(response.data.result === true) {
					Systems.updateSystem(systemId).then(function () {
						Rooms.setScheEventData(null);
						$state.go('menu.advanced.add-macros.next-action');
					});
				} else {
					toastr.error('Your edit/create action failed.  Please try again.');
				}
			}, function(error){
				toastr.error("Please try again");
			});
		}
	};

});


app.controller('MacroCommandsController', function ($scope, $state, toastr, Systems, Macro) {

	$scope.goBack = function() {
		if ($scope.addMacro.doneScreen.index === 0) {
			$state.go('menu.advanced.add-macros.macro-done' );
		} else {
			$scope.addMacro.doneScreen.index--;
		}
	};

	$scope.doNext = function() {
    var len = $scope.addMacro.doneScreen.devices.length;
		if ($scope.addMacro.doneScreen.index + 1 === len) {

			//var autoWait = $scope.addMacro.autoWait ? 1:0;
			var steps =  angular.copy($scope.addMacro.steps);
			steps = Macro.cleanUp(steps);
			console.log('steps=', steps);

			var activityData = angular.copy($scope.addMacro.doneScreen.devices);

			_.forEach(steps, function(step) { delete step.name; });
			_.forEach(activityData, function(activity) {
				activity.func = activity.func.id;
				delete activity.name;
				delete activity.repeat_min;
				delete activity.repeat_macro;
				delete activity.target_type;
				delete activity.target_addr;
				delete activity.target_port;
				delete activity.funcs;
			});

      var jsonData = JSON.stringify({
        auto_wait: 1,
        edit_type: 1,
        popup: +$scope.addMacro.disablePopup,
        activity: activityData,
        macro: steps
      });
			console.log(jsonData);

			var handleSuccess = function(response) {
				if(response.data.result === true) {
					$state.go('menu.advanced.add-macros.next-action');
				} else {
					toastr.error('Your edit/create action failed.  Please try again.');
				}
			};

			var handleError = function(error){
				toastr.error("Please try again");
			};

      // twoway device 인지 검사
      var roomDevices = Systems.getRoomDevices(),
        isTwowayDevice = false;
      if ((_.where(roomDevices.twoway_devices, {
          id: $scope.addMacro.currentDevice.id
        })).length) {
        isTwowayDevice = true;
      }

			if ($scope.macroType.id === 0) {
				// 해당 room 의 power macro 데이터 저장
				Systems.storePowerMacro($scope.addMacro.room.id, jsonData).then(handleSuccess, handleError);
			} else if ($scope.macroType.id === 1) {
				// 해당 device ID 와 연계된 button 의 데이터 저장
        Systems.storeDeviceButtonMacro($scope.addMacro.room.id, $scope.addMacro.currentDevice.id, jsonData, isTwowayDevice).then(handleSuccess, handleError);
			} else if ($scope.macroType.id === 2) {
        jsonData = JSON.stringify({
          text: $scope.addMacro.button.name,
          auto_wait: 1,
          edit_type: 1,
          popup: +$scope.addMacro.disablePopup,
          activity: activityData,
          macro: steps
        });

				// button 생성
				Systems.createCustomButtonMacro($scope.addMacro.room.id, $scope.addMacro.room.main_device_id, jsonData).then(handleSuccess, handleError);
			}
		} else {
			$scope.addMacro.doneScreen.index++;
		}

	};
});


'use strict';
/**
 * @ngdoc controller
 * @name SelectRemoteController
 * @description
 *  remote 목록을 표시하고 선택 후 remote 를 room 에 set 하는 화면
 */
app.controller('SelectRemoteController', function ($rootScope, $scope, $filter, $state, searchResults, types, toastr, Systems, Rooms) {
  $scope.searchKey = searchResults.searchKey;
  $scope.remotes = searchResults.searchedDevices;
  $scope.types = types;

  _.each($scope.remotes, function (remote) {
    _.each(types, function (type) {
      if (remote.name === type.name) {
        remote.type = type.type;
      }
    });
  });

  $scope.remoteType = 'CLIENTS';

  $scope.touchscreen = $rootScope.env.touchscreen;

  $scope.goBack = function () {
    $state.go('menu.programming.options');
  };

  $scope.invalid = true;
  $scope.$watch('addRemote.selection', function (newVal, oldVal) {
    if (newVal) {
      $scope.invalid = false;
    } else {
      $scope.invalid = true;
    }
  });
  $scope.$watch('addRemote.selections', function (newVal, oldVal) {
    $scope.invalid = true;
    _.each($scope.addRemote.selections, function (mac) {
      if (mac) {
        $scope.invalid = false;
      }
    });
  }, true);

  $scope.doNext = function () {

    var currentSystem = Systems.getCurrentSystem(),
        remote = $scope.addRemote.selection,
      jsonData;
    if (remote) {
      jsonData = JSON.stringify({
        mac: remote.mac,
        name: remote.name + ' ' + remote.mac.slice(10, 12),
        type: remote.name,
        searchKey: $scope.searchKey
      });
    }

    // remote 를 room 에 add
    var addRemote = function (roomId, json) {
      return Rooms.addRemote(roomId, json).then(function (data) {
        //toastr.success('Successfully added the remote with ID ' + $filter('remoteId')(remote.mac) +  ' to ' + $scope.addRemote.room.name + '.');
        return Systems.getCurrentSystem().id;
      });
    };

    // room 에서 remote 를 delete
    var delRemote = function (roomId, json) {
      return Rooms.delRemote(roomId, json).then(function (data) {
        toastr.success('Successfully deleted the remote with ID ' + $filter('remoteId')(remote.mac) + ' to ' + $scope.addRemote.room.name + '.');
        return Systems.getCurrentSystem().id;
      });
    };

    var updateSystem = function(id) {
      return Systems.updateSystem(id).then(function (data) {
        console.log(data);
        return $state.transitionTo('menu.programming.options', {}, { reload: true, inherit: true, notify: true  });
      });
    };

    // 기존에 Remote Control type 이 있고, 새로 추가되는 것에 Remote Control type 이 있는 경우에는 기존의 Remote Control type 의 device 를 먼저 삭제
    var id = Systems.getCurrentRoom(),
      room = _.find(currentSystem.rooms, {'id': id});

    var hasRemoteControlType = null;   // 기존에 Remote Control type 이 있는 경우 해당 device 의 mac

    _.each(room.remotes, function (remote) {
      var obj = _.find(types, {
        name: remote.type
      });
      if (obj.type === 'Remote Control') {
        hasRemoteControlType = remote.mac;
      }
    });

    // 새로 추가되는 것에 Remote Control type 이 있는 경우
    if ($scope.addRemote.selection) {
      // remote 추가
      addRemote($scope.addRemote.room.id, jsonData)
      .then(updateSystem)
      .catch(function (error) {
        toastr.error(error, 'Error');
      });
    }

    // 새로 추가되는 것에 Touchscreen type 이 있는 경우
    _.each($scope.addRemote.selections, function (mac) {
      if (mac) {
        var remote = _.find($scope.remotes, {
          mac: mac
        });

        addRemote($scope.addRemote.room.id, JSON.stringify({
          mac: remote.mac,
          name: remote.name + ' ' + remote.mac.slice(10, 12),
          type: remote.name,
          searchKey: $scope.searchKey
        }))
          .then(updateSystem)
          .catch(function (error) {
            toastr.error(error, 'Error');
          });
      }
    });
  };

});


// app.controller('AvailableRoomsController', function ($scope, $filter, $state, toastr, Systems, Rooms) {

//   $scope.system = Systems.getCurrentSystem();

//   $scope.doNext = function () {

//     var remote = $scope.addRemote.selection,
//         jsonData = JSON.stringify({ mac : remote.mac, name: remote.name + ' ' + remote.mac.slice(10,12) , type: remote.name });

//     var setRemote = function(roomId, json) {
//       return Rooms.setRemote(roomId, json).then(function(data) { 
//         toastr.success('Successfully added the remote with ID ' + $filter('remoteId')(remote.mac) +  ' to ' + $scope.addRemote.room.name + '.');
//         return Systems.getCurrentSystem().id;
//       });
//     };

//     var updateSystem = function(id) {
//       return Systems.updateSystem(id).then(function (data) {
//         console.log(data);
//         return $state.transitionTo('menu.advanced.options', {}, { reload: true, inherit: true, notify: true  });
//       });
//     }

//     setRemote($scope.addRemote.room.id, jsonData)
//       .then(updateSystem)
//       .catch(function(error) {
//         toastr.error(error, 'Error');
//       });
//   };

// });

'use strict';
/**
 * @ngdoc controller
 * @name SelectRoomController
 * @description
 *  base station 목록 표시 후 base station 선택과 함께 room 생성 화면
 */
app.controller('SelectRoomController', function ($rootScope, $scope, $state, $modal, toastr, searchResults, Systems, Rooms) {
  var currentSystem = Systems.getCurrentSystem();
  $scope.searchKey = searchResults.searchKey;
  $scope.addRoom = {'name': null, 'baseStation': null};
  $scope.baseStations = searchResults.searchedDevices;
  var virtualBS = {
    name: 'Virtual Base Station',
    mac: '000000000000',
  };

  $scope.baseStations.push(virtualBS);

  $scope.bsType = 'BASESTATIONS';

  $scope.finish = function() {
    var baseStation = $scope.addRoom.baseStation;
    var bstationName = baseStation.name + (baseStation.mac == '000000000000' ? '' : ' ' + baseStation.mac.slice(10,12));
    var bstationType = (baseStation.mac == '000000000000' ? 'VIRTUAL' : baseStation.name);
    var jsonData = JSON.stringify({ name: $scope.addRoom.name, 
                                    bstationName: bstationName, 
                                    bstationMac: baseStation.mac, 
                                    type: bstationType });

    // room 생성
    var createRoom = function(json) {
      return Rooms.create(json).then(function(data) { 
        toastr.success('Successfully added ' + $scope.addRoom.name + '.');
        return Systems.getCurrentSystem().id;
      });
    };

    var updateSystem = function(id) {
      return Systems.updateSystem(id).then(function (data) {
        console.log(data);
        //return $state.transitionTo('menu.advanced.add-room.link-remote', {}, { reload: true, inherit: true, notify: true  });
        $state.transitionTo('menu.advanced.options', {}, {reload: true, inherit: true, notify: true});
      });
    };

    createRoom(jsonData)
      .then(updateSystem)
      .catch(toastr.error);

  };

  // system 의 master base station 보다 bstationPriority 가 높은 base station 을 선택할 경우 팝업화면을 띄움
  var masterBstationType,
    masterBstationPriority,
    selectedBstationPriority;
  _.each(currentSystem.rooms, function (room) {
    _.each(room.bstations, function (bstation) {
      if (bstation.master) {
        masterBstationType = bstation.type;
      }
    })
  });
  _.each($rootScope.env.bstationPriority, function (value, key) {
    if (value.indexOf(masterBstationType) > -1) {
      masterBstationPriority = key;
    }
  });
  $scope.$watch('addRoom.baseStation', function (newVal, oldVal) {
    if (newVal) {
      _.each($rootScope.env.bstationPriority, function (value, key) {
        if (value.indexOf(newVal.name) > -1) {
          selectedBstationPriority = key;
        }
      });

      if (selectedBstationPriority > masterBstationPriority) {
        var modal = $modal({
          scope: $scope,
          title: '',
          placement: 'top',
          template: 'views/modal-tpls/new-base-station.html',
          show: false
        });

        modal.$promise.then(modal.show);
      }
    }
  });
});

app.controller('LinkRemoteController', function ($scope, $state, Systems ) {
  
  $scope.doNext = function() {
    $state.go('menu.advanced.add-remote.select');
  };
});

'use strict';
/**
 * @ngdoc controller
 * @name DbEditController
 * @description
 *  db edit 화면
 */
app.controller('DbEditController', function($scope, $state, $sanitize, $modal, $timeout, toastr, categories, Systems) {
  $scope.tabs = [
    { heading: $sanitize('<i class="fa fa-laptop tab-icon"></i><span class="tab-title">Programming Menu</span>'), route:'menu.programming.options', active:true },
    { heading: $sanitize('<i class="fa fa-wrench tab-icon"></i><span class="tab-title">Advanced Menu</span>'), route:'menu.advanced.options', active:false },
    { heading: $sanitize('<i class="fa fa-globe tab-icon"></i><span class="tab-title">Home Overview</span>'), route:'menu.system-overview', active:false },
    { heading: $sanitize('<i class="fa fa-download tab-icon"></i><span class="tab-title">Download</span>'), route:'menu.download', active:false }
  ];

  $scope.go = function(route){
    // programming 메뉴로는 이동할 필요 없음, 로딩시 programming 메뉴가 active tab 이라서 자동으로 이동하는 것을 막기 위해 예외 처리.
    if (route !== 'menu.programming.options') {
      $state.go(route);
    }
  };

  $scope.dbEdit = {};
  $scope.dbEdit.categories = categories;
  $scope.dbEdit.versions = [
    {
      name: 'Alpha',
      value: 3
    },
    {
      name: 'Beta + Alpha',
      value: 2
    },
    {
      name: 'Public + Beta + Alpha',
      value: 0
    }
  ];

  $scope.dbEdit.category = null;
  $scope.dbEdit.brand = null;
  $scope.dbEdit.model = null;

  $scope.dbEdit.newCategory = {
    id: null,
    name: null
  };
  $scope.dbEdit.newBrand = {
    id: null,
    name: null
  };
  $scope.dbEdit.newModel = {
    id: null,
    name: null
  };

  $scope.selectCategory = function(category) {
    $scope.dbEdit.brand = null;
    $scope.dbEdit.model = null;

    Systems.getTwowayBrands(category.id).then(function(response) {
      $scope.dbEdit.brands = response;
    }, function(error) {
      toastr.error(error);
    });
  };

  $scope.selectBrand = function(brand) {
    $scope.dbEdit.model = null;

    Systems.getTwowayModels(brand.id).then(function(response) {
      $scope.dbEdit.models = response;
    }, function(error) {
      toastr.error(error);
    });
  };

  $scope.selectModel = function(model) {
    //console.log(model);
  };

  $scope.addCategory = function() {
    var newCategory = angular.copy($scope.dbEdit.newCategory);
    if (!$scope.dbEdit.categories || !$scope.dbEdit.categories.length) {
      $scope.dbEdit.categories = [newCategory];
    } else {
      $scope.dbEdit.categories.push(newCategory);
    }
    $scope.dbEdit.category = newCategory;

    $scope.dbEdit.newCategory = {
      id: null,
      name: null
    };
  };

  $scope.addBrand = function() {
    var newBrand = angular.copy($scope.dbEdit.newBrand);
    if (!$scope.dbEdit.brands || !$scope.dbEdit.brands.length) {
      $scope.dbEdit.brands = [newBrand];
    } else {
      $scope.dbEdit.brands.push(newBrand);
    }
    $scope.dbEdit.brand = newBrand;

    $scope.dbEdit.newBrand = {
      id: null,
      name: null
    };
  };

  $scope.addModel = function() {
    var newModel = angular.copy($scope.dbEdit.newModel);
    if (!$scope.dbEdit.models || !$scope.dbEdit.models.length) {
      $scope.dbEdit.models = [newModel];
    } else {
      $scope.dbEdit.models.push(newModel);
    }
    $scope.dbEdit.model = newModel;

    $scope.dbEdit.newModel = {
      id: null,
      name: null
    };
  };

  $scope.getDbTwowayModel = function() {
    var data = {
        categoryId: $scope.dbEdit.category.id,
        category: $scope.dbEdit.category.name,
        brandId: $scope.dbEdit.brand.id,
        brand: $scope.dbEdit.brand.name,
        modelId: $scope.dbEdit.model.id,
        model: $scope.dbEdit.model.name
      },
      jsonData = JSON.stringify(data);

    Systems.getDbTwowayModel(jsonData).then(function(response) {
      $scope.dbEdit.version = _.find($scope.dbEdit.versions, function(obj) {
        return obj.value === response.version;
      });
      $scope.dbEdit.major = response.major;

      var obj = angular.copy(response);
      delete(obj.major);
      delete(obj.name);
      delete(obj.version);

      $scope.dbEdit.data = JSON.stringify(obj);

      toastr.success('Success: Get');
    }, function(error) {
      toastr.error(error);
    });
  };

  $scope.addDbTwowayModel = function() {
    try {
      var parsedData = JSON.parse($scope.dbEdit.data);
      if (!parsedData.variables) {
        parsedData.variables = "";
      }

      var data = {
          categoryId: $scope.dbEdit.category.id,
          category: $scope.dbEdit.category.name,
          brandId: $scope.dbEdit.brand.id,
          brand: $scope.dbEdit.brand.name,
          modelId: $scope.dbEdit.model.id,
          model: $scope.dbEdit.model.name,
          data: JSON.stringify(parsedData),
          version: $scope.dbEdit.version.value,
          major: $scope.dbEdit.major || 0
        },
        jsonData = JSON.stringify(data);

      Systems.addDbTwowayModel(jsonData).then(function(response) {
        toastr.success('Success: Add New');
      }, function(error) {
        toastr.error(error);
      });
    } catch(e) {
      toastr.error('Module data is invalid. Please check JSON format.');
    }
  };

  $scope.setDbTwowayModel = function() {
    try {
      var parsedData = JSON.parse($scope.dbEdit.data);
      if (!parsedData.variables) {
        parsedData.variables = "";
      }

      var data = {
          categoryId: $scope.dbEdit.category.id,
          category: $scope.dbEdit.category.name,
          brandId: $scope.dbEdit.brand.id,
          brand: $scope.dbEdit.brand.name,
          modelId: $scope.dbEdit.model.id,
          model: $scope.dbEdit.model.name,
          data: JSON.stringify(parsedData),
          version: $scope.dbEdit.version.value,
          major: $scope.dbEdit.major || 0
        },
        jsonData = JSON.stringify(data);

      Systems.setDbTwowayModel(jsonData).then(function(response) {
        toastr.success('Success: Set');
      }, function(error) {
        toastr.error(error);
      });
    } catch(e) {
      toastr.error('Module data is invalid. Please check JSON format.');
    }
  };

  $scope.showModal = function(mode) {
    var templateUrl = 'views/modal-tpls/db-edit/' + mode + '.html';

    var modal = $modal({
      scope: $scope,
      template: templateUrl,
      container: 'body',
      show: false
    });
    modal.$promise.then(modal.show);
  };

  $scope.clearAll = function() {
    $scope.dbEdit.brands = [];
    $scope.dbEdit.models = [];

    $scope.dbEdit.category = null;
    $scope.dbEdit.brand = null;
    $scope.dbEdit.model = null;
    $scope.dbEdit.version = null;
    $scope.dbEdit.major = 0;
    $scope.dbEdit.data = null;

    $scope.dbEdit.newCategory = {
      id: null,
      name: null
    };
    $scope.dbEdit.newBrands = {
      id: null,
      name: null
    };
    $scope.dbEdit.newModels = {
      id: null,
      name: null
    };

    toastr.success('Success: Clear All');
  };
});

'use strict';


app.controller('AddToDealerController', function ($rootScope, $scope, $state, $modal, toastr, currentSystem, Dealer, Systems, User) {

  $scope.setDealer = function() {
    var systemId = currentSystem.id;

    var jsonData = JSON.stringify($scope.credentials);

    Systems.setDealer(systemId, jsonData)
      .then( function(data) {
        if(data.result) {
          $rootScope.dealerCompany = data.company;

          var modal = $modal({
            scope: $scope,
            title: '',
            placement: 'top',
            template: 'views/modal-tpls/dealer/dealer-system-added.html',
            backdrop: 'static',
            show: false
          });

          modal.$promise.then(modal.show);
        } else {
          toastr.error(data.error);
        }
      })
      .catch(toastr.error);

    // 일반 사용자에서 logout 하고 delaer 로 로그인 후 system 선택 후 home overview 화면으로 이동
    // system 선택 후 home overview 화면으로 이동하는 코드는 DealerJobsListController 의 $scope.selectSystem function 과 같음
    $scope.dealerLogin = function () {
      User.logout();

      Dealer.login(jsonData)
        .then(function (data) {
          if (data.result) {
            // dealer company 불러오기
            $rootScope.dealerCompany = Dealer.getDealerCompany();

            Dealer.getSystemToken(JSON.stringify({system_id: systemId})).then(function (data) {
              if (data.result) {
                // token 갱신
                var token = data.token;
                Dealer.setUser(systemId, token, false, false, true);   // dealer token 보존

                User.getUserInfo().then(function (data) {
                  // token 갱신
                  Dealer.setUser(systemId, token, data.username, data.email);

                  // system select
                  Systems.setCurrentRoom(null);

                  Systems.updateSystem(systemId).then(
                    function () {
                      $state.go('menu.system-overview');
                    },
                    function (error) {
                      toastr.error(error);
                    }
                  );
                }).catch(toastr.error);
              } else {
                toastr.error(data.error);
              }
            })
              .catch(toastr.error);
          } else {
            toastr.error(data.error);
          }
        })
        .catch(toastr.error);
    };
  };

  $scope.forgot = function() {
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/dealer/dealer-forgot-password.html',
      backdrop: 'static',
      show: false
    });

    modal.$promise.then(modal.show);
  };
});

app.controller('DealerJobsListController', function ($rootScope, $scope, $state, $modal, toastr, currentUser, jobsList, Systems, Dealer, User) {
  $rootScope.virtualEditor = null;

  $scope.currentUser = currentUser;
  $scope.jobsList = jobsList;
  $scope.selectedJob = null;

  $scope.viewDetails = function(job) {
    $scope.selectedJob = null;

    var jsonData = JSON.stringify({system_id: job.id});

    Dealer.getJob(jsonData).then(function(data) {
      if(data.result) {
        $scope.selectedJob = data.job;
        console.log($scope.selectedJob);

        var modal = $modal({
          scope: $scope,
          title: '',
          placement: 'top',
          template: 'views/modal-tpls/dealer/dealer-customer-details.html',
          show: false
        });

        modal.$promise.then(modal.show);
      } else {
        toastr.error(data.error);
      }
    })
    .catch(toastr.error);
  };

  // system 을 선택할 경우, 해당 system 의 user 권한으로 작업을 수행하기 위해 그 user 의 token 으로 업데이트함
  $scope.selectSystem = function() {
    if (!$scope.selectedJob) {
      return;
    }

    var jsonData = JSON.stringify({system_id: $scope.selectedJob.id});

    Dealer.getSystemToken(jsonData).then(function(data) {
      if(data.result) {
        // token 갱신
        var token = data.token;
        Dealer.setUser($scope.selectedJob.id, token, false, false, true);   // dealer token 보존

        User.getUserInfo().then(function(data) {
          // token 갱신
          Dealer.setUser($scope.selectedJob.id, token, data.username, data.email);

          // system select
          Systems.setCurrentRoom(null);

          Systems.updateSystem($scope.selectedJob.id).then(
            function () {
              $state.go('menu.system-overview');
            },
            function (error) {
              toastr.error(error);
            }
          );
        }).catch(toastr.error);
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  $scope.detachJobModal = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/dealer/dealer-detach-job.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  // 해당 system을 dealer의 job목록에서 제거하여 사용자가 직접 편집할 수 있게 함
  $scope.detachJob = function () {
    var jsonData = JSON.stringify({system_id: $scope.selectedJob.id});

    Dealer.detachJob(jsonData).then(function (data) {
      if (data.result) {
        $state.go('dealer-jobs-list', {}, {reload: true});
        $scope.selectedJob = null;
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

});

app.controller('FileExplorerController', function ($scope, $state, $modal, toastr, Systems) {

  

});

app.controller('ExportTemplateChooseRoomController', function ($scope, $state, $modal, toastr, Systems) {

  $scope.doNext = function() {
    $state.go('file-explorer.export-template');
  };

});

app.controller('ExportTemplateController', function ($scope, $state, $modal, toastr, Systems, Dealer) {
  
  $scope.showHelpModal = function() {
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/dealer/export-template-help.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };
  
  $scope.finish = function() {
    var jsonData = JSON.stringify({
      roomId: $scope.exportTemplate.room.id,
      name: $scope.exportTemplate.filename
    });

    Dealer.createFromExisting(jsonData).then(function(data) {
      if (data.result) {
        var modal = $modal({
          scope: $scope,
          title: '',
          content: 'This room template has been exported to your Dealer Account File Explorer.',
          placement: 'top',
          template: 'views/modal-tpls/dealer/file-explorer.html',
          backdrop: 'static',
          show: false
        });

        modal.$promise.then(modal.show);
      }
    })
      .catch(toastr.error);
  };

});

app.controller('ExportDeviceChooseRoomController', function ($scope, $state, $modal, toastr, Systems) {

  $scope.doNext = function() {
    $state.go('file-explorer.export-device-choose-device');
  };

});

app.controller('ExportDeviceChooseDeviceController', function ($scope, $state, $modal, toastr, Systems) {

  $scope.doNext = function() {
    $state.go('file-explorer.export-device');
  };

});

app.controller('ExportDeviceController', function ($rootScope, $scope, $state, $modal, toastr, Systems, Dealer) {

  $scope.finish = function() {
    var jsonData = JSON.stringify({
      roomId: $scope.exportDevice.room.id,
      deviceId: $scope.exportDevice.device.id,
      name: $scope.exportDevice.filename
    });

    if ($rootScope.virtualEditor) {
      Dealer.customDevCreateFromTemplate(jsonData).then(function (data) {
        if (data.result) {
          var modal = $modal({
            scope: $scope,
            title: '',
            content: 'This device file has been exported to your Dealer Account File Explorer.',
            placement: 'top',
            template: 'views/modal-tpls/dealer/file-explorer.html',
            backdrop: 'static',
            show: false
          });

          modal.$promise.then(modal.show);
        }
      })
        .catch(toastr.error);
    } else {
      Dealer.customDevCreate(jsonData).then(function (data) {
        if (data.result) {
          var modal = $modal({
            scope: $scope,
            title: '',
            content: 'This device file has been exported to your Dealer Account File Explorer.',
            placement: 'top',
            template: 'views/modal-tpls/dealer/file-explorer.html',
            backdrop: 'static',
            show: false
          });

          modal.$promise.then(modal.show);
        }
      })
        .catch(toastr.error);
    }
  };

});

app.controller('ImportTemplateController', function ($rootScope, $scope, $state, $modal, toastr, templates, Systems, Dealer) {

  $scope.importTemplate.templates = templates;

  $scope.copyFromTemplate = function () {
    var jsonData = JSON.stringify({roomId: $scope.importTemplate.template.id});

    Dealer.copyFromTemplate(Systems.getCurrentRoom(), jsonData).then(function (data) {
      if (data.result) {
        console.log(data);

        var modal = $modal({
          scope: $scope,
          title: '',
          placement: 'top',
          template: 'views/modal-tpls/dealer/import-template-finish-virtual-editor.html',
          backdrop: 'static',
          show: false
        });

        modal.$promise.then(modal.show);
      }
    })
      .catch(toastr.error);
  };

  $scope.finish = function () {
    if ($rootScope.virtualEditor) {
      $state.go('menu.programming.options');
    }
  };

  $scope.select = function(template) {
    $scope.importTemplate.template = template;

    if ($rootScope.virtualEditor) {
      var modal = $modal({
        scope: $scope,
        title: '',
        placement: 'top',
        template: 'views/modal-tpls/dealer/import-template-warning-virtual-editor.html',
        backdrop: 'static',
        show: false
      });

      modal.$promise.then(modal.show);
    } else {
      $state.go('file-explorer.import-template-choose-room');
    }
  };

  $scope.delete = function(template) {
    $scope.importTemplate.template = template;

    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/dealer/delete-item.html',
      backdrop: 'static',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.deleteItem = function() {
    var currentRoomDelete = false;
    // 현재 load 된 room 을 삭제한 경우
    if ($scope.importTemplate.template.id === Systems.getCurrentRoom()) {
      currentRoomDelete = true;
    }

    Dealer.deleteTemplate($scope.importTemplate.template.id).then(function(data) {
      if (data.result) {
        $scope.importTemplate.template = null;

        Dealer.getTemplates().then(
          function(response) {
            $scope.importTemplate.templates = response.templates;

            if (currentRoomDelete) {
              $state.go('dealer-jobs-list');
            }
          },
          function(error) {
            toastr.error(error);
          }
        );
      }
    })
      .catch(toastr.error);
  };

});

app.controller('ImportTemplateChooseRoomController', function ($rootScope, $scope, $state, $modal, toastr, Systems, Dealer) {

  $scope.importTemplateApplyToExisting = function() {
    var jsonData;

    if ($rootScope.virtualEditor) {
      jsonData = JSON.stringify({roomId: $scope.importTemplate.template.id});

      Dealer.copyFromTemplate(Systems.getCurrentRoom(), jsonData).then(function (data) {
        if (data.result) {
          console.log(data);

          var modal = $modal({
            scope: $scope,
            title: '',
            placement: 'top',
            template: 'views/modal-tpls/dealer/import-template-finish-virtual-editor.html',
            backdrop: 'static',
            show: false
          });

          modal.$promise.then(modal.show);
        }
      })
        .catch(toastr.error);
    } else {
      jsonData = JSON.stringify({roomId: $scope.importTemplate.room.id, name: $scope.importTemplate.template.name});

      Dealer.templateApplyToExisting($scope.importTemplate.template.id, jsonData).then(function (data) {
        if (data.result) {
          console.log(data);

          var modal = $modal({
            scope: $scope,
            title: '',
            placement: 'top',
            template: 'views/modal-tpls/dealer/import-template-finish.html',
            backdrop: 'static',
            show: false
          });

          modal.$promise.then(modal.show);
        }
      })
        .catch(toastr.error);
    }
  };
  
  $scope.finish = function() {
    if ($rootScope.virtualEditor) {
      $state.go('menu.programming.options');
    } else {
      Systems.updateSystem(Systems.getCurrentSystem().id).then(function () {
        Systems.setCurrentRoom($scope.importTemplate.room.id);
        $state.go('menu.programming.options');
      });
    }
  };
  
  $scope.doNext = function() {
    if ($scope.importTemplate.room.id === 'new') {
      $state.go('file-explorer.import-template-choose-bstation');
    } else {
      var modal = $modal({
        scope: $scope,
        title: '',
        placement: 'top',
        template: 'views/modal-tpls/dealer/import-template-warning.html',
        backdrop: 'static',
        show: false
      });

      modal.$promise.then(modal.show);
    }
  };

});

app.controller('ImportTemplateChooseBstationController', function ($scope, $state, $modal, toastr, searchResults, Systems, Dealer) {

  $scope.bsType = 'BASESTATIONS';
  $scope.baseStations = searchResults.searchedDevices;

  var virtualBS = {
    name: 'Virtual Base Station',
    mac: '000000000000',
  };
  $scope.baseStations.push(virtualBS);

  $scope.finish = function() {
    var bstationType = ($scope.importTemplate.baseStation.mac == '000000000000' ? 'VIRTUAL' : $scope.importTemplate.baseStation.name);
    var jsonData = JSON.stringify({
      systemId: Systems.getCurrentSystem().id,
      name: $scope.importTemplate.room.name,
      type: bstationType,
      bstationName: $scope.importTemplate.baseStation.name,
      bstationMac: $scope.importTemplate.baseStation.mac
    });

    Dealer.templateApplyToNew($scope.importTemplate.template.id, jsonData).then(function (data) {
      if (data.result) {
        console.log(data);

        var modal = $modal({
          scope: $scope,
          title: '',
          placement: 'top',
          template: 'views/modal-tpls/dealer/import-template-new-room-finish.html',
          backdrop: 'static',
          show: false
        });

        modal.$promise.then(modal.show);
      }
    })
      .catch(toastr.error);
  };

  $scope.doNext = function () {
    var currentSystem = Systems.getCurrentSystem();
    Systems.updateSystem(currentSystem.id).then(function () {
      Systems.setCurrentRoom(currentSystem.rooms[currentSystem.rooms.length - 1].id);   // 방금 추가한 room 을 선택
      $state.go('menu.programming.options');
    });
  };

});

app.controller('ImportDeviceController', function ($scope, $state, $modal, toastr, devices, Systems, Dealer) {

  $scope.importDevice.devices = devices;

  $scope.select = function(device) {
    $scope.importDevice.device = device;
    
    $state.go('file-explorer.import-device-choose-room', {deviceId: $scope.importDevice.device.id});
  };

  $scope.delete = function(device) {
    $scope.importDevice.device = device;
    
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/dealer/delete-item.html',
      backdrop: 'static',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.deleteItem = function() {
    Dealer.deleteDevice($scope.importDevice.device.id).then(function(data) {
      if (data.result) {
        $scope.importDevice.device = null;

        Dealer.getDevices().then(
          function(response) {
            $scope.importDevice.devices = response.devices;
          },
          function(error) {
            toastr.error(error);
          }
        );
      }
    })
      .catch(toastr.error);
  };

});

app.controller('ImportDeviceChooseRoomController', function ($rootScope, $scope, $state, $modal, device, toastr, Systems, Dealer) {
  _.extend($scope.importDevice.device, device);
  $scope.importDevice.device.target_type = $scope.importDevice.device.type;

  $scope.selections = $scope.importDevice;

  $scope.lightLevels = [ {'value': 0, 'label': 'Off'}, {'value': 1, 'label': 'Low'}, {'value': 2, 'label': 'Medium'}, {'value': 3, 'label': 'High'}];
  $scope.irPortOptions = [{name: 'All', value: 255}, {name: '1', value: 1}, {name: '2', value: 2}, {
    name: '3',
    value: 3
  }, {name: '4', value: 4}];

  $scope.ir = {target:0, port:255};



  $scope.editDevice = function() {
    var targetPort;

    if ($scope.selections.device.target_type === 1) {
      if ($scope.ir.target === 2) {
        targetPort = 0;
      } else {
        targetPort = $scope.ir.port;
      }
    } else {
      targetPort = $scope.selections.device.target_port;
    }

    var obj = {
      name: $scope.selections.device.device_name,
      targetType: $scope.selections.device.target_type,
      targetAddr: $scope.selections.device.target_addr,
      targetPort: targetPort,
      favEnable: 0,
      favDelay: 400,
      favEnter: 0,
      repeatMin: $scope.selections.device.repeat_min,
      repeatMacro: $scope.selections.device.repeat_macro,
      rampStart: $scope.selections.device.ramp_start * 1000,
      rampSpeed: $scope.selections.device.ramp_speed * 1000,
      roomId: $scope.selections.room.id
    };
    if (typeof $scope.selections.device.attr != 'undefined' && typeof $scope.selections.device.attr.auto_cr != 'undefined') {
      obj.autoCr = $scope.selections.device.attr.auto_cr;
    }

    var jsonData = JSON.stringify(obj);

    if ($rootScope.virtualEditor) {
      Dealer.deviceApplyToTemplate($scope.selections.device.id, jsonData).then(function (data) {
        if (data.result) {
          console.log(data);
          Systems.updateSystem(Systems.getCurrentSystem().id).then(function () {
            Systems.setCurrentRoom($scope.selections.room.id);
            $state.go('menu.programming.options');
          });
        }
      })
        .catch(toastr.error);
    } else {
      Dealer.deviceApply($scope.selections.device.id, jsonData).then(function (data) {
        if (data.result) {
          console.log(data);
          Systems.updateSystem(Systems.getCurrentSystem().id).then(function () {
            Systems.setCurrentRoom($scope.selections.room.id);
            $state.go('menu.programming.options');
          });
        }
      })
        .catch(toastr.error);
    }
  };

  $scope.cancelEdit = function() {

  };

  $scope.$watch('selections.device', function(newVal, oldVal) {
    $scope.selections.device.target_type = +newVal.target_type;
  }, true);

  var allBstations = [],
    currentSystem = Systems.getCurrentSystem();
  _.each(currentSystem.rooms, function (room) {
    allBstations = allBstations.concat(angular.copy(room.bstations));
  });
  $scope.selections.bstations = allBstations;
	//virtual base station을 제외시켜야 함.
  var idx = _.findIndex($scope.selections.bstations, {'type': 'VIRTUAL'});
	while(0 <= idx) {
    $scope.selections.bstations.splice(idx, 1);
		idx = _.findIndex($scope.selections.bstations, {'type': 'VIRTUAL'});
	}

  $scope.doNext = function() {
    var modal = $modal({
      scope: $scope,
      title: 'Edit this device',
      template: 'views/modal-tpls/dealer/edit-device.html',
      backdrop: 'static',
      show: false
    });

    modal.$promise.then(modal.show);
  };

});

app.controller('VirtualEditorController', function ($rootScope, $scope, $state, $modal, $window, toastr, Systems) {

  $scope.exitVirtualEditor = function() {
    $rootScope.virtualEditor = null;
    var user = JSON.parse($window.localStorage.getItem('hcm_editor_dealer'));
    delete(user.virtualEditor);
    $window.localStorage.setItem('hcm_editor_dealer', JSON.stringify(user));

    $state.go('dealer-jobs-list');
  };

});

app.controller('VirtualEditorStartNewTemplateController', function ($rootScope, $scope, $state, $modal, $window, toastr, Systems, Dealer) {

  $scope.finish = function() {
    var jsonData = JSON.stringify({
      name: $scope.startNewTemplate.name
    });

    Dealer.createNewTemplate(jsonData).then(function(data) {
      if (data.result) {
        var roomId = data.roomId;

        Dealer.getRoom(roomId).then(function (data) {
          var room = data.data;

          // 기존 프로그램과의 호환을 위해 가상의 bstation 생성
          if (!room.bstations) {
            room.bstations = [$rootScope.virtualBstation];
          }

          $rootScope.virtualSystem.id = room.id;
          $rootScope.virtualSystem.rooms = [room];
          Systems.loadSystem($rootScope.virtualSystem);
          Systems.setCurrentRoom(room.id);

          $rootScope.virtualEditor = true;
          var user = JSON.parse($window.localStorage.getItem('hcm_editor_dealer'));
          user.virtualEditor = 1;
          user.room_id = roomId;
          $window.localStorage.setItem('hcm_editor_dealer', JSON.stringify(user));

          $state.go('menu.programming.options');
        })
          .catch(toastr.error);
      }
    })
      .catch(toastr.error);
  };

});

app.controller('VirtualEditorEditTemplateController', function ($rootScope, $scope, $state, $modal, $window, toastr, Systems, Dealer) {

  $scope.select = function(template) {
    $scope.editTemplate.template = template;

    Dealer.getRoom(template.id).then(function (data) {
      var room = data.data;

      // 기존 프로그램과의 호환을 위해 가상의 bstation 생성
      if (!room.bstations) {
        room.bstations = [$rootScope.virtualBstation];
      }

      $rootScope.virtualSystem.id = room.id;
      $rootScope.virtualSystem.rooms = [room];
      Systems.loadSystem($rootScope.virtualSystem);
      Systems.setCurrentRoom(room.id);

      $rootScope.virtualEditor = true;
      var user = JSON.parse($window.localStorage.getItem('hcm_editor_dealer'));
      user.virtualEditor = 1;
      user.room_id = template.id;
      $window.localStorage.setItem('hcm_editor_dealer', JSON.stringify(user));

      $state.go('menu.programming.options');
    })
      .catch(toastr.error);
  };

});

app.controller('RegisterDealerController', function ($rootScope, $scope, $state, $stateParams, $http, toastr, email, Dealer, User) {
  // As3AQCjg58mH
  // VDTLGbXjS5RIWG
  // npLieg8pVH8
  // IuY82RFBx

  $scope.credentials = {
    creation_id: $stateParams.creationId,
    email: email,
    username: email,
    password: null
  };
  $scope.confirmed_password = null;

  $scope.register = function () {
    var jsonData = JSON.stringify($scope.credentials);

    Dealer.addUserFinal(jsonData).then(function (data) {
      if (data.result) {
        User.logout();
        window.location = '/';
      }
    })
      .catch(toastr.error);
  };

  // add_user_prepare 테스트용
  /*
   $scope.addUserPrepare = function() {
   var url = '/api/dealer/add_user_prepare',
   data = {
   email: 'master@linkit.kr',
   account_type: 'Dealer Installer/Programmer',
   first_name: 'Youngguk',
   last_name: 'Kim',
   phone_number: '01032047694'
   },
   jsonData = JSON.stringify(data);

   $http.post(url, jsonData)
   .success(function(response) {
   console.log(response);
   });
   };
   $scope.addUserPrepare();
   */

});

app.controller('DealerAccountSettingsController', function ($scope, $state, $stateParams, $http, toastr, Dealer, Auth) {

  $scope.dealerInfo.selectedUser = null;
  $scope.dealerInfo.addUser = {};
  $scope.dealerInfo.userData = null;

  Auth.currentUser().then(function (user) {
    $scope.dealerInfo.currentUser = user;
  });

});

app.controller('DealerChangeEmailController', function ($scope, $state, $modal, toastr, Dealer, User) {

  $scope.dealerInfo.changeEmail = {};

  $scope.save = function () {
    $scope.dealerInfo.changeEmail.old_email = $scope.dealerInfo.currentUser.dealer_username;
    var jsonData = JSON.stringify($scope.dealerInfo.changeEmail);

    Dealer.newEmail(jsonData).then(function (data) {
      if (data.result) {
        var modal = $modal({
          scope: $scope,
          title: 'Email Address Changed',
          content: 'The email address for this account has been changed. You will need to log in again with your new account credentials.',
          template: 'views/modal-tpls/dealer/dealer-settings-complete.html',
          backdrop: 'static',
          show: false
        });

        modal.$promise.then(modal.show);
      }
    })
      .catch(toastr.error);
  };

  $scope.finish = function () {
    User.logout();
    window.location = '/';
  };

});

app.controller('DealerChangePasswordController', function ($scope, $state, $modal, Dealer, User) {

  $scope.dealerInfo.changePassword = {};

  $scope.save = function () {
    $scope.dealerInfo.changePassword.email = $scope.dealerInfo.currentUser.dealer_username;
    var jsonData = JSON.stringify($scope.dealerInfo.changePassword);

    Dealer.newPassword(jsonData).then(function (data) {
      if (data.result) {
        var modal = $modal({
          scope: $scope,
          title: 'Password Changed',
          content: 'The password for this account has been changed. You will need to log in again with your new account credentials.',
          template: 'views/modal-tpls/dealer/dealer-settings-complete.html',
          backdrop: 'static',
          show: false
        });

        modal.$promise.then(modal.show);
      }
    })
      .catch(toastr.error);
  };

  $scope.finish = function () {
    User.logout();
    window.location = '/';
  };

});

app.controller('DealerSelectUserController', function ($scope, $state, $stateParams, toastr, users, Dealer) {

  $scope.users = users;

  $scope.doNext = function () {
    $state.go('dealer-account-settings.user-info');
  };

});

app.controller('DealerUserInfoController', function ($scope, $state, $modal, toastr, Dealer) {

  var jsonData = JSON.stringify({email: $scope.dealerInfo.selectedUser.email});

  Dealer.getUserData(jsonData).then(function (data) {
    if (data.result) {
      $scope.dealerInfo.userData = data.data;
    }
  })
    .catch(toastr.error);

  $scope.delete = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/dealer/delete-user.html',
      backdrop: 'static',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.deleteUser = function () {
    Dealer.deleteUser(jsonData).then(function (data) {
      if (data.result) {
        $state.go('dealer-account-settings.main');
      }
    })
      .catch(toastr.error);
  };

});

app.controller('DealerAddUserController', function ($scope, $state, $modal, toastr, categories, Dealer) {

  $scope.categories = categories;
  $scope.dealerInfo.addUser.account_type = $scope.categories[0].value;

  $scope.doNext = function () {
    var jsonData = JSON.stringify($scope.dealerInfo.addUser);

    Dealer.addUserPrepare(jsonData).then(function (data) {
      if (data.result) {
        $state.go('dealer-account-settings.main');
      }
    })
      .catch(toastr.error);
  };

});

'use strict';
/**
 * @ngdoc controller
 * @name DeviceSelectionController
 * @description
 *  room, device 선택 화면
 */
app.controller('DeviceSelectionController', function ($scope, $state, toastr, Devices, Rooms) {
	$scope.doNext = function() {
    // device button layout 페이지에서 custom driver device 에만 add infrared hex code, add custom ip string 버튼을 보이게 하기 위해
    // brand 를 저장 - brand: MX HomePro Custom Driver
    var currentDevice = _.find(Rooms.getDevices($scope.deviceMenu.roomId), {
      id: $scope.deviceMenu.deviceId
    });
    if (currentDevice) {
      $scope.deviceMenu.deviceBrand = currentDevice.brand;
      $scope.deviceMenu.deviceTargetType = currentDevice.target_type;
    }

		// device 정보 얻기
		Devices.getOne($scope.deviceMenu.roomId, $scope.deviceMenu.deviceId).then(function (data) {

			Devices.setCurrentData(data);
			$state.go('menu.advanced.device.edit-layout');

		}).catch(toastr.error);
	};

});

/**
 * @ngdoc controller
 * @name DeviceMenuLayoutController
 * @description
 *  device button layout 편집 화면
 */
app.controller('DeviceMenuLayoutController', function ($scope, $state, $stateParams, $modal, $timeout, toastr, Systems, Devices, Rooms) {
  // roomId&deviceId&page&idx&jsonData 가 없거나 finish 버튼을 클릭하는 경우 sessionStorage 에서 삭제
  if (!$stateParams.roomId || !$stateParams.deviceId || !$stateParams.page || !$stateParams.jsonData) {
    sessionStorage.removeItem('menuActivityBtns');
  }

  // Advanced Menu 의 Automate activities in the system 메뉴에서 button 을 만들고 돌아온 경우
  if (!$scope.deviceMenu.roomId || !$scope.deviceMenu.deviceId) {
    $scope.deviceMenu.roomId = Number($stateParams.roomId);
    $scope.deviceMenu.deviceId = Number($stateParams.deviceId);

    // device button layout 페이지에서 custom driver device 에만 add infrared hex code, add custom ip string 버튼을 보이게 하기 위해
    // brand 를 저장 - brand: MX HomePro Custom Driver
    var currentDevice = _.find(Rooms.getDevices($scope.deviceMenu.roomId), {
      id: $scope.deviceMenu.deviceId
    });
    if (currentDevice) {
      $scope.deviceMenu.deviceBrand = currentDevice.brand;
      $scope.deviceMenu.deviceTargetType = currentDevice.target_type;
    }

    console.log('DeviceMenuLayoutController - deviceMenu=', $scope.deviceMenu);

    // device 정보 얻기
    Devices.getOne($scope.deviceMenu.roomId, $scope.deviceMenu.deviceId).then(function (data) {

      Devices.setCurrentData(data);

      console.log($stateParams);

      $timeout(function () {
        // 페이지를 $stateParams 로 넘어온 값으로 선택
        if ($stateParams.page > $scope.deviceMenu.menu.pages.length) {  // $stateParams 로 넘어온 page 가 현재 페이지들 수보다 큰 경우 : 빈 페이지를 만들었던 경우
          // device-interface.js 에 있는 addPage 함수 내용 복사해서 수정
          (function () {
            var menuPages = $scope.deviceMenu.menu.pages;
            var pageId = _(menuPages).pluck('id').max().value(),
              btnId = _(menuPages[menuPages.length - 1].btns).pluck('id').max().value() + 1;

            if (!$scope.deviceMenu.deviceData.room_id) {
              $scope.deviceMenu.deviceData.room_id = $stateParams.roomId;
            }
            if (!$scope.deviceMenu.deviceData.device_id) {
              $scope.deviceMenu.deviceData.device_id = $stateParams.deviceId;
            }

            var btns = [];
            for (var i = 0; i < 6; i++) {
              btns.push(Devices.makeBlankMenuButton(btnId, i, $scope.deviceMenu.deviceData));
              btnId++;
            }
            pageId++;
            console.log('pushing page with id of ' + pageId);
            $scope.deviceMenu.menu.pages.push({id: pageId, btns: btns});
          })();
        }
        $scope.deviceMenu.currentMenuPage = $stateParams.page;

        var btnData = JSON.parse($stateParams.jsonData),
          selectedBtn = _.find($scope.deviceMenu.menu.pages[$stateParams.page - 1].btns, {
            page_btn_idx: +$stateParams.idx
          });
        selectedBtn.display_text = btnData.text || btnData.display_text;
        selectedBtn.auto_wait = btnData.auto_wait;
        selectedBtn.edit_type = btnData.edit_type;
        selectedBtn.popup = btnData.popup;
        selectedBtn.activity = (typeof btnData.activity === 'undefined') ? '' : _.isObject(btnData.activity) ? btnData.activity : JSON.parse(btnData.activity);
        selectedBtn.macro = btnData.macro;
        selectedBtn.func = btnData.func;

        if (_.isArray(selectedBtn.activity) && selectedBtn.activity.length === 0) {
          selectedBtn.activity = '[]';
        }

        // 버튼을 $stateParams 로 넘어온 값으로 선택
        $timeout(function () {
          $scope.deviceMenu.selectedBtn = selectedBtn;
        }, 100);

        // roomId&deviceId&page&idx&jsonData 가 있는 경우 sessionStorage 에 정보를 저장
        if ($stateParams.roomId && $stateParams.deviceId && $stateParams.page && $stateParams.jsonData) {
          var btns = sessionStorage.getItem('menuActivityBtns') && JSON.parse(sessionStorage.getItem('menuActivityBtns')),
            duplicate = false;

          // 기존에 저장한 버튼이 있는 경우
          if (btns) {
            // 저장된 버튼 복구
            _.each(btns, function (item) {
              var btn = _.find($scope.deviceMenu.menu.pages[item.page - 1].btns, {
                page_btn_idx: +item.idx
              });

              // 현재 stateParams 로 들어온 버튼이 기존에 저장되있던 버튼인 경우
              if (item.page == $stateParams.page && item.idx == $stateParams.idx) {
                var jsonData = JSON.parse($stateParams.jsonData);

                btn.display_text = jsonData.text || jsonData.display_text;
                btn.auto_wait = jsonData.auto_wait;
                btn.edit_type = jsonData.edit_type;
                btn.popup = jsonData.popup;
                btn.activity = _.isObject(jsonData.activity) ? jsonData.activity : JSON.parse(jsonData.activity);
                btn.macro = jsonData.macro;
                btn.func = jsonData.func;

                duplicate = true;
              }
              else {
                btn.display_text = item.jsonData.text || item.jsonData.display_text;
                btn.auto_wait = item.jsonData.auto_wait;
                btn.edit_type = item.jsonData.edit_type;
                btn.popup = item.jsonData.popup;
                btn.activity = _.isObject(item.jsonData.activity) ? item.jsonData.activity : JSON.parse(item.jsonData.activity);
                btn.macro = item.jsonData.macro;
                btn.func = item.jsonData.func;
              }

              if (_.isArray(btn.activity) && btn.activity.length === 0) {
                btn.activity = '[]';
              }
            });

            if (!duplicate) {
              btns.push({
                roomId: $stateParams.roomId,
                deviceId: $stateParams.deviceId,
                page: $stateParams.page,
                idx: $stateParams.idx,
                jsonData: JSON.parse($stateParams.jsonData)
              });
            }
          }
          // 저장된 버튼이 없는 경우
          else {
            btns = [{
              roomId: $stateParams.roomId,
              deviceId: $stateParams.deviceId,
              page: $stateParams.page,
              idx: $stateParams.idx,
              jsonData: JSON.parse($stateParams.jsonData)
            }];
          }

          sessionStorage.setItem('menuActivityBtns', JSON.stringify(btns));
        }
      }, 0);

    }).catch(toastr.error);
  }

  $scope.addAutomatedActivity = function (roomId, deviceId, page, idx) {
    $state.go('menu.advanced.add-macros.button-name', {roomId: roomId, deviceId: deviceId, page: page, idx: idx});
  };

  $scope.finish = function () {
    // roomId&deviceId&page&idx&jsonData 가 없거나 finish 버튼을 클릭하는 경우 sessionStorage 에서 삭제
    sessionStorage.removeItem('menuActivityBtns');

		var pages = [],
				hardPages = angular.copy($scope.deviceMenu.hard.pages),
				hardBtns = _.flatten(hardPages, 'btns'),
				menuPages = angular.copy($scope.deviceMenu.menu.pages),
				pageId = 0,
				buttonId = 8192;

		function rewriteMacro(btn) {
			var newMacro = {};
			if(btn.macro) {
				///Devices Service의 getOne function을 통해 Device정보를 얻어 올때 각 button의 macro data를 object로 변환하였으나
				///혹시 몰라서 macro가 string type인 경우에 대해서도 처리.

        if (_.isArray(btn.macro) && btn.macro.length > 1) {
          newMacro = angular.copy(btn.macro);
        } else {
          newMacro = angular.isObject(btn.macro) ? btn.macro[0] : JSON.parse(btn.macro)[0];
        }
			}

      if (newMacro && newMacro.length > 1) {
        if (btn.func) {
          _.each(newMacro, function (item) {
            if (btn.repeatSetting === 'custom') {
              item.repeat_min = btn.repeat_min;
              item.repeat_macro = btn.repeat_macro;
            } else {
              delete item.repeat_min;
              delete item.repeat_macro;
            }

            if (!btn.press_hold) {
              item.press_hold = 0;
            } else {
              item.press_hold = 1;
            }
          });
        }

        //console.log('rewriteMacro- newMacro=', newMacro);
        return JSON.stringify(newMacro);
      } else {
        if (btn.func) {
          if (_.isEmpty(newMacro)) {
            newMacro.type = 1;
            newMacro.device_id = parseInt($scope.deviceMenu.deviceData.id, 10);
          }

          console.log('rewriteMacro-btn=', btn);

          newMacro.func = btn.func.id;

          if (btn.repeatSetting === 'custom') {
            newMacro.repeat_min = btn.repeat_min;
            newMacro.repeat_macro = btn.repeat_macro;
          } else {
            delete newMacro.repeat_min;
            delete newMacro.repeat_macro;
          }

          if (!btn.press_hold) {
            newMacro.press_hold = 0;
          } else {
            newMacro.press_hold = 1;
          }
        }

        //console.log('rewriteMacro- newMacro=', newMacro);
        return JSON.stringify([newMacro]);
      }
		}

		_.remove(hardBtns, function(button) {
			return !button.display_text;
		});

		_.forEach(hardBtns, function(hardBtn) {
			hardBtn.macro = rewriteMacro(hardBtn);
			hardBtn.page_btn_idx = -1;
			delete hardBtn.func;
			delete hardBtn.repeatSetting;
			delete hardBtn.repeat_min;
			delete hardBtn.repeat_macro;
			delete hardBtn.press_hold;
			delete hardBtn.name;
		});


		pages.push({ id: -1, btns: hardBtns });

		_.forEach(menuPages, function(page) {
			if (_.filter(page.btns, 'display_text').length) {
				page.id = pageId;
				pageId++;

				if (page.page_hidden == 1) {
					page.hidden = 1;
				} else {
					page.hidden = 0;
				}

				_.remove(page.btns, function(button) {
					return !button.display_text;
				});

				_.forEach(page.btns, function(btn) {

					if (btn.display_text) {

						btn.id = buttonId;
						buttonId++;

						btn.hidden = (true == btn.hidden ? 1:0);

						btn.macro = rewriteMacro(btn);

            if (btn.activity) {
              btn.activity = JSON.stringify(_.isObject(btn.activity) ? btn.activity : JSON.parse(btn.activity));
            }

						delete btn.func;
						delete btn.repeatSetting;
						delete btn.repeat_min;
						delete btn.repeat_macro;
						delete btn.press_hold;
					}

				});

				pages.push(page);
			}
		});

	 
		var returnDevice = angular.copy($scope.deviceMenu.deviceData); 
		_.assign(returnDevice, {pages: pages});

		//if these properties are seleceted as `None` on the client their properties will be null
		//in that case, they to be changed to -1 
		
		returnDevice.meta_pwr_on =  returnDevice.meta_pwr_on || -1;
		returnDevice.meta_pwr_off = returnDevice.meta_pwr_off || -1;
		returnDevice.meta_pwr_toggle = returnDevice.meta_pwr_toggle || -1;
		returnDevice.meta_in_toggle = returnDevice.meta_in_toggle || -1;
		

		var jsonData = JSON.stringify({ data: returnDevice });

		// 해당 device 저장
		Devices.storeOne($scope.deviceMenu.roomId, $scope.deviceMenu.deviceId, jsonData).then(function(data) {
			// system update
			Systems.updateSystem($scope.system.id);

			$state.go('menu.advanced.options');
		}, function (error) {
			toastr.error(error, 'Error');
		});

	};

});

'use strict';
/**
 * @ngdoc controller
 * @name EditMacroRoomController
 * @description
 *  room 선택 화면
 */
app.controller('EditMacroRoomController', function ($rootScope, $scope, $state, toastr, Systems, Devices) {
	$scope.system = Systems.getCurrentSystem();


	$scope.doNext = function() {
		// room 정보 얻어와서 저장
		return Systems.updateRoomDevices($scope.addMacro.room.id)
		.then(function(data) {
			var roomDevices = Systems.getRoomDevices();
			$scope.addMacro.room.devices = roomDevices.devices;

			// dev_type이 twoway(1)인 device를 모두 제거하는 과정에서 $scope.addMacro.room.devices 에서 SNP-2 가 제거되는데 SNP-2 가 있는 목록이 "Add jump to" 에서 필요해서 copy 를 만듬
			$scope.addMacro.room.devicesAndSNP2 = angular.copy(roomDevices.devices);

			// twoway_devices 중 TRF-ZW 추가한 device 목록 생성
			$scope.addMacro.room.allDevices = angular.copy($scope.addMacro.room.devices);

			///target_type이 0인 device를 모두 제거.
			_.each($scope.addMacro.room.allDevices, function (dev) {
				if(0 == dev.target_type) {
					var idx = $scope.addMacro.room.allDevices.indexOf(dev);
					$scope.addMacro.room.allDevices.splice(idx, 1);
				}
			});

			_.each($scope.system.rooms, function(room) {
				var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
				if (trfzw) {
					if (!trfzw.data) {
						trfzw.data = [];
					}
					$scope.addMacro.room.allDevices.push(trfzw);

					return false;
				}
			});

      // allDevices 에 interface device 추가
      _.each(roomDevices.twoway_devices, function (twowayDevice) {
        if (twowayDevice.device_type === 2) {
          $scope.addMacro.room.allDevices.push(twowayDevice);
          // core device 의 funcs 를 복사
          Devices.getTwowayDeviceCommands(twowayDevice.link_room_id, twowayDevice.link_dev_id)
            .then(function (data) {
              twowayDevice.funcs = data;
            });
        }
      });

      // room devices 에 entertainment twoway devices 추가 - core device 는 추가하지 않음
      var entTwowayDevices = [];
      _.each(roomDevices.twoway_devices, function (item) {
        if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
          entTwowayDevices.push(item);
        }
      });
      $scope.addMacro.room.notEntDevices = angular.copy($scope.addMacro.room.devices);
      $scope.addMacro.room.devices = $scope.addMacro.room.devices.concat(entTwowayDevices);

			$state.go('menu.advanced.edit-macros.macro-simulator', {roomId: $scope.addMacro.room.id});
		})
		.catch(toastr.error);
	};
});

/**
 * @ngdoc controller
 * @name MacroSimulatorController
 * @description
 *  macro 를 수정할 button, device 선택 화면
 */
app.controller('MacroSimulatorController', function ($rootScope, $scope, $state, $modal, toastr, Systems, Devices, Macro) {
  // Advanced Menu 의 Edit your automated activities 메뉴를 클릭해서 들어온 경우
  //   => currentRoom 을 선택된 room 으로 설정함
  $scope.addMacro.room = $scope.room;

  // 아래의 코드는 EditMacroRoomController 의 doNext 에서 하는 동작임
  Systems.updateRoomDevices($scope.addMacro.room.id)
    .then(function (data) {
      var roomDevices = Systems.getRoomDevices();
      $scope.addMacro.room.devices = roomDevices.devices;

      // dev_type이 twoway(1)인 device를 모두 제거하는 과정에서 $scope.addMacro.room.devices 에서 SNP-2 가 제거되는데 SNP-2 가 있는 목록이 "Add jump to" 에서 필요해서 copy 를 만듬
      $scope.addMacro.room.devicesAndSNP2 = angular.copy(roomDevices.devices);

      // twoway_devices 중 TRF-ZW 추가한 device 목록 생성
      $scope.addMacro.room.allDevices = angular.copy($scope.addMacro.room.devices);

		///target_type이 0인 device를 모두 제거.
		_.each($scope.addMacro.room.allDevices, function (dev) {
			if(0 == dev.target_type) {
				var idx = $scope.addMacro.room.allDevices.indexOf(dev);
				$scope.addMacro.room.allDevices.splice(idx, 1);
			}
		});

      _.each($scope.system.rooms, function (room) {
        var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
        if (trfzw) {
          if (!trfzw.data) {
            trfzw.data = [];
          }
          $scope.addMacro.room.allDevices.push(trfzw);

          return false;
        }
      });

      // allDevices 에 interface device 추가
      _.each(roomDevices.twoway_devices, function (twowayDevice) {
        if (twowayDevice.device_type === 2) {
          $scope.addMacro.room.allDevices.push(twowayDevice);
          // core device 의 funcs 를 복사
          Devices.getTwowayDeviceCommands(twowayDevice.link_room_id, twowayDevice.link_dev_id)
            .then(function (data) {
              twowayDevice.funcs = data;
            });
        }
      });

      // room devices 에 entertainment twoway devices 추가 - core device 는 추가하지 않음
      var entTwowayDevices = [];
      _.each(roomDevices.twoway_devices, function (item) {
        if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
          entTwowayDevices.push(item);
        }
      });
      $scope.addMacro.room.notEntDevices = angular.copy($scope.addMacro.room.devices);
      $scope.addMacro.room.devices = $scope.addMacro.room.devices.concat(entTwowayDevices);

      $state.go('menu.advanced.edit-macros.macro-simulator', {roomId: $scope.addMacro.room.id});
    })
    .catch(toastr.error);

	$scope.editMacro.currentPage = 1;

	//$scope.addMacro.room.devices = devices;

	function updateMainDevice() {
		// main device 얻어와서 저장
		Systems.updateMainDevice($scope.addMacro.room.id).then(function(data) {
			console.log(data);
      $scope.editMacro.mainDevice = data;
      $scope.client.page = angular.copy($scope.editMacro.mainDevice.pages[$scope.editMacro.currentPage - 1]);
      var numPages = $scope.editMacro.mainDevice.pages.length;
      $scope.editMacro.totalItems = numPages * 6;

      // twoway device 추가
      Devices.getTwowayDeviceButtons($scope.addMacro.room.id).then(function (data2) {
        $scope.addMacro.room.twowayDeviceButtons = data2 || [];

        // entertainment twoway devices - core device 는 추가하지 않음
        if ($scope.addMacro.room.twowayDeviceButtons.length) {
          var entTwowayDevices = [];
          _.each($scope.addMacro.room.twowayDeviceButtons, function (item) {
            if ($rootScope.env.entertainment_twoway.indexOf(item.category_id) > -1 && item.device_type !== 1) {
              item.display_text = item.name;
              item.room_id = $scope.addMacro.room.id;

              entTwowayDevices.push(item);
            }
          });
          $scope.client.twowayDeviceBtns = angular.copy(entTwowayDevices);
        }
      });
		})
		.catch(toastr.error);
		
		updatePowerMacro();
	}

  updateMainDevice();

	function isJsonString(str) {
		try {
			JSON.parse(str);
		} catch (e) {
			return false;
		}
		return true;
	}

	$scope.isJsonString = isJsonString;

	$scope.editPowerMacro = function() {
		$scope.macroType.id = 0;

		var roomId = $scope.addMacro.room.id;
		// 해당 room 의 power macro 데이터 얻어와서 저장
		Systems.updateCurrentMacroForPower(roomId).then( function(data) {
			$scope.addMacro.steps = Macro.convertMacroDataToDispData($scope.system, $scope.addMacro.room, data.macro);
			//$scope.addMacro.autoWait = data.auto_wait ? true:false;
			$scope.addMacro.doneScreen.devices = Macro.convertActivityDataToDispData($scope.system, $scope.addMacro.room, data.activity);
      $scope.addMacro.disablePopup = '' + data.popup;
			$state.go('menu.advanced.edit-macros.finish');
		}).catch(function (error) {
			alert(error);
		});
	};

	$scope.editBtnMacro = function(btn) {
		$scope.addMacro.steps = Macro.convertMacroDataToDispData($scope.system, $scope.addMacro.room, btn.macro);
		//$scope.addMacro.autoWait = btn.auto_wait ? true:false;
		$scope.addMacro.doneScreen.devices = Macro.convertActivityDataToDispData($scope.system, $scope.addMacro.room, btn.activity);
    $scope.addMacro.disablePopup = '' + btn.popup;
		$scope.addMacro.linkId = btn.link_id;
		$scope.addMacro.btnId = btn.id;

	
		$scope.macroType.id = btn.link_id !== -1? 1:2;
		
		switch ($scope.macroType.id) {
			case 1:
				$scope.addMacro.currentDevice = _.find($scope.addMacro.room.devices, {'id': btn.link_id});
				break;
			case 2:
				$scope.addMacro.button.name = btn.display_text;
				break;
			default:
				break;
		}
		$state.go('menu.advanced.edit-macros.finish');
	};

	$scope.pageChanged = function(page) {
		$scope.client.page = angular.copy($scope.editMacro.mainDevice.pages[page - 1]);
	};
	
	function updatePowerMacro() {
		// 해당 room 의 power macro 데이터 얻어와서 저장
		Systems.updateCurrentMacroForPower($scope.addMacro.room.id).then( function(data) {
			$scope.powerMacro = data.macro;
		}).catch(function (error) {
			toastr.error(error);
		});
	}
	
	$scope.deleteMacro = function(mode, linkId, roomId, deviceId, btnId) {
		var handleSuccess = function(response) {
			if(response.data.result === true) {
				toastr.success('Automated Activity has been deleted');
				updateMainDevice();
			} else {
				toastr.error('Delete failed to work. Please try again.');
			}
		};

		var handleError = function(error){
			toastr.error(error, 'Error');
		};
		
		if (mode === 'power') {
			// 해당 room 의 power macro 삭제
			Systems.deletePowerMacro($scope.addMacro.room.id).then(handleSuccess, handleError);
		} else if (mode === 'device-btn') {
			if (linkId == '-1') {
				// button ID 에 해당하는 button 삭제
				Systems.deleteButtonMacro(roomId, deviceId, btnId).then(handleSuccess, handleError);
			} else {
				// 해당 device ID 와 연계된 button 의 macro 삭제
        if (!linkId) {  // twoway device
          Systems.deleteButtonMacroWithDevice(roomId, btnId, true).then(handleSuccess, handleError);
        } else {
          Systems.deleteButtonMacroWithDevice(roomId, linkId).then(handleSuccess, handleError);
        }
			}
		}
	};
	
	$scope.deleteMacroModal = function(mode, linkId, roomId, deviceId, btnId) {
		var scope = $scope.$new();
		scope.data = {
			mode: mode,
			linkId: linkId,
			roomId: roomId,
			deviceId: deviceId,
			btnId: btnId
		};
		
		var modal = $modal({
			scope: scope,
			template: 'views/modal-tpls/delete-macro.html',
			container: 'body',
			show: false
		});
		modal.$promise.then(modal.show);
	};

});

/**
 * @ngdoc controller
 * @name EditMacroController
 * @description
 *  macro device, commands 입력
 */
app.controller('EditMacroController', function ($scope, $timeout, $state, $modal, toastr, Devices, Systems) {

	$scope.selectedItem = {};

	$scope.remove = function(scope) {
		scope.remove();
	};

	$scope.options = {
		accept: function(sourceNodeScope, destNodesScope, destIndex) {
			//console.log('sourceNodeScope=', sourceNodeScope);
			//console.log('destNodesScope=', destNodesScope);
			if (sourceNodeScope.$modelValue.type == null || destNodesScope.nodropEnabled || destNodesScope.outOfDepth(sourceNodeScope)) {
				return false;
			}

	        var subDepth = 0;
	        function countSubDepth(scope) {
				var i, childNodesScope,
					count = 0,
					nodes = scope.childNodes();

				//console.log('childNodes=', nodes);

				for (i = 0; i < nodes.length; i++) {
					childNodesScope = nodes[i].$childNodesScope;

					if (childNodesScope && childNodesScope.childNodesCount() > 0) {
						count = 1;
						countSubDepth(childNodesScope);
					}
				}
				subDepth += count;				
			}

			var maxSubDepth = function (scope) {
				subDepth = 0;
				if (scope.hasChild()) {
					countSubDepth(scope.$childNodesScope);
					subDepth++;
				}
				return subDepth;
			};

			/*
			console.log('sourceNodeScope.childNodes()=', sourceNodeScope.childNodes());
			console.log('sourceNodeScope.childNodesCount()=', sourceNodeScope.childNodesCount());
			console.log('maxSubDepth(sourceNodeScope)=', maxSubDepth(sourceNodeScope));
			console.log('destNodesScope.depth()=', destNodesScope.depth());
			console.log('destNodesScope.maxDepth=', destNodesScope.maxDepth);
			console.log('destNodesScope.$treeScope.maxDepth=', destNodesScope.$treeScope.maxDepth);
			*/

			//angular-ui-tree module의 maxSubDepth function에서 sub depth count를 child node 존재 유무를 기준으로 함.
			//그래서 2depth인 tree인 경우 depth count를 0으로 return하여 outOfDepth function을 호출해도 제대로된 결과를 받지 못하고,
			// (max-depth -1) depth인 tree에 2depth의 tree를 drag/drop하면 삽입이 됨.
			//그래서, 별도의 maxSubDepth function을 구현함.
			var sumDepth =  (maxSubDepth(sourceNodeScope)+1) + destNodesScope.depth();
			var maxDepth = destNodesScope.maxDepth || destNodesScope.$treeScope.maxDepth;

			if(sumDepth  > maxDepth){
				return false;
			}

			return true;
		},
	};

	var getRootNodesScope = function() {
		return angular.element(document.getElementById("tree-root")).scope();
	};

	$scope.collapseAll = function() {
		var scope = getRootNodesScope();
		scope.collapseAll();
	};

	$scope.expandAll = function() {
		var scope = getRootNodesScope();
		scope.expandAll();
	};


	$scope.addStep = function (device, func, twowayDevice) {
		if (twowayDevice) {
			return {'name': 'S : ' + func.name + ' (' + func.room + ')', 'type': 16, 'sub_type': 'scene', 'id': func.id, 'cmd': 1 };		// 1 : Activate, 2 : Deactivate
		} else {
			return {'name': device.name + ': ' + func.name, 'type': 1, 'device_id': device.id, 'func': func.id };
		}
	};

	$scope.deleteAll = function() {
		var arr = $scope.addMacro.steps;
		while(arr.length > 0) {
			arr.pop();                     
		}
	};

	$scope.goBack = function() {
		$state.go('menu.advanced.edit-macros.macro-simulator', {roomId: $scope.addMacro.room.id});
	};

	$scope.doNext = function() {
		$state.go('menu.advanced.edit-macros.macro-done');
	};

	$scope.refreshScenes = function(device) {
		Devices.updateTwowayDevice($scope.addMacro.room.id, device.id)
			.then(function(data) {
				console.log(data);

				var item = _.where($scope.addMacro.room.allDevices, {'id': device.id})[0];
				item.data = data;
			}, function() {
				var modal = $modal({
					scope: $scope,
					title: "Couldn't refresh the list",
					container: '.editor-container',
					template: 'views/modal-tpls/refresh-the-list.html',
					show: false
				});

				modal.$promise.then(modal.show);
			})
			.catch(toastr.error);
	};

});

/**
 * @ngdoc controller
 * @name EditMacroDoneController
 * @description
 *  macro 설정 완료 화면
 */
app.controller('EditMacroDoneController', function ($rootScope, $scope, $state, toastr, Systems, Macro) {
  $scope.powerOffBtnOption = $rootScope.env.powerOffBtnOption;

	$scope.addMacro.doneScreen.index = 0;

	$scope.goBack = function() {
		$state.go('menu.advanced.edit-macros.finish');
	};

	$scope.doNext = function() {
    if ($scope.addMacro.disablePopup == '0') {
			var steps =  angular.copy($scope.addMacro.steps);
			steps = Macro.cleanUp(steps);
			console.log('steps=', steps);

			//var autoWait = $scope.addMacro.autoWait ? 1:0;
			var jsonData = JSON.stringify( { auto_wait: 1 , edit_type: 1, popup: 0, activity: [], macro: steps } );
			console.log('jsonData=', jsonData);

			var handleSuccess = function(response) {
				if(response.data.result === true) {
					$state.go('menu.advanced.edit-macros.next-action');
				} else {
					toastr.error('Your edit/create action failed.  Please try again.');
				}
			};

			var handleError = function(error){
				toastr.error("Please try again");
			};

			if ($scope.macroType.id === 0) {
				// 해당 room 의 power macro 데이터 저장
				Systems.storePowerMacro($scope.addMacro.room.id, jsonData).then(handleSuccess, handleError);
			} else if ($scope.macroType.id === 1) {
				// 해당 device ID 와 연계된 button 의 데이터 저장
        if (!$scope.addMacro.linkId) {  // twoway device
          Systems.storeDeviceButtonMacro($scope.addMacro.room.id, $scope.addMacro.btnId, jsonData, true).then(handleSuccess, handleError);
        } else {
          Systems.storeDeviceButtonMacro($scope.addMacro.room.id, $scope.addMacro.linkId, jsonData).then(handleSuccess, handleError);
        }
			} else {
				// button ID 에 해당하는 button 의 데이터 저장
				Systems.storeCustomButtonMacro($scope.addMacro.room.id, $scope.addMacro.room.main_device_id, $scope.addMacro.btnId, jsonData).then(handleSuccess, handleError);
			}
		} else {
			$state.go('menu.advanced.edit-macros.macro-commands');
		}
	};

});


app.controller('EditMacroCommandsController', function ($scope, $state, toastr, Systems, Macro) {

	$scope.goBack = function() {
		if ($scope.addMacro.doneScreen.index === 0) {
			$state.go('menu.advanced.edit-macros.macro-done' );
		} else {
			$scope.addMacro.doneScreen.index--;
		}
	};


	$scope.doNext = function() {
		var len = $scope.addMacro.doneScreen.devices.length;
		if ($scope.addMacro.doneScreen.index + 1 === len) {
			//var autoWait = $scope.addMacro.autoWait ? 1:0;
			var steps =  angular.copy($scope.addMacro.steps);
			steps = Macro.cleanUp(steps);
			console.log('steps=', steps);

			var activityData = angular.copy($scope.addMacro.doneScreen.devices);

			_.forEach(steps, function(step) { delete step.name; });
			_.forEach(activityData, function(activity) {
				activity.func = activity.func.id;
				delete activity.name;
				delete activity.repeat_min;
				delete activity.repeat_macro;
				delete activity.target_type;
				delete activity.target_addr;
				delete activity.target_port;
				delete activity.funcs;
			});

      var jsonData = JSON.stringify({
        auto_wait: 1,
        edit_type: 1,
        popup: +$scope.addMacro.disablePopup,
        activity: activityData,
        macro: steps
      });

			var handleSuccess = function(response) {
				if(response.data.result === true) {
					$state.go('menu.advanced.edit-macros.next-action');
				} else {
					toastr.error('Your edit/create action failed.  Please try again.');
				}
			};

			var handleError = function(error){
				toastr.error('Please try again');
			};

			if ($scope.macroType.id === 0) {
				Systems.storePowerMacro($scope.addMacro.room.id, jsonData).then(handleSuccess, handleError);
			} else if ($scope.macroType.id === 1) {
        if (!$scope.addMacro.linkId) {  // twoway device
          Systems.storeDeviceButtonMacro($scope.addMacro.room.id, $scope.addMacro.btnId, jsonData, true).then(handleSuccess, handleError);
        } else {
          Systems.storeDeviceButtonMacro($scope.addMacro.room.id, $scope.addMacro.linkId, jsonData).then(handleSuccess, handleError);
        }
			} else {
				Systems.storeCustomButtonMacro($scope.addMacro.room.id, $scope.addMacro.room.main_device_id, $scope.addMacro.btnId, jsonData).then(handleSuccess, handleError);
			}
		} else {
			$scope.addMacro.doneScreen.index++;
		}

	};
});

'use strict';


app.controller('ChooseExternalServiceController', function ($rootScope, $scope, $state, $modal, ExternalService, toastr, Systems, User) {
  $scope.services = [];
  if(-1 != $rootScope.env.external_service.indexOf('amazon_echo')) {
    var service = {name:'Amazon Alexa', value:'amazon_echo'};
    $scope.services.push(service);
  }
  if(-1 != $rootScope.env.external_service.indexOf('g_assist')) {
    var service = {name:'Google Assistant', value:'g_assist'};
    $scope.services.push(service);
  }  

  $scope.doNext = function () {
    if('amazon_echo' == $scope.extService.service) {
      if ($scope.extService.amazonEcho) {
        $state.go('menu.advanced.external-service.choose-scene-type');
      } else {
        $state.go('menu.advanced.external-service.alexa-setup');
      }
    } else if('g_assist' == $scope.extService.service) {
      if('' == $scope.extService.gAssist) {
        // create google assistant service
        ExternalService.createService($scope.extService.service, $scope.system.id).then(function (data) {
          if (data.result) {
            $scope.extService.gAssist = data.data;
            console.log('$scope.extService.gAssist=', $scope.extService.gAssist);

          } else {
            toastr.error(data.error);
          }
        })
          .catch(toastr.error);      
      } else {
        $state.go('menu.advanced.external-service.custom-action-name');
      }

    }
  };

});

app.controller('AlexaSetupController', function ($scope, $state, toastr, ExternalService) {

  $scope.goChooseSceneType = function () {
    // create Amazon service
    ExternalService.createService($scope.extService.service, $scope.system.id).then(function (data) {
      if (data.result) {
        $scope.extService.amazonEcho = data.data;

        $state.go('menu.advanced.external-service.choose-scene-type');
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  $scope.finishAlexaSetup = function () {
    // create Amazon service
    ExternalService.createService($scope.extService.service, $scope.system.id).then(function (data) {
      if (data.result) {
        $scope.extService.amazonEcho = data.data;

        $state.go('menu.advanced.options');
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  $scope.goAdvancedMenu = function () {
    $state.go('menu.advanced.options');
  };

});

app.controller('ChooseSceneTypeController', function ($scope, $state, $stateParams) {

  if (!$scope.extService.service) {
    $scope.extService.service = 'amazon_echo';
  }
  if (!$scope.extService.amazonEcho) {
    var amazonEcho = _.find($scope.system.ext_service, {
      name: 'amazon_echo'
    });
    if (amazonEcho) {
      $scope.extService.amazonEcho = angular.copy(amazonEcho);
    }
  }

  // Home Overview 화면에서 수정하러 들어온 경우
  if ($stateParams.scene) {
    $scope.$parent.extService.scene = JSON.parse($stateParams.scene);
  }

  $scope.doNext = function () {
    // Home Overview 화면에서 수정하러 들어온 경우
    if ($stateParams.scene) {
      $state.go('menu.advanced.external-service.spoken-text', {extService: JSON.stringify($scope.extService)});
    } else {
      $state.go('menu.advanced.external-service.spoken-text');
    }
  };

});

app.controller('SpokenTextController', function ($scope, $state, $stateParams) {

  if ($stateParams.extService) {
    $scope.$parent.extService = JSON.parse($stateParams.extService);
  }

  $scope.doNext = function () {
    $state.go('menu.advanced.add-macros.button-name', {extService: JSON.stringify($scope.extService)});
  };

});

app.controller('SceneCreateCompleteController', function ($scope, $state, $stateParams, toastr, ExternalService) {

  if ($stateParams.extService) {
    $scope.$parent.extService = JSON.parse($stateParams.extService);

    // scene 편집
    if ($scope.extService.scene.id) {
      ExternalService.setScene($scope.system.id, $scope.extService.scene.roomId, $scope.extService.service, JSON.stringify($scope.extService.scene)).then(function (data) {
        if (data.result) {
          $scope.showView = true;

          console.log(data);
        } else {
          toastr.error(data.error);
        }
      })
        .catch(toastr.error);
    }
    // scene 생성
    else {
      ExternalService.createScene($scope.system.id, $scope.roomId, $scope.extService.service, JSON.stringify($scope.extService.scene)).then(function (data) {
        if (data.result) {
          $scope.showView = true;

          console.log(data);
        } else {
          if('amazon_echo' == $scope.extService.service) {
            $state.go('menu.advanced.external-service.spoken-text', {extService: $stateParams.extService});
          } else {
            $state.go('menu.advanced.external-service.custom-phrase', {extService: $stateParams.extService});
          }
          toastr.error(data.error);          
        }
      })
        .catch(function (error) {
          if('amazon_echo' == $scope.extService.service) {
            $state.go('menu.advanced.external-service.spoken-text', {extService: $stateParams.extService});
          } else {
            $state.go('menu.advanced.external-service.custom-phrase', {extService: $stateParams.extService});
          }
          toastr.error(error);
        });
    }

  }

  $scope.goChooseSceneType = function () {
    $state.go('menu.advanced.external-service.choose-scene-type');
  };

  $scope.goCustomActionName = function () {
    $scope.extService.scene.name = '';
    $scope.extService.scene.response_text = '';
    $scope.extService.scene.spoken_text = '';
    $scope.extService.scene.text = [];
    $state.go('menu.advanced.external-service.custom-action-name');
  };

  $scope.goAdvancedMenu = function () {
    $state.go('menu.advanced.options');
  };

});

app.controller('CustomActionName', function ($scope, $state, $stateParams) {

  if (!$scope.extService.service) {
    $scope.extService.service = 'g_assist';
  }
  if (!$scope.extService.gAssist) {
    var gAssist = _.find($scope.system.ext_service, {
      name: 'g_assist'
    });
    if (gAssist) {
      $scope.extService.gAssist = angular.copy(gAssist);
    }
  }

  // Home Overview 화면에서 수정하러 들어온 경우
  if ($stateParams.scene) {
    $scope.$parent.extService.scene = JSON.parse($stateParams.scene);
  }

  $scope.doNext = function () {
    // Home Overview 화면에서 수정하러 들어온 경우
    if ($stateParams.scene) {
      $state.go('menu.advanced.external-service.custom-phrase', {extService: JSON.stringify($scope.extService)});
    } else {
      $state.go('menu.advanced.external-service.custom-phrase');
    }
  };

  $scope.doNext = function () {
    $state.go('menu.advanced.external-service.custom-phrase');
  };

});

app.controller('CustomPhrase', function ($scope, $state, $stateParams, toastr) {

  if ($stateParams.extService) {
    $scope.$parent.extService = JSON.parse($stateParams.extService);
  }

  if(null == $scope.extService.scene.text) {
    $scope.extService.scene.text = [];
  }

  $scope.doNext = function () {

    var arrPhrase = $scope.extService.scene.text;
    var arrCompare = [];
    for(var idx in arrPhrase) {
      var phrase = arrPhrase[idx];
      if(null != phrase && 0 < phrase.length) {
        if(0 > arrCompare.indexOf(phrase)) {
          arrCompare.push(phrase);
        } else {
          toastr.error('An entered Command Phrase is already being used in the system. Command Phrases must be unique.');
          for (var idx in arrPhrase) {
            arrPhrase[idx] = '';
          }
          return;
        }
      }
    }
    
    $state.go('menu.advanced.external-service.assist-response');
  };

});

app.controller('AssistResponse', function ($scope, $state, $stateParams) {

  if ($stateParams.extService) {
    $scope.$parent.extService = JSON.parse($stateParams.extService);
  }

  $scope.extService.isCustomRes = false;
  if($scope.extService.scene.response_text && $scope.extService.scene.response_text.length > 0) {
    $scope.extService.isCustomRes = true;
  }

  $scope.doNext = function () {
    if(false == $scope.extService.isCustomRes) {
      $scope.extService.scene.response_text = '';
    }
  
    $state.go('menu.advanced.add-macros.button-name', {extService: JSON.stringify($scope.extService)});
  };

});


'use strict';
/**
 * @ngdoc controller
 * @name NewOrExistingController
 * @description
 *  existing device, new device 선택 화면
 */
app.controller('NewOrExistingController', function ($scope, $state) {

	$scope.doNext = function () {
		if ($scope.learnIr.exists) {
			$state.go('menu.advanced.learn-ir.select-device');
		} else {
			$state.go('menu.advanced.learn-ir.new-device');
		}
	};
});

/**
 * @ngdoc controller
 * @name ExistingDeviceController
 * @description
 *  room, device 선택 화면
 */
app.controller('ExistingDeviceController', function ($scope, $state, toastr, Devices) {

	$scope.doNext = function () {
		// device 정보 얻기
		Devices.getOne($scope.learnIr.roomId, $scope.learnIr.deviceId).then(function (data) {
			Devices.setCurrentData(data);
			$state.go('menu.advanced.learn-ir.edit');
		}).catch(toastr.error);
	};
});

/**
 * @ngdoc controller
 * @name NewDeviceController
 * @description
 *  새 device 정보 입력 화면
 */
app.controller('NewDeviceController', function ($scope, $state, Devices, Systems, categories) {
	$scope.types = _.pluck(categories, 'name');
	$scope.learnIr.room = _.find($scope.system.rooms, { 'id': $scope.room.id });
	$scope.learnIr.device = { target: {} };
	$scope.learnIr.device.category = categories[0].name;
	$scope.learnIr.device.target.type = 1;
	$scope.learnIr.device.target.port = 255;
	//$scope.learnIr.device.target.addr = null;
	//$scope.learnIr.device.target.addr = $scope.learnIr.room.bstations[0].mac;

	var system = Systems.getCurrentSystem(),
		allBstations = [];
	_.each(system.rooms, function (room) {
		allBstations = allBstations.concat(angular.copy(room.bstations));
	});
	$scope.bstations = allBstations;
	//virtual base station을 제외시켜야 함.
	var idx = _.findIndex($scope.bstations, {'type': 'VIRTUAL'});
	while(0 <= idx) {
		$scope.bstations.splice(idx, 1);
		idx = _.findIndex($scope.bstations, {'type': 'VIRTUAL'});
	}
	
	$scope.doNext = function () {
		var tempDeviceData = { id: '-1', name: $scope.learnIr.device.name, meta_in_toggle: -1, meta_pwr_toggle: -1, meta_pwr_on: -1, meta_pwr_off: -1, repeat: { min: 1, macro: 3 }, dbCon: [], dbKey: [] };
		Devices.initHardBtnPages();
		Devices.addHardBtnProps(tempDeviceData);
		Devices.setCurrentData(tempDeviceData);
		$state.go('menu.advanced.learn-ir.edit');
	};
});

/**
 * @ngdoc controller
 * @name EditIrController
 * @description
 *  button learn 화면
 */
app.controller('EditIrController', function ($scope, $state, toastr, Devices, Systems) {


	function handleSuccess(data) {
		$state.go('menu.advanced.options');
	}

	function handleError(error) {
		toastr.error(error, 'Error');
		$scope.finishBtnDisabled = false;
	}

	$scope.goBack = function () {
		if ($scope.learnIr.exists) {
			$state.go('menu.advanced.learn-ir.select-device');
		} else {
			$state.go('menu.advanced.learn-ir.new-device');
		}
	};

	$scope.finishBtnDisabled = false;
	$scope.finish = function () {
		$scope.finishBtnDisabled = true;

		var pages = [],
			hardPages = angular.copy($scope.learnIr.hard.pages),
			hardBtns = _.flatten(hardPages, 'btns'),
			menuPages = angular.copy($scope.learnIr.menu.pages),
			pageId = 0,
			buttonId = 8192;

		function rewriteMacro(btn) {
			var newMacro = {};
			if (btn.macro) {
				///Devices Service의 getOne function을 통해 Device정보를 얻어 올때 각 button의 macro data를 object로 변환하였으나
				///혹시 몰라서 macro가 string type인 경우에 대해서도 처리.
				newMacro = angular.isObject(btn.macro) ? btn.macro[0] : JSON.parse(btn.macro)[0];
			}

			if (btn.func) {
				if (_.isEmpty(newMacro)) {
					newMacro.type = 1;
					newMacro.device_id = parseInt($scope.learnIr.deviceData.id, 10);
				}

				console.log('rewriteMacro-btn=', btn);

				newMacro.func = btn.func.id;

				if (btn.repeatSetting === 'custom') {
					newMacro.repeat_min = btn.repeat_min;
					newMacro.repeat_macro = btn.repeat_macro;
				} else {
					delete newMacro.repeat_min;
					delete newMacro.repeat_macro;
				}

				if (!btn.press_hold) {
					newMacro.press_hold = 0;
				} else {
					newMacro.press_hold = 1;
				}
			}

			//console.log('rewriteMacro- newMacro=', newMacro);
			return JSON.stringify([newMacro]);
		}

		_.remove(hardBtns, function (button) {
			return !button.display_text;
		});

		_.forEach(hardBtns, function (hardBtn) {
			hardBtn.macro = rewriteMacro(hardBtn);
			hardBtn.page_btn_idx = -1;
			delete hardBtn.func;
			delete hardBtn.repeatSetting;
			delete hardBtn.repeat_min;
			delete hardBtn.repeat_macro;
			delete hardBtn.press_hold;
			delete hardBtn.name;
		});


		pages.push({ id: -1, btns: hardBtns });

		_.forEach(menuPages, function (page) {
			if (_.filter(page.btns, 'display_text').length) {
				page.id = pageId;
				pageId++;

				if (page.page_hidden == 1) {
					page.hidden = 1;
				} else {
					page.hidden = 0;
				}

				_.remove(page.btns, function (button) {
					return !button.display_text;
				});

				_.forEach(page.btns, function (btn) {

					if (btn.display_text) {

						btn.id = buttonId;
						buttonId++;

						btn.hidden = (true == btn.hidden ? 1 : 0);

						btn.macro = rewriteMacro(btn);

						delete btn.func;
						delete btn.repeatSetting;
						delete btn.repeat_min;
						delete btn.repeat_macro;
						delete btn.press_hold;
					}

				});

				pages.push(page);
			}
		});

		var data = angular.copy($scope.learnIr.deviceData);

		data.meta_pwr_on = data.meta_pwr_on || -1;
		data.meta_pwr_off = data.meta_pwr_off || -1;
		data.meta_pwr_toggle = data.meta_pwr_toggle || -1;
		data.meta_in_toggle = data.meta_in_toggle || -1;

		var jsonData;

		if ($scope.learnIr.exists) {
			_.assign(data, { pages: pages });
			jsonData = JSON.stringify({ data: data });
			Devices.storeOne($scope.learnIr.roomId, $scope.learnIr.deviceId, jsonData).then(handleSuccess, handleError);
		} else {
			var device = $scope.learnIr.device;
			_.assign(data, {
				pages: pages,
				name: device.name,
				category: device.category,
				brand: device.brand,
				model: device.model,
				target: device.target
			});
			jsonData = JSON.stringify({ data: data });
			console.log(data);

			// learn device 생성
			Devices.createCustomOne($scope.learnIr.room.id, jsonData)
				.then(function (data) {
					var id = Systems.getCurrentSystem().id;
					return Systems.updateSystem(id);
				})
				.then(handleSuccess)
				.catch(handleError);
		}

	};

	var currentRoom;
	if ($scope.$parent.learnIr && $scope.$parent.learnIr.room) {
		currentRoom = $scope.$parent.learnIr.room;
		console.log('currentRoom', currentRoom);
	} else if ($scope.learnIr) {
		currentRoom = _.where($scope.system.rooms, {
			id: $scope.learnIr.roomId
		})[0];
	}

	if (currentRoom) {
		var masterBstation = _.where(currentRoom.bstations, {
			master: 1
		})[0];
		console.log('masterBstation', masterBstation);
		$scope.masterBstaion = masterBstation;
	}
});

'use strict';
/**
 * @ngdoc controller
 * @name LoginController
 * @description
 *  로그인, 비밀번호 리셋 관련
 */
app.controller('LoginController', function ($rootScope, $scope, $state, $stateParams, toastr, Systems, Auth, User, Dealer) {
  // 로그인이 필요한 url 에 로그인 없이 접근하면 e=401 파리미터를 가지고 /login 으로 오는데 그때에 대한 처리
  if (!_.isEmpty($stateParams)) {
    if ($stateParams.e === '401') {
      $scope.error = '<div class="alert alert-warning"><i class="fa fa-lg fa-warning"></i>&nbsp;&nbsp;You must log in to access this page.</div>';
    }
  }

  // 이미 로그인한 사용자면 system-overview 메뉴로 이동시킴
  Auth.currentUser().then(function(user) {
    if(user) {
      $state.go('menu.system-overview');
    }
  });

  // dealer 로그인인지 검사
  $scope.dealerLogin = false;
  $scope.$watch('credentials.username', function (newVal, oldVal) {
    if (newVal && newVal.match('@')) {
      $scope.dealerLogin = true;
    } else {
      $scope.dealerLogin = false;
    }
  });

  // 로그인 화면에서 아이디/패스워드 입력 후 Sign in 버튼 클릭하면 실행되는 함수
	$scope.login = function() {
		var jsonData = JSON.stringify( $scope.credentials );  // 아이디/패스워드
		console.log(jsonData);

    // dealer 로그인과 일반 사용자 로그인을 구분
    // dealer
    if ($scope.dealerLogin) {
      Dealer.login(jsonData)
        .then( function(data) {
          if(data.result) {
            // dealer company 불러오기
            $rootScope.dealerCompany = Dealer.getDealerCompany();

            $state.go('dealer-jobs-list');
          } else {
            toastr.error(data.error);
          }
        })
        .catch(toastr.error);
    }
    // 일반 사용자
    else {
      // system-select 화면에서 dealer 에게 assign 된 system 인 경우 user-login-change 로 보내기 위해서 sessionStorage 에 저장
      sessionStorage.setItem('credentials', jsonData);

      // 로그인
      User.login(jsonData)
        .then( function(data) {
          if(data.result) {
            if (data.status) {  // result=true, status=true 일 때 로그인 처리
              if (data.systems.length) {  // 등록된 system 이 있는 경우
                Systems.loadSystems(data.systems);
                $state.go('system-select');
              } else {  // 등록된 system 이 없는 경우
                $scope.loginForm.$setPristine();
                $scope.credentials.password = '';
                toastr.warning('It seems you do not have any systems registered on your account. Please register one <a href="/#/register">here</a>.', 'Warning', { allowHtml: true });
              }
            } else {  // result=true, status=false 인 경우: forgot password 로 reset 했는데 base station 이 없거나 통신문제가 발생한 경우
              $rootScope.systemsInfo = angular.copy(data.info);
              $rootScope.credentials = angular.copy($scope.credentials);
              $state.go('ignore-offline-systems');
            }
          } else {
            if (data.code === 'ACCESS_DENY') {
              window.location = '/user-login-change#/auto-login';
            } else {
              toastr.error(data.error);
              $scope.credentials.password = '';
              $scope.loginForm.$setPristine();
            }
          }
        })
        .catch(toastr.error);
    }
	};

});

/**
 * @ngdoc controller
 * @name IgnoreOfflineController
 * @description
 *  forgot password 로 reset 했는데 base station 이 없거나 통신문제가 발생한 경우의 로그인 처리
 */
app.controller('IgnoreOfflineController', function($rootScope, $scope, $state, $stateParams, $modal, toastr, User, Systems) {
  // LoginController 에서 IgnoreOfflineController 로 데이터를 전달하기 위해 $rootScope 를 사용하였으나 전달이 끝나면 $rootScope 의 프로퍼티를 삭제
  $scope.systemsInfo = angular.copy($rootScope.systemsInfo);
  $scope.credentials = angular.copy($rootScope.credentials);
  delete($rootScope.systemsInfo);
  delete($rootScope.credentials);

  $scope.forceIgnore = false;
  $scope.ignore = false;
  $scope.processing = false;

  _.each($scope.systemInfo, function(obj) {
    if (obj.connection == 2) {  // 2의 값이 하나라도 나온 경우는 system들이 복구가 안되는 상황이므로 반드시 ignore 하여 로그인해야함
      $scope.forceIgnore = true;
      $scope.ignore = true;
    }
  });

  $scope.goBack = function() {
    $state.go('login');
  };

  $scope.doNext = function() {
    var modal = $modal({
      scope: $scope,
      template: 'views/modal-tpls/ignore-offline.html',
      container: 'body',
      show: false
    });
    modal.$promise.then(modal.show);
  };

  $scope.reset = function() {
    if ($scope.ignore) {  // ignore 에 체크한 경우 bstationChk=0 값을 넣어서 로그인
      $scope.credentials.bstationChk = 0;
    }

    $scope.processing = true;
    login($scope.credentials);
  };

  function login(credentials) {
    var jsonData = JSON.stringify(credentials);

    User.login(jsonData)
      .then( function(data) {
        if(data.result) {
          if (data.status) {
            if (data.systems.length) {
              Systems.loadSystems(data.systems);
              $state.go('system-select');
            } else {
              toastr.warning('It seems you do not have any systems registered on your account. Please register one <a href="/#/register">here</a>.', 'Warning', { allowHtml: true });
            }
          } else {
            $rootScope.systemsInfo = angular.copy(data.info);
            $rootScope.credentials = angular.copy($scope.credentials);
            $state.go('ignore-offline-systems', {}, {reload: true});
          }
        } else {
          toastr.error(data.error);
        }

        $scope.processing = false;
      })
      .catch(toastr.error);
  }
});

/**
 * @ngdoc controller
 * @name IgnoreOfflineCreateController
 * @description
 *  forgot password 로 reset 했는데 base station 이 없거나 통신문제가 발생한 경우의 system 생성을 위한 로그인 처리
 */
app.controller('IgnoreOfflineCreateController', function($rootScope, $scope, $state, $stateParams, $modal, toastr, User, Registration) {
  // RegisterController 에서 IgnoreOfflineController 로 데이터를 전달하기 위해 $rootScope 를 사용하였으나 전달이 끝나면 $rootScope 의 프로퍼티를 삭제
  $scope.systemsInfo = angular.copy($rootScope.systemsInfo);
  $scope.credentials = angular.copy($rootScope.credentials);
  delete($rootScope.systemsInfo);
  delete($rootScope.credentials);

  $scope.forceIgnore = false;
  $scope.ignore = false;
  $scope.processing = false;

  _.each($scope.systemInfo, function(obj) {
    if (obj.connection == 2) {  // 2의 값이 하나라도 나온 경우는 system들이 복구가 안되는 상황이므로 반드시 ignore 하여 로그인해야함
      $scope.forceIgnore = true;
      $scope.ignore = true;
    }
  });

  $scope.goBack = function() {
    $state.go('login');
  };

  $scope.doNext = function() {
    var modal = $modal({
      scope: $scope,
      template: 'views/modal-tpls/ignore-offline.html',
      container: 'body',
      show: false
    });
    modal.$promise.then(modal.show);
  };

  $scope.reset = function() {
    if ($scope.ignore) {  // ignore 에 체크한 경우 bstationChk=0 값을 넣어서 로그인
      $scope.credentials.bstationChk = 0;
    }

    $scope.processing = true;
    loginForCreate($scope.credentials);
  };

  function loginForCreate(credentials) {
    var jsonData = JSON.stringify(credentials);

    User.loginForCreate(jsonData).then(function(data) {
      if(data.result) {
        if (data.status) {
          Registration.setSystemKey($scope.credentials.key);
          Registration.setBStationMac(data.mac);
          Registration.setSystemType($scope.credentials.modelType);

          $state.go('add-new-system');
        } else {
          $rootScope.systemsInfo = angular.copy(data.info);
          $rootScope.credentials = angular.copy($scope.credentials);
          $state.go('ignore-offline-systems-for-create', {}, {reload: true});
        }
      } else {
        toastr.error(data.error);
      }

      $scope.processing = false;
    }).catch(toastr.error);
  }
});

/**
 * @ngdoc controller
 * @name ForgotController
 * @description
 *  로그인 화면 하단의 "Forgot your password?" 를 클릭해서 들어온 경우 password reset 처리
 */
app.controller('ForgotController', function ($scope, $state, toastr, Auth, User){
  $scope.forgot = {};

  // 로그인 된 상태에서 접근하면 system-overview 로 이동
  Auth.currentUser().then(function(user) {
    if(user) {
      $state.go('menu.system-overview');
    }
  });

  $scope.resetPassword = function() {

    var jsonData = JSON.stringify( $scope.forgot );
    console.log(jsonData);

    // reset password
    User.resetPassword($scope.forgot)
      .then(function() {
        toastr.success('Your password has been reset.');
        $state.go('login');
      })
      .catch(toastr.error);
    
  };
});

'use strict';
/**
 * @ngdoc controller
 * @name MainMenuLayoutController
 * @description
 *  entertainment main screen layout 편집 화면
 */
app.controller('MainMenuLayoutController', function ($rootScope, $scope, $state, $filter, $timeout, toastr, Rooms, Systems, Devices) {
	// scheduled event 기능을 eng, alpha 버전에서만 보이게 하기 위한 처리
	if ($rootScope.version.substr(-1) === 'e' || $rootScope.version.substr(-1) === 'a') {
		$scope.isAlpha = true;
	} else {
		$scope.isAlpha = false;
	}

	$scope.mainMenu.changingIcon = false;
	$scope.mainMenu.mainDevice.btns = [];
	$scope.mainMenu.mainDevice.id = Systems.getMainDevice().id;

	$scope.mainMenu.showBtnsType = 'devices';  // devices/events

	// icon 목록이 있는 json 파일로부터 데이터 얻기
	Devices.getInfoJson().success(function (data) {
		Devices.setIcons(data);
		$scope.mainMenu.deviceIcons = Devices.getIcons();

		console.log($scope.mainMenu.deviceIcons);
	}).error(function () {

	});

	// (1) mainDevice btns

	var btnIndexOffset = 0;

	_.forEach(Systems.getMainDevice().pages, function(page) {
		 _.forEach(page.btns, function(btn) {
			console.log(btn);
			btn.hidden_option = btn.hidden_option ? true:false;
			btn.hidden = btn.hidden ? true:false;
			btn.page_btn_idx += btnIndexOffset;
			$scope.mainMenu.mainDevice.btns.push(btn);
		 });
		 btnIndexOffset += 6;
	});

	$scope.mainMenu.mainDevice.btns = _($scope.mainMenu.mainDevice.btns).sortBy('page_btn_idx').value();
	console.log($scope.mainMenu.mainDevice.btns);

	$scope.deleteDevice = function (btn) {
		_.remove($scope.mainMenu.mainDevice.btns, { id: btn.id });
	};

	$scope.makeCopy = function(obj) {
		return angular.copy(obj);
	};

	$scope.isEmpty = function(obj) {
		return _.isEmpty(obj);
	};


	$scope.switchToImageView = function() {
		$scope.mainMenu.selectedImgName = angular.copy($scope.mainMenu.selectedBtn.image_name);
		$scope.mainMenu.changingIcon = true;
	};
	
	$scope.saveImage = function() {
		$scope.mainMenu.selectedBtn.image_name = angular.copy($scope.mainMenu.selectedImgName);
		$scope.mainMenu.changingIcon = false;
	};

	$scope.replaceButton = function (btnId) {
		var idx = _.findIndex($scope.mainMenu.mainDevice.btns, {'id': btnId});
		$scope.mainMenu.mainDevice.btns[idx] = $scope.mainMenu.selectedBtn;
	};

	$scope.clearSelectedBtn = function() {
		$timeout(function() {
				$scope.mainMenu.selectedBtn = {};
		}, 200);
	};

	// (2) scheduled events

	// scheduled event 목록 얻기
	Rooms.getEvents(Systems.getCurrentSystem().id, $scope.mainMenu.room.id).then(function(data) {
		$scope.mainMenu.scheEvents = data;

    _.forEach($scope.mainMenu.scheEvents, function(event) {
      event.hidden_option = event.hidden_option ? true:false;
      event.hidden = event.hidden ? true:false;
    });
	}).catch(function(error) {
		alert(error);
	});

	$scope.deleteEvent = function (event) {
		_.remove($scope.mainMenu.scheEvents, { id: event.id });

		var jsonData = JSON.stringify({ id: event.id });

		Rooms.deleteEvent(Systems.getCurrentSystem().id, $scope.mainMenu.room.id, jsonData)
			.then(function(data) {
				Systems.updateSystem(Systems.getCurrentSystem().id);
			})
			.catch(toastr.error);
	};

	$scope.switchToEventImageView = function() {
		$scope.mainMenu.selectedEventImgName = angular.copy($scope.mainMenu.selectedEvent.image_name);
		$scope.mainMenu.changingEventIcon = true;
	};

	$scope.saveEventImage = function() {
		$scope.mainMenu.selectedEvent.image_name = angular.copy($scope.mainMenu.selectedEventImgName);
		$scope.mainMenu.changingEventIcon = false;
	};

	$scope.replaceEvent = function (eventId) {
		var idx = _.findIndex($scope.mainMenu.scheEvents, {'id': eventId});
		$scope.mainMenu.scheEvents[idx] = $scope.mainMenu.selectedEvent;
	};

	$scope.clearSelectedEvent = function() {
		$timeout(function() {
			$scope.mainMenu.selectedEvent = {};
		}, 200);
	};

	// (3) 공통

  $scope.finish = function () {
    // devices
    function saveDevices() {
      var numPages = Math.ceil($scope.mainMenu.mainDevice.btns.length / 6);
      var pages = [];

      for (var pageId = 0; pageId < numPages; pageId++) {
        var page = {id: pageId};
        var start = pageId * 6;

        page.btns = $scope.mainMenu.mainDevice.btns.slice(start, start + 6);
        for (var index = 0; index < page.btns.length; index++) {
          page.btns[index].page_btn_idx = index;
          page.btns[index].hidden_option = page.btns[index].hidden_option ? 1 : 0;
          page.btns[index].hidden = page.btns[index].hidden ? 1 : 0;
          console.log('finish- page.btn=', page.btns[index])
        }
        pages.push(page);
      }
      console.log(pages);

      Systems.setMainDevice({id: $scope.mainMenu.mainDevice.id, pages: pages});

      // main device 정보 서버에 저장
      Systems.storeMainDevice($scope.mainMenu.room.id).then(function (data) {
        saveEvents();
      }, function (error) {
        alert(error);
      });
    }

    // events
    function saveEvents() {
      _.forEach($scope.mainMenu.scheEvents, function (event, i) {
        event.hidden_option = event.hidden_option ? 1 : 0;
        event.hidden = event.hidden ? 1 : 0;
        event.order = i;
      });

      var jsonData = JSON.stringify({data: $scope.mainMenu.scheEvents});

      Rooms.storeEvents(Systems.getCurrentSystem().id, $scope.mainMenu.room.id, jsonData).then(function (data) {
        Systems.updateSystem(Systems.getCurrentSystem().id).then(
          function (data) {
            console.log(data);
						//$state.transitionTo($state.current, null, {reload: true, inherit: true, notify: true});
						$state.go('menu.advanced.options');
          },
          function (error) {
            alert(error);
          }
        );
      }, function (error) {
        alert(error);
      });
    }

    // Finish 버튼을 클릭하면 먼저 devices 를 저장하고 그 다음에 events 를 저장함
    saveDevices();
	};
});

'use strict';
/**
 * @ngdoc controller
 * @name PowerOffController
 * @description
 *  power off 설정 화면 (아래 controller 들의 state 를 포함하는 state 의 controller)
 */
app.controller('PowerOffController',  function ($scope, room, Devices) {
  Devices.initFunctions();
  $scope.powerOff = { 'devices': [], 'device': {} };
  $scope.powerOff.index = 0;

  $scope.room = room;
});

/**
 * @ngdoc controller
 * @name PowerOffDevicesController
 * @description
 *  device 선택 화면
 */
app.controller('PowerOffDevicesController', function ($scope, $state, $http, Systems, Devices, Auth, currentMacro) {

  var currentSystem = Systems.getCurrentSystem(),
       dataActivity = currentMacro;
  console.log(dataActivity);

  if(dataActivity && dataActivity.edit_type === 0 && dataActivity.activity.length ) {
    _.forEach(dataActivity.activity, function(data) {
      var device = Systems.getDevice($scope.room.id, data.device_id);

      if (device) {
        $scope.powerOff.devices.push({
          'device_id': data.device_id,
          'name': device.name,
          'type': data.type,
          'func': data.func,
          'repeat_min': device.repeat_min,
          'repeat_macro': device.repeat_macro,
          'target_type': device.target_type,
          'target_addr': device.target_addr,
          'target_port': device.target_port
        });
      }
    });
  }
  // else {
  //   _.forEach($scope.room.devices, function(device) {
  //     if ( _.isEmpty($scope.powerOff.devices)) {
  //       $scope.powerOff.devices.push({
  //         'device_id': device.id,
  //         'name': device.name,
  //         'type': 0,
  //         'func': null,
  //         'repeat_min': device.repeat_min,
  //         'repeat_macro': device.repeat_macro,
  //         'target_type': device.target_type,
  //         'target_addr': device.target_addr,
  //         'target_port': device.target_port,
  //       });
  //     }
  //   });
  // }

  $scope.doNext = function() {

    $state.go('menu.programming.power-off.commands');
  };

  Auth.currentUser().then(function(user) {
    $scope.optMenuShow = user.optMenuShow;
  });

  $scope.goAdvanced = function() {
    $state.go('menu.advanced.add-macros.macro-type');
  };

});

/**
 * @ngdoc controller
 * @name PowerOffDeviceCommandsController
 * @description
 *  command 선택 화면
 */
app.controller('PowerOffDeviceCommandsController', function ($scope, $state, Systems, Devices) {

  $scope.goBack = function() {
    if ($scope.powerOff.index === 0) {
      $scope.powerOff.devices = [];
      Devices.initFunctions();
      $state.go('menu.programming.power-off.devices-to-use' );
    } else {
      $scope.powerOff.index--;
    }
  };


  $scope.incrIdx = function() { $scope.powerOff.index++; };

  $scope.finish = function() {
    //Save to server

    var sendData = angular.copy($scope.powerOff.devices);

    for (var i=0,  tot=sendData.length; i < tot; i++) {
      sendData[i].func = sendData[i].func.id;
      delete sendData[i].name;
      delete sendData[i].repeat_min;
      delete sendData[i].repeat_macro;
      delete sendData[i].target_type;
      delete sendData[i].target_addr;
      delete sendData[i].target_port;
      delete sendData[i].funcs;
    }

    var jsonData = JSON.stringify( { auto_wait : 1, edit_type : 0, popup : 1, activity : sendData, macro : [] } );
    console.log(jsonData);

    // 해당 room 의 power macro 데이터 저장
    Systems.storePowerMacro($scope.room.id, jsonData).then(function(response) {
      if(response.data.result === true) {
        $state.go('menu.programming.options');
      }
      else {
        alert('Your edit/create action failed. Please try again.');
      }
    }, function(error){
      alert("Please try again");
    });
  };

});

'use strict';
/**
 * @ngdoc controller
 * @name RegisterController
 * @description
 *  새로운 system 을 등록
 */
app.controller('RegisterController', function ($rootScope, $scope, $state, toastr, availableModels, Registration, Auth, User, Dealer) {

  // 로그인 된 상태에서 접근하면 system-overview 로 이동
  Auth.currentUser().then(function(user) {
    if(user) {
      $state.go('menu.system-overview');
    }
  });

  // 초기값 설정
  $scope.register = {
    option: 'create',
    modelType: availableModels[0]
  };
  $scope.availableModels = availableModels;

  $scope.doNext = function() {
    if ($scope.register.addDealerAccount) {
      Dealer.verifyDealer(JSON.stringify({
        username: $scope.register.dealerUsername,
        password: $scope.register.dealerPassword
      })).then(function (data) {
        if (data.result) {
          Registration.setDealerAccount({
            username: $scope.register.dealerUsername,
            password: $scope.register.dealerPassword
          });

          doNextStep();
        } else {
          toastr.error('The Dealer/Employee username or password is incorrect. Please check your login credentials.');
        }
      }).catch(toastr.error);
    } else {
      doNextStep();
    }

    function doNextStep() {
      var key = $scope.register.key.first + $scope.register.key.second + $scope.register.key.third + $scope.register.key.fourth,
        key = key.toUpperCase(),
        type = $scope.register.modelType;
      console.log(key);

      if ($scope.register.option === 'create') {    // 사용자 생성을 하면서 system 생성
        Registration.setSystemKey(key);
        Registration.setSystemType(type);
        $state.go('send-email');
      } else if ($scope.register.option === 'existing') {   // existing account 로 로그인 하면서 system 생성

        var jsonData = JSON.stringify({
          key: key,
          modelType: type,
          username: $scope.register.username,
          password: $scope.register.password
        });
        console.log(jsonData);

        // 시스템 생성을 위한 로그인
        User.loginForCreate(jsonData).then(function (data) {
          if (data.result) {
            if (data.status) {  // result 와 status 가 모두 true 일 때
              Registration.setSystemKey(key);
              Registration.setBStationMac(data.mac);
              Registration.setSystemType(type);

              $state.go('add-new-system');
            } else {  // status=false: forgot password 로 reset 했는데 base station 이 없거나 통신문제가 발생한 경우
              $rootScope.systemsInfo = angular.copy(data.info);
              $rootScope.credentials = {
                key: key,
                modelType: type,
                username: $scope.register.username,
                password: $scope.register.password
              };
              $state.go('ignore-offline-systems-for-create');
            }
          } else {
            toastr.error(data.error);
            $scope.register.password = '';
            $scope.registerForm.$setPristine();
          }
        }).catch(toastr.error);
      }
    }
	};
});

/**
 * @ngdoc controller
 * @name SendEmailController
 * @description
 *  사용자 등록시 temporary password 발송
 */
app.controller('SendEmailController', function ($scope, $state, toastr, Registration, Auth) {

  // 로그인 된 상태에서 접근하면 system-overview 로 이동
  Auth.currentUser().then(function(user) {
    if(user) {
      $state.go('menu.system-overview');
    }
  });

	var keyData = Registration.getSystemKey(),
      type = Registration.getSystemType();

	$scope.doNext = function() {

		var emailData = $scope.email;

		var jsonData = JSON.stringify( { modelType: type, key: keyData, email: emailData });
		console.log(jsonData);

    // 사용자 등록시 temporary password 를 요청한 email 로 보내기
    Registration.sendTempPswd(jsonData).then(function(data) {
      Registration.setEmail(emailData);
      $state.go('add-user');
    })
    .catch(toastr.error);
	};
});



app.controller('AddUserController', function ($scope, $timeout, $state, toastr, Registration, Auth, User) {

  // 로그인 된 상태에서 접근하면 system-overview 로 이동
  Auth.currentUser().then(function(user) {
    if(user) {
      $state.go('menu.system-overview');
    }
  });

  $scope.doNext = function() {
		var user = {};
  	user.username = $scope.user.username;
		user.password = $scope.user.password;
		user.tempPassword = $scope.user.tempPassword;
		user.userEmail = $scope.user.email;
		user.email = Registration.getEmail();
		user.key = Registration.getSystemKey();
		user.modelType = Registration.getSystemType();

		var jsonData = JSON.stringify(user);
		console.log(jsonData);

		User.create(jsonData).
			then(function(data) {
				Registration.setBStationMac(data.mac);
        
        $timeout(function() {
          $state.go('add-new-system');  
        }, 500);
        		
			}).
			catch(toastr.error);
	};
});

/**
 * @ngdoc controller
 * @name AddNewSystemController
 * @description
 *  사용자 생성
 */
app.controller('AddNewSystemController', function ($rootScope, $scope, $state, toastr, timeZones, Registration, Systems, User, Dealer) {

  $scope.baseStationMac = Registration.getBStationMac();
  $scope.timeZones = timeZones;
  $scope.system = {};
  $scope.system.timezone_dst = true;
  $scope.timeFormat = 'hh:mm:ss a';
  //$scope.system.timeZone = null;

  // DST: daylight saving time true/false 설정
  $scope.checkDST = function() {
    if ($scope.system.timeZone.s_time === '0.0.0 00:00:00') {
      $scope.system.timezone_dst = false;
    } else {
      $scope.system.timezone_dst = true;
    }
  };

 	$scope.doNext = function() {

 		var systemName = $scope.system.name,
          roomName = $scope.system.roomName,
   customerAddress = $scope.system.customerAddress,
     customerPhone = $scope.system.customerPhone,
      customerName = $scope.system.customerName,
     installerName = $scope.system.installerName,
           macAddr = Registration.getBStationMac(),
           keyData = Registration.getSystemKey(),
              type = Registration.getSystemType();

    var jsonData = JSON.stringify({
      systemName: systemName,
      modelType: type,
                                      key: keyData,
                                      timezone_id:  $scope.system.timeZone.id,
                                      timezone_dst: $scope.system.timezone_dst ? 1:0,
                                      time_type: $scope.timeFormat === 'hh:mm:ss a' ? 0:1 ,
                                      roomName: roomName,
                                      customerAddress: customerAddress,
                                      customerPhone: customerPhone,
                                      customerName: customerName,
                                      installerName: installerName,
      bstationMac: macAddr,
                                      bstationName: type + ' ' + macAddr.slice(10,12)  } );

		console.log(jsonData);

    $scope.systemId = null;

    // system 생성
	  Systems.createOne(jsonData)
      .then(function(data) {
        $scope.systemId = data;

        var dealerAccount = Registration.getDealerAccount();

        if (dealerAccount) {
          var jsonData = JSON.stringify(dealerAccount);

          Systems.setDealer($scope.systemId, jsonData)
            .then(function (data) {
              if (data.result) {

              } else {
                toastr.error(data.error);
              }
            })
            .catch(toastr.error);
        }

  			return Systems.fetch();
      })
      .then(function(data) {
        var dealerAccount = Registration.getDealerAccount();

        // 일반 사용자에서 logout 하고 delaer 로 로그인 후 system 선택 후 home overview 화면으로 이동
        // system 선택 후 home overview 화면으로 이동하는 코드는 DealerJobsListController 의 $scope.selectSystem function 과 같음
        if (dealerAccount) {
          var jsonData = JSON.stringify(dealerAccount);

          User.logout();

          Dealer.login(jsonData)
            .then(function (data) {
              if (data.result) {
                // dealer company 불러오기
                $rootScope.dealerCompany = Dealer.getDealerCompany();

                Dealer.getSystemToken(JSON.stringify({system_id: $scope.systemId})).then(function (data) {
                  if (data.result) {
                    // token 갱신
                    var token = data.token;
                    Dealer.setUser($scope.systemId, token, false, false, true);   // dealer token 보존

                    User.getUserInfo().then(function (data) {
                      // token 갱신
                      Dealer.setUser($scope.systemId, token, data.username, data.email);

                      // system select
                      Systems.setCurrentRoom(null);

                      Systems.updateSystem($scope.systemId).then(
                        function () {
                          // system 이 dealer 에게 assign 되는 경우에 최초로 system-select 해서 system-overview 들어가는 경우 녹색 toastr 띄우기 위한 설정
                          $state.go('menu.system-overview', {dealer: $scope.systemId});
                        },
                        function (error) {
                          toastr.error(error);
                        }
                      );
                    }).catch(toastr.error);
                  } else {
                    toastr.error(data.error);
                  }
                })
                  .catch(toastr.error);
              } else {
                toastr.error(data.error);
              }
            })
            .catch(toastr.error);
        } else {
          $state.go('system-select');
        }
      })
      .catch(function(error) {
        toastr.error(error, 'Error');
      });
	};
});

'use strict';


app.controller('SystemSelectController', function ($scope, $state, $stateParams, $modal, toastr, resolvedSystems, Devices, Systems) {

  	$scope.systems = resolvedSystems;

    $scope.doNext = function(){
      Systems.setCurrentRoom(null);

      var selectedSystem = _.findWhere($scope.systems, {
        id: $scope.choice
      });

      // dealer 에게 assign 된 system 인 경우
      if (selectedSystem.dealer_id) {
        // user-login-change 로 이동시킴
        window.location = '/user-login-change#/auto-login/' + $scope.choice + '/' + selectedSystem.dealer_id;
      }
      // 일반 system
      else {
        return Systems.updateSystem($scope.choice).then(
          function (data) {
            var masterBstation;
            _.each(data.rooms, function(room) {
              if (!masterBstation) {
                masterBstation = _.where(room.bstations, {
                  master: 1
                })[0];
              }
            });
            if (!masterBstation) {
              $state.go('recovery-bstation', {rooms: angular.toJson(data.rooms)});
              return;
            }

            // TRF-ZW data update
            _.each(data.rooms, function(room) {
              var trfzw = _.find(room.twoway_devices, {model: 'TRF-ZW1'});
              if (trfzw) {
                Devices.updateTwowayDevice(room.id, trfzw.id)
                  .then(function(data) {
                    trfzw.data = data;
                  }, function() {
                    var modal = $modal({
                      container: '.editor-container',
                      template: 'views/modal-tpls/connection-problem.html',
                      show: false
                    });

                    modal.$promise.then(modal.show);
                  })
                  .catch(toastr.error);

                return false;
              }
            });

            if ($stateParams.dealer) {
              $state.go('menu.system-overview', {dealer: $stateParams.dealer});
            } else {
              $state.go('menu.system-overview');
            }
          },
          function (error) {
            toastr.error(error);
          }
        );
      }
    };

});

/**
 * @ngdoc controller
 * @name SystemDeleteController
 * @description
 *  system 목록 표시, system 삭제 화면
 */
app.controller('SystemDeleteController', function ($rootScope, $scope, $state, toastr, resolvedSystems, Systems, User) {

    var currentSystemId = Systems.getCurrentSystem().id;
  $scope.currentSystemId = currentSystemId;

    $scope.systems = resolvedSystems;
    $scope.sysToDel = {};

    $scope.deleteSystem = function(){

      // system 삭제
      Systems.deleteSystem($scope.sysToDel.id).then(function (data) {
          // dealer 로그인 상태이면서 삭제하려는 system 이 현재 system 인 경우
          if ($rootScope.dealerCompany && currentSystemId === $scope.sysToDel.id) {
            $state.go('dealer-jobs-list');
          } else {
            // 남은 system 이 없는 경우 system 등록 화면으로 이동
            if (Systems.getAll().length === 0) {
              User.logout().then(function () {
                $state.transitionTo('register', null, {reload: true, inherit: true, notify: true});
              });
            }
            // 삭제한 system 이 현재 system 이었던 경우 system 선택 화면으로 이동
            else if (currentSystemId === $scope.sysToDel.id) {
              $state.go('system-select');
            }
            // 삭제한 system 이 현재 system 이 아니었던 경우 현재 system 의 home overview 화면으로 이동
            else {
              $state.go('menu.system-overview');
            }
          }
        },
        function (error) {
          toastr.error(error);
        }
      );
    };

});

/**
 * @ngdoc controller
 * @name SystemSettingsController
 * @description
 *  system 설정 메인 화면
 */
app.controller('SystemSettingsController', function ($scope, $modal, $http, $window, timeZones, currentSystem, Systems, Auth) {

  $scope.factoryUser = false;

  $scope.currentUser = JSON.parse($window.localStorage.getItem('hcm_editor'));

  // 현재 로그인한 사용자가 factory_user.json 에 있는지 검사
  $http.get('/scripts/factory_user.json')
    .success(function(response) {
      if (response && response.users) {
        Auth.currentUser().then(function(user) {
          if (response.users.indexOf(user.username) > -1) {
            $scope.factoryUser = true;
          }
        });
      }
    });

  $scope.goNewLinkCode = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      container: '#system-settings',
      template: 'views/modal-tpls/external-service/new-link-code.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.genNewLinkCode = function () {
    var currentSystemId = Systems.getCurrentSystem().id;
    Systems.genNewLinkCode(currentSystemId)
    .then(function (data) {
      var modal = $modal({
        scope: $scope,
        title: '',
        container: '#system-settings',
        template: 'views/modal-tpls/external-service/new-link-code-success.html',
        show: false
      });
  
      modal.$promise.then(modal.show);
    })
    .catch(function (error) {
      toastr.error(error);
    });

  };

});

/**
 * @ngdoc controller
 * @name ChangeMasterFromFactoryController
 * @description
 *  factory master change 화면
 */
app.controller('ChangeMasterFromFactoryController', function ($rootScope, $scope, $state, toastr, searchResults, Systems) {
  $scope.baseStations = searchResults.searchedDevices;
  $scope.bsReplace = {};

  var systemId = Systems.getCurrentSystem().id;

  var orgMac = Systems.getCurrentSystem().rooms[0].bstations[0].mac;

  $scope.doNext = function() {
    var jsonData = JSON.stringify({ modelType: $scope.bsReplace.target.name, mac: $scope.bsReplace.target.mac, org_mac: orgMac });
    Systems.changeMasterFromFactory(systemId, jsonData)
      .then(function(data) {
        Systems.updateSystem(systemId).then(function () {
          $state.go('system-settings.change-master-from-factory-success');
        });
      })
      .catch(toastr.error);
  };
});

app.controller('ChangeMasterFromFactorySuccessController', function ($rootScope, $scope, $state) {
  $scope.doNext = function() {
    $state.go('system-settings.main');
  };
});

/**
 * @ngdoc controller
 * @name ReplaceMainController
 * @description
 *  변경할 base station 선택 화면
 */
app.controller('ReplaceMainController', function  ($scope, $state){

  $scope.doNext = function () {
    if ($scope.bsReplace.selection.master) {
      $state.go('system-settings.replace.master');
    } else {
      $state.go('system-settings.replace.slave');
    }
  };
});

app.controller('ReplaceMasterController', function  ($rootScope, $scope, $state, Systems, availableModels){
  // 기존의 master base station 의 priority 이상인 base station 의 목록만 표시하기 위해 아래와 같이 처리
  var basePriority,
    bsModels = [];
  _.each($rootScope.env.bstationPriority, function (value, key) {
    if (value.indexOf($scope.bsReplace.selection.type) > -1) {
      basePriority = key;
    }
  });
  _.each(availableModels, function (item) {
    _.each($rootScope.env.bstationPriority, function (value, key) {
      if (value.indexOf(item) > -1 && key >= basePriority) {
        bsModels.push(item);
      }
    });
  });

  $scope.bsReplace.modelType = bsModels[0];
  $scope.availableModels = bsModels;


  $scope.doNext = function () {
    var key = $scope.bsReplace.key.first + $scope.bsReplace.key.second + $scope.bsReplace.key.third + $scope.bsReplace.key.fourth,
        key = key.toUpperCase(),
        type = $scope.bsReplace.modelType,
        jsonData = JSON.stringify({ modelType: type, key: key});

    console.log(jsonData);
    Systems.checkKey(jsonData).then(function(data) {
      console.log(data);
      $scope.bsReplace.target = data;
      $state.go('system-settings.replace.target');
    })
    .catch(function(error) {
      alert(error);
    });
  };
  
});

app.controller('ReplaceSlaveController', function  ($rootScope, $scope, $state, searchResults){
  $scope.baseStations = searchResults.searchedDevices;

  var bsReplaceRoom = null;
  var findBS = null;
  _.each($scope.system.rooms, function (room) {
    findBS = _.find(room.bstations, { id: $scope.bsReplace.selection.id });
    if(findBS) {
      bsReplaceRoom = room;
      return false;
    }
  });

  $scope.bsReplace.room = bsReplaceRoom;

  //$scope.bsReplace가 virtual base station이거나, 속한 room에 다른 basestation이 존재한다면 virtual base station이 포함되지 않도록 처리.
  if('VIRTUAL' != $scope.bsReplace.selection.type && 2 > bsReplaceRoom.bstations.length) {
    var virtualBS = {
      name: 'Virtual Base Station',
      mac: '000000000000',
    };
    $scope.baseStations.push(virtualBS);  
  }

  $scope.bsType = 'BASESTATIONS';

  $scope.doNext = function () {
    //$scope.bsReplace.target = angular.copy($scope.bsReplace.bsSelect);
    $state.go('system-settings.replace.target');
  };
});

app.controller('ReplaceTargetController', function  ($scope, $state, toastr, Systems, Rooms){
  // base station 이름 변경: modelType + macAddress 맨 끝 2자리 ex) HS-C100 EE
  function editBstation() {
    var roomId,
      bstation = {},
      orgMac = $scope.bsReplace.selection.mac,
      mac,
      type;

    _.each($scope.system.rooms, function(room) {
      _.each(room.bstations, function(bstation) {
        if (bstation.mac == orgMac) {
          roomId = room.id;
        }
      });
    });

    if ($scope.bsReplace.selection.master) {
      mac = $scope.bsReplace.key.first + $scope.bsReplace.key.second + $scope.bsReplace.key.third + $scope.bsReplace.key.fourth;
      mac = mac.toUpperCase();
      mac = mac.substr(4);
      type = $scope.bsReplace.modelType;
    } else {
      mac = $scope.bsReplace.target.mac;
      type = ($scope.bsReplace.target.mac == '000000000000' ? 'VIRTUAL' : $scope.bsReplace.target.name);
    }

    bstation.mac = mac;
    bstation.type = type;
    bstation.name = (bstation.mac == '000000000000' ? $scope.bsReplace.target.name : type + ' ' + mac.substr(10));
    bstation.light_level = 2;

    var jsonData = JSON.stringify( { name : bstation.name, mac: bstation.mac, type: bstation.type, lightLevel: bstation.light_level } );
    // base station 의 type, name, lightLevel 을 설정

    Rooms.baseStations('edit', roomId, jsonData).
      then(function(data) {

      }).
      catch(toastr.error);
  }

  var changeMaster = function(json) {
    return Systems.changeMaster(json).then(function(data) { 
      return Systems.getCurrentSystem().id;
    });
  };

  var changeSlave = function(json) {
    return Systems.changeSlave(json).then(function(data) { 
      return Systems.getCurrentSystem().id;
    });
  };

  var updateSystem = function(id) {
    editBstation();

    return Systems.updateSystem(id).then(function (data) {
      console.log(data);
      return $state.transitionTo('system-settings.replace.prompt', {}, { reload: true, inherit: true, notify: true  });
    });
  };


  $scope.replace = function () {

    if ($scope.bsReplace.selection.master) {
      var key = $scope.bsReplace.key.first + $scope.bsReplace.key.second + $scope.bsReplace.key.third + $scope.bsReplace.key.fourth,
          key = key.toUpperCase(),
          type = $scope.bsReplace.modelType;


      changeMaster(JSON.stringify({ org_mac: $scope.bsReplace.selection.mac, modelType: type, key: key}))
        .then(updateSystem)
        .catch(function (error) {
          alert(error);
        });
      

    } else {

      changeSlave(JSON.stringify({ org_mac: $scope.bsReplace.selection.mac, new_mac: $scope.bsReplace.target.mac, modelType: $scope.bsReplace.target.name, org_room_id: $scope.bsReplace.room.id }))
        .then(updateSystem)
        .catch(function (error) {
          alert(error);
        });
    }
    
  };
});

app.controller('ReplacePromptController', function  ($scope, $state){
  
});

/**
 * @ngdoc controller
 * @name SystemOverviewController
 * @description
 *  system overview 화면
 */
app.controller('SystemOverviewController', function ($rootScope, $scope, $stateParams, $http, $modal, $popover, $state, toastr, types, remoteTypes, Systems, Rooms, Database, Devices, ExternalService) {
  $scope.types = types;

  // scheduled event 기능을 eng, alpha 버전에서만 보이게 하기 위한 처리
  if ($rootScope.version.substr(-1) === 'e' || $rootScope.version.substr(-1) === 'a') {
    $scope.isAlpha = true;
  } else {
    $scope.isAlpha = false;
  }

  $scope.system = Systems.getCurrentSystem();
  $scope.selections = { 'selectedRoom': {}, 'selectedBstation': {}, 'selectedRemote': {}, 'selectedDevice': {}};

  if ($stateParams.dealer == $scope.system.id) {
    toastr.success('This system has been successfully assigned to your Dealer/Employee account.');
  }

  // remote 정보에 type 을 넣어줌
  _.each($scope.system.rooms, function (room) {
    _.each(room.remotes, function (remote) {
      var obj = _.find(remoteTypes, {
        name: remote.type
      });

      remote.remoteType = obj.type;
    });
  });

  var favoriteOptionModal = $modal({
    scope: $scope,
    title: 'Add Favorite Channels to this device',
    placement: 'top',
    template: 'views/modal-tpls/system-overview/fav-option.html',
    container: '#system-overview',
    show: false
  });

  $scope.popover = { 'device': '', 'twowayDevice': ''};

  $scope.findPunchDevice = function(room) {
    console.log('punch vol id', $scope.popover.device.punch_vol);
    $scope.popover.punchName = _.find(room.devices, {'id': $scope.popover.device.punch_vol}) ? _.find(room.devices, {'id': $scope.popover.device.punch_vol}).name : _.find(room.twoway_devices, {'id': $scope.popover.device.punch_vol}).name;
  };

  $scope.findPunchTwowayDevice = function(room) {
    var trfzw = _.find($scope.system.rooms[0].twoway_devices, {model: 'TRF-ZW1'}),
      item;
    if (trfzw) {
      item = _.find(trfzw.extra_info.punch, {roomId: room.id});
    }

    if (!(trfzw && item && item.from != -1)) {
      $scope.popover.punchName = _.find(room.devices, {'id': $scope.popover.twowayDevice.punch_vol}) ? _.find(room.devices, {'id': $scope.popover.twowayDevice.punch_vol}).name : _.find(room.twoway_devices, {'id': $scope.popover.twowayDevice.punch_vol}).name;
    } else {
      $scope.popover.punchName = _.find(room.devices, {'id': item.from}) ? _.find(room.devices, {'id': item.from}).name : _.find(room.twoway_devices, {'id': item.from}).name;
    }
  };

  $scope.containsMaster = function(room) {
    return _.contains(_.pluck(room.bstations, 'master'), 1);
  };

  $scope.changeValue = function(value) {
    return value/1000;
  };

  $scope.isEntDevice = Database.checkIfEntertainment;

  $scope.showFavModal = function (room, device) {
    angular.copy(room, $scope.selections.selectedRoom);
    angular.copy(device, $scope.selections.selectedDevice);

    favoriteOptionModal.$promise.then(favoriteOptionModal.show);
  };


  $scope.toggleFavorite = function(room, device) {
    var currentSystem = Systems.getCurrentSystem(),
            roomIndex = _.findIndex(currentSystem.rooms, { 'id': room.id }),
          deviceIndex = _.findIndex(currentSystem.rooms[roomIndex].devices, { 'id': device.id }),
        currentDevice = currentSystem.rooms[roomIndex].devices[deviceIndex];

    if (!_.isEqual(currentDevice, device)) {
      var jsonData =  { "favOption": device.fav_option, "favDelay": device.fav_delay, "favEnter": device.fav_enter  };
      console.log(jsonData);

      var url;
      if ($rootScope.virtualEditor) {
        url = '/api/dealer/device/favorite/' + room.id + '/' + device.id;
      } else {
        url = '/api/device/favorite/' + room.id + '/' + device.id;
      }

      // favorite option 변경 API 호출
      $http({
        method: 'POST',
        url: url,
        data : jsonData
      }).
      success(function(data) {
        if(data.result) {
          currentDevice.fav_option = device.fav_option;
          currentDevice.fav_delay = device.fav_delay;
          currentDevice.fav_enter = device.fav_enter;
          Systems.loadSystem(currentSystem);
          $state.transitionTo($state.current, null, { reload: true, inherit: true, notify: true  });
        }
        else {
          toastr.error(data.error);
        }
      })
      .error(function(error) {
        toastr.error('Please try again');
      });
    }

  };

  $scope.changeEventState = function(systemId, roomId, eventId, defaultState) {
    if (defaultState) {
      defaultState = 1;
    } else {
      defaultState = 0;
    }

    var jsonData = JSON.stringify({ id: eventId, default_state: defaultState });
    Rooms.changeEventState(systemId, roomId, jsonData)
      .then(function(data) {
        Systems.updateSystem(systemId).then(function () {
        });
      })
      .catch(toastr.error);
  };

  Rooms.disableScheEventEdit();
  Rooms.setScheEventData(null);

  $scope.eventEdit = function(room, event) {
    Rooms.enableScheEventEdit();

    Rooms.setScheEventData({
      room: angular.copy(room),
      event: angular.copy(event)
    });

    $state.go('menu.advanced.add-macros.sche-event-edit');
  };

  // Amazon Alexa
  var amazonEcho = _.find($scope.system.ext_service, {
    name: 'amazon_echo'
  });
  if (amazonEcho) {
    $scope.amazonEcho = angular.copy(amazonEcho);
  }

  // Google Assistant
  var gAssist = _.find($scope.system.ext_service, {
    name: 'g_assist'
  });
  if (gAssist) {
    $scope.gAssist = angular.copy(gAssist);
  }

  $scope.editExtServiceScene = function (service, roomId, sceneId) {
    var room = _.find($scope.system.rooms, {
        id: roomId
      });
    var scene;
    if('amazon_echo' == service) {
      scene = _.find(room.amazon_echo.scenes, {
        id: sceneId
      });
    } else {
      scene = _.find(room.g_assist.scenes, {
        id: sceneId
      });
    }

    if(scene.macro && typeof(scene.macro) === 'string') {
      scene.macro = JSON.parse(scene.macro);
    }
    if(scene.text && typeof(scene.text) === 'string') {
      scene.text = JSON.parse(scene.text);
    }
    scene.roomId = roomId;
    scene.service = service;

    $state.go('menu.advanced.options', {scene: JSON.stringify(scene)});
  };

  $scope.deleteExtServiceScene = function (service, roomId, sceneId) {
    $scope.delExtServiceSceneData = {
      service: service,
      roomId: roomId,
      sceneId: sceneId
    };

    var modal = $modal({
      scope: $scope,
      title: '',
      container: '#system-overview',
      template: 'views/modal-tpls/external-service/delete-scene.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.deleteExtServiceSceneItem = function () {
    ExternalService.delScene($scope.system.id, $scope.delExtServiceSceneData.roomId, $scope.delExtServiceSceneData.service, JSON.stringify({id: $scope.delExtServiceSceneData.sceneId})).then(function (data) {
      if (data.result) {
        var room = _.find($scope.system.rooms, {
            id: $scope.delExtServiceSceneData.roomId
        });
        var scene;
        if('amazon_echo' == $scope.delExtServiceSceneData.service) {
          scene = _.find(room.amazon_echo.scenes, {
              id: $scope.delExtServiceSceneData.sceneId
            });
          room.amazon_echo.scenes = _.without(room.amazon_echo.scenes, scene);
        } else {
          scene = _.find(room.g_assist.scenes, {
            id: $scope.delExtServiceSceneData.sceneId
          });
          room.g_assist.scenes = _.without(room.g_assist.scenes, scene);
        }
        
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  $scope.showAlexaInfo = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/external-service/alexa-info.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.amazonEmail = function () {
    $scope.selectedService = 'amazon_echo';
    var modal = $modal({
      scope: $scope,
      title: '',
      container: '#system-overview',
      template: 'views/modal-tpls/external-service/send-email.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.amazonSendEmail = function () {
    ExternalService.sendEmail($scope.system.id, 'amazon_echo').then(function (data) {
      if (data.result) {
        console.log(data);
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  $scope.deleteAmazon = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      container: '#system-overview',
      template: 'views/modal-tpls/external-service/remove-amazon-alexa.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.deleteAmazonService = function () {
    ExternalService.deleteService($scope.system.id, 'amazon_echo').then(function (data) {
      if (data.result) {
        location.reload();
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  _.each($scope.system.rooms, function (room) {
    _.each(room.twoway_devices, function (device) {
      if (device.ext_service && (device.ext_service & 1)) {
        device.amazonEchoSupport = true;
      }
      if (device.ext_service && (device.ext_service & 2)) {
        device.gAssistSupport = true;
      }
    });
  });

  $scope.showGAssistInfo = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/external-service/g-assist-info.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.showGAssistActionInfo = function (scene) {
    $scope.selectedScene = scene;
    if($scope.selectedScene.text && typeof($scope.selectedScene.text) === 'string') {
      $scope.selectedScene.text =  JSON.parse($scope.selectedScene.text);
    }
    
    var modal = $modal({
      scope: $scope,
      title: '',
      placement: 'top',
      template: 'views/modal-tpls/external-service/g-assist-action-info.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };


  $scope.gAssistEmail = function () {
    $scope.selectedService = 'g_assist';
    var modal = $modal({
      scope: $scope,
      title: '',
      container: '#system-overview',
      template: 'views/modal-tpls/external-service/send-email.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.gAssistSendEmail = function () {
    ExternalService.sendEmail($scope.system.id, 'g_assist').then(function (data) {
      if (data.result) {
        console.log(data);
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  $scope.deleteGAssist = function () {
    var modal = $modal({
      scope: $scope,
      title: '',
      container: '#system-overview',
      template: 'views/modal-tpls/external-service/remove-g-assist.html',
      show: false
    });

    modal.$promise.then(modal.show);
  };

  $scope.deleteGAssistService = function () {
    ExternalService.deleteService($scope.system.id, 'g_assist').then(function (data) {
      if (data.result) {
        location.reload();
      } else {
        toastr.error(data.error);
      }
    })
      .catch(toastr.error);
  };

  _.each($scope.system.rooms, function (room) {
    _.each(room.twoway_devices, function (device) {
      if (device.ext_service && (device.ext_service & 1)) {
        device.amazonEchoSupport = true;
      }
      if (device.ext_service && (device.ext_service & 2)) {
        device.gAssistSupport = true;
      }
    });
  });


  var masterBstation;
  _.each($scope.system.rooms, function (room) {
    if (!masterBstation) {
      masterBstation = _.where(room.bstations, {
        master: 1
      })[0];
    }
  });
  if (masterBstation) {
    $scope.masterBstationMac = masterBstation.mac;

    $scope.showAlexaVoiceService = function () {
      var modal = $modal({
        scope: $scope,
        title: '',
        placement: 'top',
        template: 'views/modal-tpls/external-service/alexa-voice-service.html',
        show: false
      });

      modal.$promise.then(modal.show);
    };

    $scope.viewAlexaVoiceService = function () {
      Systems.searchHardware(false, masterBstation.type, {mac: masterBstation.mac})
        .then(function (data) {
          var searchedDevice = _.find(data.searchedDevices, function (item) {
            return item.mac === masterBstation.mac;
          });

          if (searchedDevice && searchedDevice.ip) {
            window.open('http://' + searchedDevice.ip + ':51535');
          }
        })
        .catch(function (error) {
          toastr.error(error);
        });
    };

    $scope.getExtServiceVersion = function (service) {
      $scope.extServiceVersion = {};
      $scope.extServiceVersion.service = service;

      var modal = $modal({
        scope: $scope,
        title: '',
        container: '#system-overview',
        template: 'views/modal-tpls/external-service/about-this-device.html',
        show: false
      });

      modal.$promise.then(modal.show);

      Systems.getFirmwareVersion(masterBstation.type, $scope.extServiceVersion.service, $scope.system.id)
        .then(function (data) {
          $scope.extServiceVersion = data;
          $scope.extServiceVersion.service = service;
        })
        .catch(function (error) {
          //toastr.error(error);
        });
    };

  }
  
});

'use strict';
/**
 * @ngdoc controller
 * @name PunchToDeviceController
 * @description
 *  punch to device 선택 화면
 */
app.controller('PunchToDeviceController', function ($scope, $state, deviceId) {
  $scope.checkPunchTo = function(id) {
    var index = $scope.volumeControl.punchTo.indexOf(id);

    if (index < 0) {
      $scope.volumeControl.punchTo.push(id);
    } else {
      $scope.volumeControl.punchTo.splice(index, 1);
    }

    $scope.volumeControl.punchTo.sort(function(a, b) {
      return a - b;
    });
  };

  $scope.doNext = function() {
    $state.go('menu.programming.volume-control.punch-from-device');
  };
});

/**
 * @ngdoc controller
 * @name PunchFromDeviceController
 * @description
 *  punch from device 선택 화면
 */
app.controller('PunchFromDeviceController', function ($rootScope, $scope, $state, $http, Systems) {
  $scope.volumeControl.punchToNames = [];

  _.each($scope.volumeControl.punchTo, function(id) {
    var name = _.findWhere($scope.volumeControl.devices.destination, {
      id: id
    })['name'];

    $scope.volumeControl.punchToNames.push(name);
  });


  
  // punch to 에서 선택되었는지 검사
  function currentlySelected(punchFromId) {
    var selected = false;

    if ($scope.volumeControl.punchTo.indexOf(punchFromId) > -1) {
      selected = true;
    }

    return selected;
  }

  _.each($scope.volumeControl.devices.source, function(device) {
    if (device.id != -1) {
      device.currentlySelected = currentlySelected(device.id);
    }
  });


  $scope.setExtraInfo = function(device, punchFrom) {
    var extraInfo = device.extra_info,
      jsonData;

    if (extraInfo) {
      //extraInfo = JSON.parse(extraInfo);

      _.each(extraInfo.punch, function(room) {
        if (room.roomId == $scope.room.id) {
          room.from = punchFrom;
        }
      });
    } else {
      extraInfo = {
        punch: []
      };

      _.each($scope.system.rooms, function(room) {
        extraInfo.punch.push({
          roomId: room.id,
          from: (room.id == $scope.room.id) ? punchFrom : ''
        });
      });
    }

    jsonData = JSON.stringify(extraInfo);

    // set_extra_info API 호출
    $http({
      method: 'POST',
      url: '/api/twoway_device/set_extra_info/' + device.room_id + '/' + device.id,
      data : {
        data: jsonData
      }
    }).
    success(function(data) {

    }).
    error(function() {
      alert("Please try again");
    });
  };

  $scope.finish = function() {
    var punchFrom = $scope.volumeControl.punchFrom,
      punchToData,
      jsonData;

    $scope.volumeControl.punchToTrfzw = [];
    $scope.volumeControl.punchToTwowayDevices = [];
    $scope.volumeControl.punchToNormaDevices = [];

    _.each($scope.volumeControl.punchTo, function(id) {
      var device = _.findWhere($scope.volumeControl.devices.destination, {
        id: id
      });

      if (device.model === 'TRF-ZW1') {
        $scope.volumeControl.punchToTrfzw.push(id);
      }

      if (device.twoway_device && !device.other_room) {
        $scope.volumeControl.punchToTwowayDevices.push(id);
      } else {
        $scope.volumeControl.punchToNormaDevices.push(id);
      }
    });

    // punch 세팅 API 호출
    function punch(url, jsonData, transition) {
      $http({
        method: 'POST',
        url: url,
        data : jsonData
      }).
      success(function(data) {
        if(data.result) {

          if (transition) {
            var currentSystem = Systems.getCurrentSystem();
            Systems.updateSystem(currentSystem.id).then(function (data) {
              $state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true  });
            });
          }

        }
        else {
          alert(data.error);
        }
      }).
      error(function() {
        alert("Please try again");
      });
    }

    // step1: TRF-ZW
    //          => $scope.setExtraInfo 호출
    // step2: twoway device
    //          => 다른 room 에 있는 TRF-ZW1 일 경우 '/api/twoway_device/punch/' 는 호출하지 않음
    //          => 나머지 device 는 '/api/twoway_device/punch_multi/' 를 호출
    // step3: normal device

    // step1
    _.each($scope.volumeControl.punchToTrfzw, function(id) {
      var device = _.findWhere($scope.volumeControl.devices.destination, {
        id: id
      });

      // TRF-ZW1 인 경우 set_extra_info 를 수행함
      $scope.setExtraInfo(device, punchFrom.id);
    });

    // step1 이 끝나고 step2, step3 가 없으면 programming options 화면으로 이동
    if (!$scope.volumeControl.punchToTwowayDevices.length && !$scope.volumeControl.punchToNormaDevices.length) {
      var currentSystem = Systems.getCurrentSystem();
      Systems.updateSystem(currentSystem.id).then(function (data) {
        $state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true  });
      });
    }
    // step2 또는 step3 가 있는 경우
    else {
      // step2 가 있는 경우
      if ($scope.volumeControl.punchToTwowayDevices.length) {
        punchToData = $scope.volumeControl.punchToTwowayDevices;
        jsonData = JSON.stringify( { punchFrom : punchFrom.id, punchTo : punchToData });

        // punch 세팅 API 호출
        $http({
          method: 'POST',
          url: '/api/twoway_device/punch_multi/' + $scope.room.id,
          data : jsonData
        }).
        success(function(data) {
          if(data.result) {

            // step2 가 끝나고 step3 가 없으면 programming options 화면으로 이동
            if (!$scope.volumeControl.punchToNormaDevices.length) {

              var currentSystem = Systems.getCurrentSystem();
              Systems.updateSystem(currentSystem.id).then(function (data) {
                $state.transitionTo('menu.programming.options', null, { reload: true, inherit: true, notify: true  });
              });

            }
            // step3
            else {
              punchToData = $scope.volumeControl.punchToNormaDevices;
              jsonData = JSON.stringify( { punchFrom : punchFrom.id, punchTo : punchToData });

              var url;
              if ($rootScope.virtualEditor) {
                url = '/api/dealer/device/punch_multi/' + $scope.room.id;
              } else {
                url = '/api/device/punch_multi/' + $scope.room.id;
              }

              // punch 세팅 API 호출
              punch(url, jsonData, true);
            }

          }
          else {
            alert(data.error);
          }
        }).
        error(function() {
          alert("Please try again");
        });
      }
      // step2 가 없고 step3 만 있는 경우
      else {
        punchToData = $scope.volumeControl.punchToNormaDevices;
        jsonData = JSON.stringify( { punchFrom : punchFrom.id, punchTo : punchToData });

        var url;
        if ($rootScope.virtualEditor) {
          url = '/api/dealer/device/punch_multi/' + $scope.room.id;
        } else {
          url = '/api/device/punch_multi/' + $scope.room.id;
        }

        // punch 세팅 API 호출
        punch(url, jsonData, true);
      }
    }

  };
  
});